@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,18 +28,15 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/align.c,v 3.22 1998/11/05 22:56:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/align.c,v 4.19 2000/01/15 22:23:24 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_ALIGN_C_
+
+#include "tgifdefs.h"
+#include "cmdids.h"
 
-#ifndef _NO_EXTERN
 #include "align.e"
-#endif
 #include "auxtext.e"
 #include "button.e"
 #include "choice.e"
@@ -52,6 +49,7 @@
 #include "mark.e"
 #include "mainmenu.e"
 #include "menu.e"
+#include "menuinfo.e"
 #include "move.e"
 #include "msg.e"
 #include "obj.e"
@@ -60,6 +58,8 @@
 #include "rect.e"
 #include "select.e"
 #include "setup.e"
+#include "strtbl.e"
+#include "util.e"
 
 typedef struct DistrVRec {
    struct ObjRec *obj;
@@ -71,6 +71,8 @@ typedef struct DistrVRec {
 int	horiAlign=ALIGN_L;
 int	vertAlign=ALIGN_T;
 
+int	alignDirectType=INVALID;
+
 void DistrSelObjs()
 {
    register int i;
@@ -86,7 +88,8 @@ void DistrSelObjs()
    }
 
    if (numObjLocked != 0) {
-      MsgBox("Can not distribute with locked objects.", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_CANNOT_DISTRIBUTE_LOCKED_OBJS),
+            TOOL_NAME, INFO_MB);
       return;
    }
 
@@ -227,17 +230,8 @@ void DistrSelObjs()
          obj_ptr = sel_ptr->obj;
          if (obj_ptr->marked) {
             count++;
-            tmp_sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
-            if (tmp_sel_ptr == NULL) FailAllocMessage();
-            tmp_sel_ptr->obj = obj_ptr;
-            tmp_sel_ptr->prev = tmp_bot_sel;
-            tmp_sel_ptr->next = NULL;
-            if (tmp_bot_sel == NULL) {
-               tmp_top_sel = tmp_sel_ptr;
-            } else {
-               tmp_bot_sel->next = tmp_sel_ptr;
-            }
-            tmp_bot_sel = tmp_sel_ptr;
+            AddObjIntoSel(obj_ptr, tmp_bot_sel, NULL, &tmp_top_sel,
+                  &tmp_bot_sel);
          }
       }
       if (count != 0) {
@@ -329,12 +323,12 @@ void DistrSelObjs()
       free(dv_ptr);
 
       if (horiAlign != ALIGN_N) {
-         sprintf(gszMsgBox, "Vertices are %1d pixels apart horizontally.",
+         sprintf(gszMsgBox, TgLoadString(STID_VERTICES_HORI_APART),
                round(h_dist));
          Msg(gszMsgBox);
       }
       if (vertAlign != ALIGN_N) {
-         sprintf(gszMsgBox, "Vertices are %1d pixels apart vertically.",
+         sprintf(gszMsgBox, TgLoadString(STID_VERTICES_VERT_APART),
                round(v_dist));
          Msg(gszMsgBox);
       }
@@ -549,8 +543,8 @@ void DistrSelObjs()
             move_cmd->dx = dx;
             move_cmd->dy = 0;
             tmp_sel_ptr->obj = sel_ptr->next->obj;
-            PrepareToRecord(CMD_MOVE, NULL, NULL, 0);
-            RecordCmd(CMD_MOVE, sub_cmd, tmp_sel_ptr, tmp_sel_ptr, 1);
+            PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
+            RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
 
             MoveObj(sel_ptr->next->obj, dx, 0);
          }
@@ -586,8 +580,8 @@ void DistrSelObjs()
             move_cmd->dx = 0;
             move_cmd->dy = dy;
             tmp_sel_ptr->obj = sel_ptr->next->obj;
-            PrepareToRecord(CMD_MOVE, NULL, NULL, 0);
-            RecordCmd(CMD_MOVE, sub_cmd, tmp_sel_ptr, tmp_sel_ptr, 1);
+            PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
+            RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
 
             MoveObj(sel_ptr->next->obj, 0, dy);
          }
@@ -609,44 +603,43 @@ void DistrSelObjs()
 
       switch (horiAlign) {
       case ALIGN_L:
-         sprintf(gszMsgBox, "Left sides of objects are %1d pixels apart.",
-               round(h_dist));
+         sprintf(gszMsgBox, TgLoadString(STID_LEFT_SIDES_APART), round(h_dist));
          Msg(gszMsgBox);
          break;
       case ALIGN_C:
-         sprintf(gszMsgBox, "Center of objects are %1d pixels apart.",
+         sprintf(gszMsgBox, TgLoadString(STID_CENTERS_APART),
                round(h_dist/((double)2.0)));
          Msg(gszMsgBox);
          break;
       case ALIGN_R:
-         sprintf(gszMsgBox, "Right sides of objects are %1d pixels apart.",
+         sprintf(gszMsgBox, TgLoadString(STID_RIGHT_SIDES_APART),
                round(h_dist));
          Msg(gszMsgBox);
          break;
       case ALIGN_S:
-         sprintf(gszMsgBox, "Objects are spaced %1d pixels apart horizontally.",
+         sprintf(gszMsgBox, TgLoadString(STID_SPACED_APART_HORI),
                round(h_dist));
          Msg(gszMsgBox);
          break;
       }
       switch (vertAlign) {
       case ALIGN_T:
-         sprintf(gszMsgBox, "Top sides of objects are %1d pixels apart.",
+         sprintf(gszMsgBox, TgLoadString(STID_TOP_SIDES_APART),
                round(v_dist));
          Msg(gszMsgBox);
          break;
       case ALIGN_M:
-         sprintf(gszMsgBox, "Middle of objects are %1d pixels apart.",
+         sprintf(gszMsgBox, TgLoadString(STID_MIDDLES_APART),
                round(v_dist/((double)2.0)));
          Msg(gszMsgBox);
          break;
       case ALIGN_B:
-         sprintf(gszMsgBox, "Bottom sides of objects are %1d pixels apart.",
+         sprintf(gszMsgBox, TgLoadString(STID_BOTTOM_SIDES_APART),
                round(v_dist));
          Msg(gszMsgBox);
          break;
       case ALIGN_S:
-         sprintf(gszMsgBox, "Objects are spaced %1d pixels apart vertically.",
+         sprintf(gszMsgBox, TgLoadString(STID_SPACED_APART_VERT),
                round(v_dist));
          Msg(gszMsgBox);
          break;
@@ -663,49 +656,111 @@ void DistrSelObjs()
    justDupped = FALSE;
 }
 
+void DistributeDirect(alignment)
+   int alignment;
+{
+   int saved_h_align=horiAlign, saved_v_align=vertAlign;
+   int v_mode=(alignment%MAXALIGNS);
+   int h_mode=((alignment-v_mode)/MAXALIGNS);
+
+   horiAlign = h_mode;
+   vertAlign = v_mode;
+   DistrSelObjs();
+   horiAlign = saved_h_align;
+   vertAlign = saved_v_align;
+}
+
+static
+char *GetDistrDirectMenuStr(i)
+   int i;
+   /* returned string has been translated */
+{
+   char *psz=NULL, *msg=NULL;
+
+   if (curChoice == VERTEXMODE) {
+      int v_mode=(i%MAXALIGNS);
+      int h_mode=((i-v_mode)/MAXALIGNS);
+
+      if (v_mode == ALIGN_N) {
+         if (h_mode == ALIGN_N) {
+            msg = TgLoadCachedString(CSTID_PARANED_NONE);
+         } else {
+            msg = TgLoadString(STID_DISTR_VERTEX_HORI);
+         }
+      } else {
+         if (h_mode == ALIGN_N) {
+            msg = TgLoadString(STID_DISTR_VERTEX_VERT);
+         } else {
+            msg = TgLoadString(STID_DISTR_VERTEX_VERT_AND_HORI);
+         }
+      }
+   } else {
+      msg = DistrDirectLoadString(i);
+   }
+   psz = UtilStrDup(msg);
+   if (psz == NULL) FailAllocMessage();
+   return psz;
+}
+
+TgMenu *CreateDistributeDirectMenu(parent_menu, x, y, menu_info,
+      status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   int i=0, num_entries=(MAXALIGNS*MAXALIGNS);
+   TgMenu *menu=NULL;
+   TgMenuInfo stMenuInfo;
+   TgMenuItemInfo *item_info=NULL;
+
+   memcpy(&stMenuInfo, menu_info, sizeof(TgMenuInfo));
+   stMenuInfo.items = (TgMenuItemInfo*)malloc(
+         (num_entries+1)*sizeof(TgMenuItemInfo));
+   if (stMenuInfo.items == NULL) FailAllocMessage();
+   memset(stMenuInfo.items, 0,
+         (num_entries+1)*sizeof(TgMenuItemInfo));
+   for (item_info=stMenuInfo.items, i=0; i < num_entries; item_info++, i++) {
+      item_info->menu_str = (char*)(Pixmap*)(&distrDirectPixmap[i]);
+      item_info->shortcut_str = NULL;
+      item_info->status_str = GetDistrDirectMenuStr(i);
+      item_info->submenu_info = NULL;
+      item_info->cmdid = CMDID_DISTRIBUTEDIRECT;
+   }
+   stMenuInfo.items[num_entries].cmdid = INVALID;
+
+   /* the status_str has been translated in GetDistrDirectMenuStr() */
+   menu = TgCreateMenuFromMenuInfo(parent_menu, x, y, &stMenuInfo, TRUE);
+   for (item_info=stMenuInfo.items, i=0; i < num_entries; item_info++, i++) {
+      UtilFree(item_info->status_str);
+   }
+   memset(stMenuInfo.items, 0, (num_entries+1)*sizeof(TgMenuItemInfo));
+   free(stMenuInfo.items);
+   stMenuInfo.items = NULL;
+   if (menu != NULL) {
+      menu->track_menubar = TRUE;
+      TgAdjustMenuGeometry(menu, choiceImageW, choiceImageH, MAXALIGNS);
+   }
+   return menu;
+}
+
 static
 int AutoJustifiable(obj_ptr)
    struct ObjRec *obj_ptr;
 {
    struct TextRec *text_ptr;
-   int auto_justifiable=TRUE, rotate, just;
+   int auto_justifiable=TRUE, just;
 
    if (obj_ptr->type!=OBJ_TEXT || obj_ptr->detail.t->lines!=1) return FALSE;
    if (obj_ptr->ctm != NULL) return FALSE;
 
-   text_ptr=obj_ptr->detail.t;
-   rotate=text_ptr->rotate;
-   just=text_ptr->just;
+   text_ptr = obj_ptr->detail.t;
+   just = text_ptr->minilines.just;
 
-   switch (rotate) {
-   case ROTATE0:
-      if ((horiAlign==ALIGN_L && just==JUST_L) ||
-            (horiAlign==ALIGN_C && just==JUST_C) ||
-            (horiAlign==ALIGN_R && just==JUST_R)) {
-         auto_justifiable = FALSE;
-      }
-      break;
-   case ROTATE90:
-      if ((vertAlign==ALIGN_T && just==JUST_L) ||
-            (vertAlign==ALIGN_M && just==JUST_C) ||
-            (vertAlign==ALIGN_B && just==JUST_R)) {
-         auto_justifiable = FALSE;
-      }
-      break;
-   case ROTATE180:
-      if ((horiAlign==ALIGN_L && just==JUST_R) ||
-            (horiAlign==ALIGN_C && just==JUST_C) ||
-            (horiAlign==ALIGN_R && just==JUST_L)) {
-         auto_justifiable = FALSE;
-      }
-      break;
-   case ROTATE270:
-      if ((vertAlign==ALIGN_T && just==JUST_R) ||
-            (vertAlign==ALIGN_M && just==JUST_C) ||
-            (vertAlign==ALIGN_B && just==JUST_L)) {
-         auto_justifiable = FALSE;
-      }
-      break;
+   if ((horiAlign==ALIGN_L && just==JUST_L) ||
+         (horiAlign==ALIGN_C && just==JUST_C) ||
+         (horiAlign==ALIGN_R && just==JUST_R)) {
+      auto_justifiable = FALSE;
    }
    return auto_justifiable;
 }
@@ -715,51 +770,21 @@ void AutoJustify(obj_ptr)
    struct ObjRec *obj_ptr;
 {
    struct TextRec *text_ptr;
-   int new_ltx, new_lty, dx, dy, ltx, lty, rotate;
+   int new_ltx, new_lty, dx, dy, ltx, lty;
 
-   text_ptr=obj_ptr->detail.t;
-   rotate=text_ptr->rotate;
+   text_ptr = obj_ptr->detail.t;
 
    ltx = obj_ptr->obbox.ltx; lty = obj_ptr->obbox.lty;
-   switch (rotate) {
-   case ROTATE0:
-      switch (horiAlign) {
-      case ALIGN_L: text_ptr->just = JUST_L; break;
-      case ALIGN_C: text_ptr->just = JUST_C; break;
-      case ALIGN_R: text_ptr->just = JUST_R; break;
-      }
-      break;
-   case ROTATE90:
-      switch (vertAlign) {
-      case ALIGN_T: text_ptr->just = JUST_L; break;
-      case ALIGN_M: text_ptr->just = JUST_C; break;
-      case ALIGN_B: text_ptr->just = JUST_R; break;
-      }
-      break;
-   case ROTATE180:
-      switch (horiAlign) {
-      case ALIGN_L: text_ptr->just = JUST_R; break;
-      case ALIGN_C: text_ptr->just = JUST_C; break;
-      case ALIGN_R: text_ptr->just = JUST_L; break;
-      }
-      break;
-   case ROTATE270:
-      switch (vertAlign) {
-      case ALIGN_T: text_ptr->just = JUST_R; break;
-      case ALIGN_M: text_ptr->just = JUST_C; break;
-      case ALIGN_B: text_ptr->just = JUST_L; break;
-      }
-      break;
+   switch (horiAlign) {
+   case ALIGN_L: text_ptr->minilines.just = JUST_L; break;
+   case ALIGN_C: text_ptr->minilines.just = JUST_C; break;
+   case ALIGN_R: text_ptr->minilines.just = JUST_R; break;
    }
    UpdTextBBox(obj_ptr);
    dx = dy = 0;
    new_ltx = obj_ptr->obbox.ltx; new_lty = obj_ptr->obbox.lty;
-   switch (rotate) {
-   case ROTATE0: dx = ltx-new_ltx; break;
-   case ROTATE180: dx = ltx-new_ltx; break;
-   case ROTATE90: dy = lty-new_lty; break;
-   case ROTATE270: dy = lty-new_lty; break;
-   }
+   dx = ltx-new_ltx;
+
    if (text_ptr->cached_bitmap != None) {
       XFreePixmap(mainDisplay, text_ptr->cached_bitmap);
    }
@@ -789,7 +814,7 @@ void AlignSelObjs()
             !(topVSel->obj->type==OBJ_POLYGON && topVSel->v_index[0]==0))) &&
             horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
             vertAlign!=ALIGN_N && vertAlign!=ALIGN_S) {
-         if (MsgBox("Okay to move all vertices to one point? [ync](y)",
+         if (MsgBox(TgLoadString(STID_MOVE_ALL_VERTICES_YNC),
                TOOL_NAME, YNC_MB) != MB_ID_YES) {
             return;
          }
@@ -871,19 +896,20 @@ void AlignSelObjs()
                break;
             }
             RecordReplaceAnObj(obj_ptr);
-         }
-         else
+         } else {
             AbortPrepareCmd(CMD_REPLACE);
+         }
       }
       EndCompositeCmd();
    } else {
-      struct MoveSubCmdRec *move_cmd;
-      struct SubCmdRec *sub_cmd;
-      struct SelRec *tmp_sel_ptr;
+      struct MoveSubCmdRec *move_cmd=NULL;
+      struct SubCmdRec *sub_cmd=NULL;
+      struct SelRec *tmp_sel_ptr=NULL;
       struct ObjRec *locked_obj=NULL;
+      char *psz=NULL;
 
       if (numObjLocked > 1) {
-         MsgBox("Can not align objects.  Too many objects locked.", TOOL_NAME,
+         MsgBox(TgLoadString(STID_CANNOT_ALIGN_OBJS_TOO_MANY), TOOL_NAME,
                INFO_MB);
          return;
       } else if (numObjLocked == 1) {
@@ -964,8 +990,8 @@ void AlignSelObjs()
                move_cmd->dx = dx;
                move_cmd->dy = dy;
                tmp_sel_ptr->obj = obj_ptr;
-               PrepareToRecord(CMD_MOVE, NULL, NULL, 0);
-               RecordCmd(CMD_MOVE, sub_cmd, tmp_sel_ptr, tmp_sel_ptr, 1);
+               PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
+               RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
 
                MoveObj(obj_ptr, dx, dy);
             }
@@ -979,33 +1005,8 @@ void AlignSelObjs()
       free(sub_cmd);
       free(tmp_sel_ptr);
 
-      switch ((horiAlign<<ALIGN_SHIFT)|vertAlign) {
-      case ALIGN_NN: break;
-      case ALIGN_NT: Msg("top sides are aligned."); break;
-      case ALIGN_NM: Msg("vertical centers are aligned."); break;
-      case ALIGN_NB: Msg("bottom sides are aligned."); break;
-      case ALIGN_NS: break;
-      case ALIGN_LN: Msg("left sides are aligned."); break;
-      case ALIGN_LT: Msg("left and top sides are aligned."); break;
-      case ALIGN_LM: Msg("left side and middle are aligned."); break;
-      case ALIGN_LB: Msg("left and bottom sides are aligned."); break;
-      case ALIGN_LS: Msg("left sides are aligned."); break;
-      case ALIGN_CN: Msg("horizontal centers are aligned."); break;
-      case ALIGN_CT: Msg("centers and top sides are aligned."); break;
-      case ALIGN_CM: Msg("centers and middle are aligned."); break;
-      case ALIGN_CB: Msg("centers and bottom sides are aligned."); break;
-      case ALIGN_CS: Msg("horizontal centers are aligned."); break;
-      case ALIGN_RN: Msg("right sides are aligned."); break;
-      case ALIGN_RT: Msg("right and top sides are aligned."); break;
-      case ALIGN_RM: Msg("right and middle are aligned."); break;
-      case ALIGN_RB: Msg("right and bottom sides are aligned."); break;
-      case ALIGN_RS: Msg("right sides are aligned."); break;
-      case ALIGN_SN: break;
-      case ALIGN_ST: Msg("top sides are aligned."); break;
-      case ALIGN_SM: Msg("vertical centers are aligned."); break;
-      case ALIGN_SB: Msg("bottom sides are aligned."); break;
-      case ALIGN_SS: break;
-      }
+      psz = AlignedLoadString(horiAlign, vertAlign);
+      if (psz != NULL) Msg(psz);
    }
    ltx = selLtX; lty = selLtY; rbx = selRbX, rby = selRbY;
    UpdSelBBox();
@@ -1037,8 +1038,8 @@ void AlignSelToPage()
             !(topVSel->obj->type==OBJ_POLYGON && topVSel->v_index[0]==0))) &&
             horiAlign!=ALIGN_N && horiAlign!=ALIGN_S &&
             vertAlign!=ALIGN_N && vertAlign!=ALIGN_S) {
-         if (MsgBox("Okay to move all vertices to one point? [ync](y)",
-               TOOL_NAME, YNC_MB) != MB_ID_YES) {
+         if (MsgBox(TgLoadString(STID_MOVE_ALL_VERTICES_YNC), TOOL_NAME,
+               YNC_MB) != MB_ID_YES) {
             return;
          }
       }
@@ -1103,9 +1104,9 @@ void AlignSelToPage()
                break;
             }
             RecordReplaceAnObj(obj_ptr);
-         }
-         else
+         } else {
             AbortPrepareCmd(CMD_REPLACE);
+         }
       }
       EndCompositeCmd();
    } else {
@@ -1136,8 +1137,8 @@ void AlignSelToPage()
          move_cmd->dx = dx;
          move_cmd->dy = dy;
 
-         PrepareToRecord(CMD_MOVE, NULL, NULL, 0);
-         RecordCmd(CMD_MOVE, sub_cmd, topSel, botSel, numObjSelected);
+         PrepareToRecord(CMD_MOVE, topSel, botSel, numObjSelected);
+         RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
          for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next) {
             if (!sel_ptr->obj->locked) {
                MoveObj(sel_ptr->obj, dx, dy);
@@ -1160,11 +1161,12 @@ void AlignSelToPage()
 
 void AlignSelToGrid()
 {
-   register int x=0, y=0, i;
-   struct ObjRec *obj_ptr;
-   struct SelRec *sel_ptr;
-   struct VSelRec *vsel_ptr;
+   int x=0, y=0, i;
+   struct ObjRec *obj_ptr=NULL;
+   struct SelRec *sel_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL;
    int grid_x, grid_y, dx, dy, ltx, lty, rbx, rby, auto_justifiable;
+   int saved_draw_orig_x=0, saved_draw_orig_y=0;
 
    if (topSel == NULL && topVSel == NULL) return;
 
@@ -1187,6 +1189,12 @@ void AlignSelToGrid()
             break;
          }
          PrepareToReplaceAnObj(obj_ptr);
+
+         saved_draw_orig_x = drawOrigX;
+         saved_draw_orig_y = drawOrigY;
+
+         drawOrigX = drawOrigY = 0;
+
          for (i = 0; i < vsel_ptr->n; i++) {
             if (horiAlign!=ALIGN_N && horiAlign!=ALIGN_S) x = vsel_ptr->x[i];
             if (vertAlign!=ALIGN_N && vertAlign!=ALIGN_S) y = vsel_ptr->y[i];
@@ -1216,6 +1224,9 @@ void AlignSelToGrid()
                v[vsel_ptr->v_index[i]].y += dy;
             }
          }
+         drawOrigX = saved_draw_orig_x;
+         drawOrigY = saved_draw_orig_y;
+
          if (changed) {
             AdjObjSplineVs(obj_ptr);
             switch (obj_ptr->type) {
@@ -1276,11 +1287,16 @@ void AlignSelToGrid()
          case ALIGN_M: y = (obj_ptr->obbox.lty+obj_ptr->obbox.rby)/2; break;
          case ALIGN_B: y = obj_ptr->obbox.rby; break;
          }
+         saved_draw_orig_x = drawOrigX;
+         saved_draw_orig_y = drawOrigY;
+
+         drawOrigX = drawOrigY = 0;
+
          if (zoomedIn) {
             GridXY(ZOOMED_SIZE(x), ZOOMED_SIZE(y), &grid_x, &grid_y);
             if (horiAlign==ALIGN_N || horiAlign==ALIGN_S) x = ABS_SIZE(grid_x);
             if (vertAlign==ALIGN_N || vertAlign==ALIGN_S) y = ABS_SIZE(grid_y);
-
+  
             dx = ABS_SIZE(grid_x) - x;
             dy = ABS_SIZE(grid_y) - y;
          } else {
@@ -1291,6 +1307,9 @@ void AlignSelToGrid()
             dx = ABS_SIZE(grid_x) - x;
             dy = ABS_SIZE(grid_y) - y;
          }
+         drawOrigX = saved_draw_orig_x;
+         drawOrigY = saved_draw_orig_y;
+
          if (dx != 0 || dy != 0) {
             if (auto_justifiable) {
                MoveObj(obj_ptr, dx, dy);
@@ -1299,8 +1318,8 @@ void AlignSelToGrid()
                move_cmd->dx = dx;
                move_cmd->dy = dy;
                tmp_sel_ptr->obj = obj_ptr;
-               PrepareToRecord(CMD_MOVE, NULL, NULL, 0);
-               RecordCmd(CMD_MOVE, sub_cmd, tmp_sel_ptr, tmp_sel_ptr, 1);
+               PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
+               RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
 
                MoveObj(obj_ptr, dx, dy);
             }
@@ -1328,85 +1347,298 @@ void AlignSelToGrid()
 void HoriAlignSubMenu(index)
    int index;
 {
+   char *psz=NULL;
+
    horiAlign = index;
-   switch (horiAlign) {
-   case ALIGN_N: Msg("Horizontal alignment set to NONE."); break;
-   case ALIGN_L: Msg("Will align on the LEFT."); break;
-   case ALIGN_C: Msg("Will align objects at the horizontal CENTER."); break;
-   case ALIGN_R: Msg("Will align on the RIGHT."); break;
-   case ALIGN_S: Msg("Will equally SPACE objects horizontally."); break;
-   }
+   psz = HoriAlignLoadString(horiAlign);
+   if (psz != NULL) Msg(psz);
    ShowHoriAlign();
-   UpdateSubMenu(MENU_HORIALIGN);
+   UpdatePinnedMenu(MENU_HORIALIGN);
 }
 
-static char * alignHoriDescription[] =
+void RefreshHoriAlignMenu(menu)
+   TgMenu *menu;
 {
-   "Set horizontal alignment to none",
-   "Set horizontal alignment to left aligned",
-   "Set horizontal alignment to center aligned",
-   "Set horizontal alignment to right aligned",
-   "Set horizontal distribution to equal-spacing",
-   NULL
-};
+   int i, num_items=menu->num_items;
+   TgMenuItem *menuitems=menu->menuitems;
+
+   for (i=0; i < num_items; i++) {
+      TgMenuItem *menu_item=(&menuitems[i]);
+      TgMenuItem stMenuItem;
+
+      memset(&stMenuItem, 0, sizeof(TgMenuItem));
+      stMenuItem.state = TGBS_NORMAL;
+      stMenuItem.checked = (i == horiAlign);
+      TgSetMenuItemInfo(menu_item, TGMU_MASK_STATE|TGMU_MASK_CHECK,
+            &stMenuItem);
+   }
+}
+
+TgMenu *CreateHoriAlignMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      TgMenuItem *menu_item=NULL;
+      TgMenuItem stMenuItem;
+
+      menu->track_menubar = TRUE;
+      TgAdjustMenuGeometry(menu, choiceImageW, choiceImageH, MAXALIGNS);
+      menu_item = (&menu->menuitems[horiAlign]);
+
+      memset(&stMenuItem, 0, sizeof(TgMenuItem));
+      stMenuItem.checked = TRUE;
+      if (!TgSetMenuItemInfo(menu_item, TGMU_MASK_CHECK, &stMenuItem)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+      menu->refresh_proc = ((RefreshMenuFunc*)RefreshHoriAlignMenu);
+   }
+   return menu;
+}
 
 int HoriAlignMenu(X, Y, TrackMenubar)
    int X, Y, TrackMenubar;
 {
-   int index, * fore_colors, * valid, * init_rv;
+   int rc=INVALID;
+   TgMenu *menu=(horiAlignMenuInfo.create_proc)(NULL, X, Y, &horiAlignMenuInfo,
+         INVALID);
 
-   DefaultColorArrays(MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
-   free(valid);
-   init_rv[horiAlign] = TRUE;
    activeMenu = MENU_HORIALIGN;
-   index = PxMpMenuLoop(X, Y, choiceImageW, choiceImageH, MAXALIGNS, 1,
-         MAXALIGNS, fore_colors, alignHoriPixmap, init_rv,
-         alignHoriDescription, SINGLECOLOR, TrackMenubar);
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
 
-   if (index >= 0) HoriAlignSubMenu(index);
-   return index;
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
 }
 
 void VertAlignSubMenu(index)
    int index;
 {
+   char *psz=NULL;
+
    vertAlign = index;
-   switch (vertAlign) {
-   case ALIGN_N: Msg("Vertical alignment set to NONE."); break;
-   case ALIGN_T: Msg("Will align at the TOP."); break;
-   case ALIGN_M: Msg("Will align objects in the MIDDLE vertically."); break;
-   case ALIGN_B: Msg("Will align at the BOTTOM."); break;
-   case ALIGN_S: Msg("Will equally SPACE objects vertically."); break;
-   }
+   psz = VertAlignLoadString(vertAlign);
+   if (psz != NULL) Msg(psz);
    ShowVertAlign();
-   UpdateSubMenu(MENU_VERTALIGN);
+   UpdatePinnedMenu(MENU_VERTALIGN);
 }
 
-static char * alignVertDescription[] =
+void RefreshVertAlignMenu(menu)
+   TgMenu *menu;
 {
-   "Set vertical alignment to none",
-   "Set vertical alignment to top aligned",
-   "Set vertical alignment to middel aligned",
-   "Set vertical alignment to bottom aligned",
-   "Set vertical distribution to equal-spacing",
-   NULL
-};
+   int i, num_items=menu->num_items;
+   TgMenuItem *menuitems=menu->menuitems;
+
+   for (i=0; i < num_items; i++) {
+      TgMenuItem *menu_item=(&menuitems[i]);
+      TgMenuItem stMenuItem;
+
+      memset(&stMenuItem, 0, sizeof(TgMenuItem));
+      stMenuItem.state = TGBS_NORMAL;
+      stMenuItem.checked = (i == vertAlign);
+      TgSetMenuItemInfo(menu_item, TGMU_MASK_STATE|TGMU_MASK_CHECK,
+            &stMenuItem);
+   }
+}
+
+TgMenu *CreateVertAlignMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      TgMenuItem *menu_item=NULL;
+      TgMenuItem stMenuItem;
+
+      menu->track_menubar = TRUE;
+      TgAdjustMenuGeometry(menu, choiceImageW, choiceImageH, MAXALIGNS);
+      menu_item = (&menu->menuitems[vertAlign]);
+
+      memset(&stMenuItem, 0, sizeof(TgMenuItem));
+      stMenuItem.checked = TRUE;
+      if (!TgSetMenuItemInfo(menu_item, TGMU_MASK_CHECK, &stMenuItem)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+      menu->refresh_proc = ((RefreshMenuFunc*)RefreshVertAlignMenu);
+   }
+   return menu;
+}
 
 int VertAlignMenu(X, Y, TrackMenubar)
    int X, Y, TrackMenubar;
 {
-   int index, * fore_colors, * valid, * init_rv;
+   int rc=INVALID;
+   TgMenu *menu=(vertAlignMenuInfo.create_proc)(NULL, X, Y, &vertAlignMenuInfo,
+         INVALID);
 
-   DefaultColorArrays(MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
-   free(valid);
-   init_rv[vertAlign] = TRUE;
    activeMenu = MENU_VERTALIGN;
-   index = PxMpMenuLoop(X, Y, choiceImageW, choiceImageH, MAXALIGNS, 1,
-         MAXALIGNS, fore_colors, alignVertPixmap, init_rv,
-         alignVertDescription, SINGLECOLOR, TrackMenubar);
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
 
-   if (index >= 0) VertAlignSubMenu(index);
-   return index;
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
+}
+
+void AlignDirect(alignment)
+   int alignment;
+{
+   int saved_h_align=horiAlign, saved_v_align=vertAlign;
+   int v_align=(alignment%(MAXALIGNS-1));
+   int h_align=((alignment-v_align)/(MAXALIGNS-1));
+
+   horiAlign = h_align;
+   vertAlign = v_align;
+   switch (alignDirectType) {
+   case ALIGN_OBJS_DIRECT: AlignSelObjs(); break;
+   case ALIGN_TO_GRID_DIRECT: AlignSelToGrid(); break;
+   case ALIGN_TO_PAGE_DIRECT: AlignSelToPage(); break;
+   }
+   horiAlign = saved_h_align;
+   vertAlign = saved_v_align;
+}
+
+static
+char *GetAlignDirectMenuStr(i)
+   int i;
+   /* returned string has been translated */
+{
+   char *psz=NULL, *msg=NULL;
+
+   if (curChoice == VERTEXMODE) {
+      if (alignDirectType == ALIGN_TO_PAGE_DIRECT) {
+         if (i == 0) {
+            msg = TgLoadCachedString(CSTID_PARANED_NONE);
+         } else {
+            msg = AlignVerticesDirectToPageLoadString(i);
+         }
+      } else {
+         int v_align=(i%(MAXALIGNS-1));
+         int h_align=((i-v_align)/(MAXALIGNS-1));
+
+         if (v_align == ALIGN_N) {
+            if (h_align == ALIGN_N) {
+               msg = TgLoadCachedString(CSTID_PARANED_NONE);
+            } else {
+               msg = (alignDirectType==ALIGN_TO_GRID_DIRECT ?
+                     TgLoadString(STID_ALIGN_VERTICES_HORI_TO_GRID) :
+                     TgLoadString(STID_ALIGN_VERTICES_VERT));
+            }
+         } else {
+            if (h_align == ALIGN_N) {
+               msg = (alignDirectType==ALIGN_TO_GRID_DIRECT ?
+                     TgLoadString(STID_ALIGN_VERTICES_VERT_TO_GRID) :
+                     TgLoadString(STID_ALIGN_VERTICES_HORI));
+            } else {
+               msg = (alignDirectType==ALIGN_TO_GRID_DIRECT ?
+                     TgLoadString(STID_ALIGN_VERTICES_TO_GRID) :
+                     TgLoadString(STID_ALIGN_VERTICES));
+            }
+         }
+      }
+   } else {
+      if (alignDirectType == ALIGN_TO_PAGE_DIRECT) {
+         if (i == 0) {
+            msg = TgLoadCachedString(CSTID_PARANED_NONE);
+         } else {
+            msg = AlignDirectToPageLoadString(i);
+         }
+      } else {
+         if (i == 0) {
+            msg = TgLoadCachedString(CSTID_PARANED_NONE);
+         } else if (alignDirectType == ALIGN_TO_GRID_DIRECT) {
+            msg = AlignDirectToGridLoadString(i);
+         } else {
+            msg = AlignDirectLoadString(i);
+         }
+      }
+   }
+   psz = UtilStrDup(msg);
+   if (psz == NULL) FailAllocMessage();
+   return psz;
+}
+
+static
+TgMenu *CreateAlignDirectMenu(parent_menu, x, y, menu_info)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+{
+   int i=0, num_entries=((MAXALIGNS-1)*(MAXALIGNS-1));
+   TgMenu *menu=NULL;
+   TgMenuInfo stMenuInfo;
+   TgMenuItemInfo *item_info=NULL;
+
+   memcpy(&stMenuInfo, menu_info, sizeof(TgMenuInfo));
+   stMenuInfo.items = (TgMenuItemInfo*)malloc(
+         (num_entries+1)*sizeof(TgMenuItemInfo));
+   if (stMenuInfo.items == NULL) FailAllocMessage();
+   memset(stMenuInfo.items, 0, (num_entries+1)*sizeof(TgMenuItemInfo));
+   for (item_info=stMenuInfo.items, i=0; i < num_entries; item_info++, i++) {
+      item_info->menu_str = (char*)(Pixmap*)(&alignDirectPixmap[i]);
+      item_info->shortcut_str = NULL;
+      item_info->status_str = GetAlignDirectMenuStr(i);
+      item_info->submenu_info = NULL;
+      item_info->cmdid = CMDID_ALIGNDIRECT;
+   }
+   stMenuInfo.items[num_entries].cmdid = INVALID;
+
+   /* the status_str has been translated in GetAlignDirectMenuStr() */
+   menu = TgCreateMenuFromMenuInfo(parent_menu, x, y, &stMenuInfo, TRUE);
+   for (item_info=stMenuInfo.items, i=0; i < num_entries; item_info++, i++) {
+      UtilFree(item_info->status_str);
+   }
+   memset(stMenuInfo.items, 0, (num_entries+1)*sizeof(TgMenuItemInfo));
+   free(stMenuInfo.items);
+   stMenuInfo.items = NULL;
+   if (menu != NULL) {
+      menu->track_menubar = TRUE;
+      TgAdjustMenuGeometry(menu, choiceImageW, choiceImageH, (MAXALIGNS-1));
+   }
+   return menu;
+}
+
+TgMenu *CreateAlignObjsDirectMenu(parent_menu, x, y, menu_info,
+      status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   alignDirectType = ALIGN_OBJS_DIRECT;
+   return CreateAlignDirectMenu(parent_menu, x, y, menu_info);
+}
+
+TgMenu *CreateAlignToGridDirectMenu(parent_menu, x, y, menu_info,
+      status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   alignDirectType = ALIGN_TO_GRID_DIRECT;
+   return CreateAlignDirectMenu(parent_menu, x, y, menu_info);
+}
+
+TgMenu *CreateAlignToPageDirectMenu(parent_menu, x, y, menu_info,
+      status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   alignDirectType = ALIGN_TO_PAGE_DIRECT;
+   return CreateAlignDirectMenu(parent_menu, x, y, menu_info);
 }
 
 void CenterAnEndPoint()
@@ -1416,12 +1648,15 @@ void CenterAnEndPoint()
    int cx, cy, x1, y1, xn, yn, d1, dn, num_pts, index, x, y, tmp_x, tmp_y;
 
    if (curChoice != NOTHING) return;
-   if (topSel == NULL) return;
+   if (topSel == NULL) {
+      MsgBox(TgLoadCachedString(CSTID_NO_OBJ_SELECTED), TOOL_NAME, INFO_MB);
+      return;
+   }
    if (numObjSelected != 2 ||
          (!((topSel->obj->type == OBJ_POLY && botSel->obj->type != OBJ_POLY) ||
          (topSel->obj->type != OBJ_POLY && botSel->obj->type == OBJ_POLY)))) {
-      MsgBox("Please select one polyline and one non-polyline object.",
-            TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_SELECT_ONE_POLY_ONE_NON_POLY), TOOL_NAME,
+            INFO_MB);
       return;
    }
    if (topSel->obj->type == OBJ_POLY) {
@@ -1432,8 +1667,8 @@ void CenterAnEndPoint()
       other_obj = topSel->obj;
    }
    if (poly_obj->locked) {
-      MsgBox("Can not move a vertex for a locked polyline object.",
-            TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_CANNOT_MOVE_A_VERTEX_LOCKED), TOOL_NAME,
+            INFO_MB);
       return;
    }
    cx = (other_obj->obbox.ltx+other_obj->obbox.rbx)>>1;
