@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.21 2000/02/26 22:55:27 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.25 2000/05/20 19:21:13 william Exp $
  */
 
 #define _INCLUDE_FROM_ARC_C_
@@ -870,9 +870,11 @@ void MakeArcRotatedVs(obj_ptr, oval_vs, angle1, angle2,
       }
    }
    *rotated_n = 0;
-   *rotated_vlist = (XPoint*)malloc((num_vs+3)*sizeof(XPoint));
-   pv = (IntPoint*)malloc((num_vs+3)*sizeof(IntPoint));
+   *rotated_vlist = (XPoint*)malloc((num_vs+5)*sizeof(XPoint));
+   pv = (IntPoint*)malloc((num_vs+5)*sizeof(IntPoint));
    if (*rotated_vlist == NULL || pv == NULL) FailAllocMessage();
+   memset(*rotated_vlist, 0, (num_vs+5)*sizeof(XPoint));
+   memset(pv, 0, (num_vs+5)*sizeof(IntPoint));
 
    cur_index = 0;
    seen_angle1 = FALSE;
@@ -880,7 +882,7 @@ void MakeArcRotatedVs(obj_ptr, oval_vs, angle1, angle2,
          angle+=angle90, v_index+=6) {
       /* the quadrant being considered is angle-angle90 to angle */
       double angle_in_quadrant; /* 0 < angle_in_quadrant <= 90.0 degrees */
-      int count;
+      int count=(-1);
       double sin_val, cos_val;
 
       if (v_index >= 24) v_index = 0;
@@ -997,6 +999,7 @@ void MakeArcRotatedVs(obj_ptr, oval_vs, angle1, angle2,
       sn = 2;
       sv = (XPoint*)malloc((num_vs+1)*sizeof(XPoint));
       if (sv == NULL) FailAllocMessage();
+      memset(sv, 0, (num_vs+1)*sizeof(XPoint));
       sv[0].x = (short)(OFFSET_X(first_pt.x));
       sv[0].y = (short)(OFFSET_Y(first_pt.y));
       sv[1].x = (short)(OFFSET_X(pv[0].x));
@@ -1011,6 +1014,7 @@ void MakeArcRotatedVs(obj_ptr, oval_vs, angle1, angle2,
       XPoint *new_sv=(XPoint*)malloc((sn+3)*sizeof(XPoint));
 
       if (new_sv == NULL) FailAllocMessage();
+      memset(new_sv, 0, (sn+3)*sizeof(XPoint));
       for (i=0; i < sn; i++) {
          new_sv[i].x = sv[i].x;
          new_sv[i].y = sv[i].y;
@@ -1563,21 +1567,9 @@ void ContinueArc(OrigX, OrigY)
    char buf[80], r_buf[80], x_buf[80], y_buf[80];
    char *cl_or_r=(curChoice==DRAWEDGEARC ? "cl" : "r");
    struct ObjRec *obj_ptr;
-   XGCValues values;
    XEvent input, ev;
 
-   values.foreground = xorColorPixels[colorIndex];
-   values.function = GXxor;
-   values.fill_style = FillSolid;
-#ifdef NO_THIN_LINE
-   values.line_width = 1;
-#else
-   values.line_width = 0;
-#endif
-   values.line_style = LineSolid;
-   XChangeGC(mainDisplay, drawGC,
-         GCForeground | GCFunction | GCFillStyle | GCLineWidth | GCLineStyle,
-         &values);
+   SetXorDrawGC(xorColorPixels[colorIndex]);
 
    grid_x = saved_x = OrigX;
    grid_y = saved_y = OrigY; 
@@ -1607,6 +1599,7 @@ void ContinueArc(OrigX, OrigY)
 
       if (input.type == Expose || input.type == VisibilityNotify) {
          ExposeEventHandler(&input, TRUE);
+         SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if (input.type == ButtonPress) {
          if (input.xbutton.button != Button1 || drawing_arc) {
             XUngrabPointer(mainDisplay, CurrentTime);
