@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.21 2000/01/15 22:23:26 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.27 2000/04/15 19:15:40 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -129,6 +129,16 @@ void UnlinkAttr(AttrPtr)
    }
 }
 
+void UpdAttrOwner(first_attr, owner_obj)
+   struct AttrRec *first_attr;
+   struct ObjRec *owner_obj;
+{
+   while (first_attr != NULL) {
+      first_attr->owner = owner_obj;
+      first_attr = first_attr->next;
+   }
+}
+
 static
 char *FindEqual(s)
    char *s;
@@ -1366,10 +1376,11 @@ int ChangeAttrJust(ObjPtr, AttrPtr)
    return FALSE;
 }
 
-TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings, pixels)
+TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings,
+      status_strings, pixels)
    TgMenu *parent_menu;
    int x, y, num_items, *pixels;
-   char **menu_strings;
+   char **menu_strings, **status_strings;
 {
    int i=0;
    TgMenu *menu=NULL;
@@ -1385,6 +1396,10 @@ TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings, pixels)
    for (item_info=stMenuInfo.items, i=0; i < num_items; item_info++, i++) {
       item_info->menu_str = UtilStrDup(menu_strings[i]);
       if (item_info->menu_str == NULL) FailAllocMessage();
+      if (status_strings != NULL && status_strings[i] != NULL) {
+         item_info->status_str = UtilStrDup(status_strings[i]);
+         if (item_info->status_str == NULL) FailAllocMessage();
+      }
       item_info->submenu_info = NULL;
       item_info->cmdid = INVALID;
    }
@@ -1397,6 +1412,9 @@ TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings, pixels)
     * Since these are real attribute names and values, no translation is needed.
     */
    menu = TgCreateMenuFromMenuInfo(parent_menu, x, y, &stMenuInfo, TRUE);
+   for (item_info=stMenuInfo.items, i=0; i < num_items; item_info++, i++) {
+      UtilFree(item_info->status_str);
+   }
    memset(stMenuInfo.items, 0, (num_items+1)*sizeof(TgMenuItemInfo));
    free(stMenuInfo.items);
    stMenuInfo.items = NULL;
@@ -1408,9 +1426,12 @@ TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings, pixels)
       for (i=0; i < num_items; i++) {
          TgMenuItem *menu_item=(&menu->menuitems[i]);
 
-         stMenuItem.multicolor_pixel = pixels[i];
-         if (!TgSetMenuItemInfo(menu_item, TGMU_MASK_MULTICOLOR, &stMenuItem)) {
-            return TgDestroyMenu(menu, TRUE);
+         if (pixels != NULL) {
+            stMenuItem.multicolor_pixel = pixels[i];
+            if (!TgSetMenuItemInfo(menu_item, TGMU_MASK_MULTICOLOR,
+                  &stMenuItem)) {
+               return TgDestroyMenu(menu, TRUE);
+            }
          }
          menu_item->menu_str_allocated = TRUE;
       }
@@ -1422,38 +1443,147 @@ void FreeEditAttrInfo(pEditAttrInfo)
    EditAttrInfo *pEditAttrInfo;
    /* pEditAttrInfo will also be freed before returning */
 {
-   int i=0, num_attrs=0, *fore_colors=NULL;
+   int i=0, num_attrs=0, *fore_colors=NULL, *attr_indices=NULL;
    char **attr_names=NULL, **attr_values=NULL, **attr_strings=NULL;
+   char **status_strings=NULL;
 
    if (pEditAttrInfo == NULL) return;
 
    num_attrs = pEditAttrInfo->num_attrs;
    fore_colors = pEditAttrInfo->fore_colors;
+   attr_indices = pEditAttrInfo->attr_indices;
    attr_names = pEditAttrInfo->attr_names;
    attr_values = pEditAttrInfo->attr_values;
    attr_strings = pEditAttrInfo->attr_strings;
+   status_strings = pEditAttrInfo->status_strings;
 
-   free(pEditAttrInfo->fore_colors);
+   if (pEditAttrInfo->fore_colors != NULL) free(pEditAttrInfo->fore_colors);
+   if (pEditAttrInfo->attr_indices != NULL) free(pEditAttrInfo->attr_indices);
 
    for (i=0; i < num_attrs; i++) {
-      UtilFree(attr_names[i]);
-      UtilFree(attr_values[i]);
-      UtilFree(attr_strings[i]);
+      if (attr_names != NULL) UtilFree(attr_names[i]);
+      if (attr_values != NULL) UtilFree(attr_values[i]);
+      if (attr_strings != NULL) UtilFree(attr_strings[i]);
+      if (status_strings != NULL) UtilFree(status_strings[i]);
    }
-   free(pEditAttrInfo->attr_names);
-   free(pEditAttrInfo->attr_values);
-   free(pEditAttrInfo->attr_strings);
+   if (attr_names != NULL) free(attr_names);
+   if (attr_values != NULL) free(attr_values);
+   if (attr_strings != NULL) free(attr_strings);
+   if (status_strings != NULL) free(status_strings);
 
    free(pEditAttrInfo);
 }
 
+int HasEditAttrsInContextMenu(obj_ptr, pp_name_attr)
+   struct ObjRec *obj_ptr;
+   struct AttrRec **pp_name_attr;
+{
+   struct AttrRec *attr_ptr=NULL;
+
+   if (pp_name_attr != NULL) *pp_name_attr = NULL;
+   for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev) {
+      if (strcmp(attr_ptr->attr_name.s, "edit_attrs_in_context_menu=") == 0) {
+         if (pp_name_attr != NULL) *pp_name_attr = attr_ptr;
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
+static
+void FreeRestrictedAttrNames(ppsz_restricted, num_restricted)
+   char **ppsz_restricted;
+   int num_restricted;
+{
+   if (ppsz_restricted != NULL) {
+      int i=0;
+
+      for (i=0; i < num_restricted; i++) {
+         if (ppsz_restricted[i] != NULL) {
+            UtilFree(ppsz_restricted[i]);
+         }
+      }
+      free(ppsz_restricted);
+   }
+}
+
+static
+void GetRestrictedAttrNames(restricted_attr_obj, pppsz_restricted,
+      pn_num_restricted)
+   struct ObjRec *restricted_attr_obj;
+   char ***pppsz_restricted;
+   int *pn_num_restricted;
+{
+   MiniLinesInfo *minilines=(&restricted_attr_obj->detail.t->minilines);
+   MiniLineInfo *miniline=minilines->first->next;
+   char **ppsz_restricted=NULL;
+   int num_restricted=0, index=0;
+
+   if (pppsz_restricted != NULL) (*pppsz_restricted) = NULL;
+   if (pn_num_restricted != NULL) (*pn_num_restricted) = 0;
+   if (miniline == NULL) return;
+
+   for ( ; miniline != NULL; miniline=miniline->next, num_restricted++) {
+   }
+   ppsz_restricted = (char**)malloc(num_restricted*sizeof(char*));
+   if (ppsz_restricted == NULL) FailAllocMessage();
+   memset(ppsz_restricted, 0, num_restricted*sizeof(char*));
+
+   miniline = minilines->first->next;
+   for ( ; miniline != NULL; miniline=miniline->next) {
+      int need_to_free_tmp_buf=FALSE;
+      char *tmp_buf=ConvertMiniLineToString(miniline, &need_to_free_tmp_buf);
+
+      if (tmp_buf != NULL) {
+         UtilTrimBlanks(tmp_buf);
+         if (*tmp_buf == '\0') {
+            num_restricted--;
+         } else {
+            int len=strlen(tmp_buf)+2; /* extra char for '=' */
+
+            ppsz_restricted[index] = (char*)malloc((len+1)*sizeof(char));
+            if (ppsz_restricted[index] == NULL) FailAllocMessage();
+            sprintf(ppsz_restricted[index], "%s=", tmp_buf);
+            index++;
+         }
+         if (need_to_free_tmp_buf) UtilFree(tmp_buf);
+      } else {
+         num_restricted--;
+      }
+      if (num_restricted <= 0) break;
+   }
+   if (num_restricted <= 0) {
+      free(pppsz_restricted);
+   } else {
+      if (pppsz_restricted != NULL) (*pppsz_restricted) = ppsz_restricted;
+      if (pn_num_restricted != NULL) (*pn_num_restricted) = num_restricted;
+   }
+}
+
+static
+int IsRestrictedAttr(attr_name, ppsz_restricted, num_restricted)
+   char *attr_name, **ppsz_restricted;
+   int num_restricted;
+{
+   int i=0;
+
+   for (i=0; i < num_restricted; i++) {
+      if (strcmp(attr_name, ppsz_restricted[i]) == 0) {
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
 EditAttrInfo *CreateEditAttrInfo(obj_ptr)
    struct ObjRec *obj_ptr;
 {
-   int i=0, num_attrs=0, *fore_colors=NULL, *pixel_ptr=NULL;
-   struct AttrRec *attr_ptr=NULL, *attr_ptr1=NULL;
+   int i=0, num_attrs=0, *fore_colors=NULL, *pixel_ptr=NULL, restricted=FALSE;
+   int num_restricted=0, num_created=0, *attr_indices=NULL;
+   struct AttrRec *attr_ptr=NULL, *attr_ptr1=NULL, *restricted_attr=NULL;
    EditAttrInfo *pEditAttrInfo=NULL;
    char **attr_names=NULL, **attr_values=NULL, **attr_strings=NULL;
+   char **status_strings=NULL, **ppsz_restricted=NULL;
 
    pEditAttrInfo = (EditAttrInfo*)malloc(sizeof(EditAttrInfo));
    if (pEditAttrInfo == NULL) FailAllocMessage();
@@ -1466,12 +1596,24 @@ EditAttrInfo *CreateEditAttrInfo(obj_ptr)
       free(pEditAttrInfo);
       return NULL;
    }
+   restricted = HasEditAttrsInContextMenu(obj_ptr, &restricted_attr);
+   if (restricted) {
+      GetRestrictedAttrNames(restricted_attr->obj, &ppsz_restricted,
+            &num_restricted);
+      if (ppsz_restricted == NULL || num_restricted <= 0) {
+         free(pEditAttrInfo);
+         return NULL;
+      }
+   }
    attr_names = (char**)malloc(num_attrs*sizeof(char*));
    attr_values = (char**)malloc(num_attrs*sizeof(char*));
    attr_strings = (char**)malloc(num_attrs*sizeof(char*));
+   status_strings = (char**)malloc(num_attrs*sizeof(char*));
    fore_colors = pixel_ptr = (int*)malloc(num_attrs*sizeof(int));
+   attr_indices = (int*)malloc(num_attrs*sizeof(int));
    if (attr_names == NULL || attr_values == NULL || attr_strings == NULL ||
-         fore_colors == NULL) {
+         status_strings == NULL || fore_colors == NULL ||
+         attr_indices == NULL) {
       FailAllocMessage();
    }
    attr_ptr1 = attr_ptr;
@@ -1483,22 +1625,40 @@ EditAttrInfo *CreateEditAttrInfo(obj_ptr)
       tmp_buf = ConvertAttrNameFirstMiniLineToString(attr_ptr1,
             &need_to_free_tmp_buf);
 
-      attr_names[i] = UtilStrDup(attr_ptr1->attr_name.s);
-      attr_values[i] = UtilStrDup(attr_ptr1->attr_value.s);
-      attr_strings[i] = UtilStrDup(tmp_buf);
-      if (attr_names[i] == NULL || attr_values[i] == NULL ||
-            attr_strings[i] == NULL) {
+      if (restricted) {
+         if (!IsRestrictedAttr(attr_ptr1->attr_name.s, ppsz_restricted,
+               num_restricted)) {
+            if (need_to_free_tmp_buf) UtilFree(tmp_buf);
+            continue;
+         }
+      }
+      attr_names[num_created] = UtilStrDup(attr_ptr1->attr_name.s);
+      attr_values[num_created] = UtilStrDup(attr_ptr1->attr_value.s);
+      attr_strings[num_created] = UtilStrDup(tmp_buf);
+      sprintf(gszMsgBox, TgLoadCachedString(CSTID_EDIT_ATTR_IN_EDITOR),
+            tmp_buf);
+      status_strings[num_created] = UtilStrDup(gszMsgBox);
+      if (attr_names[num_created] == NULL || attr_values[num_created] == NULL ||
+            attr_strings[num_created] == NULL ||
+            status_strings[num_created] == NULL) {
          FailAllocMessage();
       }
-      *pixel_ptr++ = colorPixels[minilines->first->first_block->seg->color];
+      pixel_ptr[num_created] =
+            colorPixels[minilines->first->first_block->seg->color];
+      attr_indices[num_created] = i;
 
       if (need_to_free_tmp_buf) UtilFree(tmp_buf);
+      num_created++;
    }
-   pEditAttrInfo->num_attrs = num_attrs;
+   pEditAttrInfo->num_attrs = num_created;
    pEditAttrInfo->fore_colors = fore_colors;
+   pEditAttrInfo->attr_indices = attr_indices;
    pEditAttrInfo->attr_names = attr_names;
    pEditAttrInfo->attr_values = attr_values;
    pEditAttrInfo->attr_strings = attr_strings;
+   pEditAttrInfo->status_strings = status_strings;
+
+   FreeRestrictedAttrNames(ppsz_restricted, num_restricted);
 
    return pEditAttrInfo;
 }
@@ -1552,7 +1712,7 @@ void MoveAttr()
    button = CornerLoop(&x, &y);
    activeMenu = INVALID;
    if (button == Button1 || button == Button2) {
-      TgMenu *menu=CreateAttrMenu(NULL, x, y, num_attrs, attr_strings,
+      TgMenu *menu=CreateAttrMenu(NULL, x, y, num_attrs, attr_strings, NULL,
             fore_colors);
 
       if (menu != NULL) {
