@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.28 2000/01/15 22:23:27 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.30 2000/05/04 13:53:07 william Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -1384,6 +1384,11 @@ void DumpTextObj(FP, ObjPtr)
          (pen == NONEPAT || (trans_pat && pen == BACKPAT))) {
       return;
    }
+   if (ObjPtr->ctm != NULL && (ObjPtr->obbox.ltx == ObjPtr->obbox.rbx ||
+         ObjPtr->obbox.lty == ObjPtr->obbox.rby)) {
+      /* zero width or height text */
+      return;
+   }
    fprintf(FP, "%% TEXT\n");
    fprintf(FP, "%s\n", gPsCmd[PS_NEWPATH]);
 
