@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/box.c,v 4.8 2000/01/15 22:23:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/box.c,v 4.12 2000/05/01 20:18:31 william Exp $
  */
 
 #define _INCLUDE_FROM_BOX_C_
@@ -42,6 +42,7 @@
 #include "color.e"
 #include "cursor.e"
 #include "cutpaste.e"
+#include "drawing.e"
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
@@ -388,27 +389,12 @@ void ContinueBox(OrigX, OrigY)
    int  OrigX, OrigY;
 {
    int end_x, end_y, grid_x, grid_y, saved_x, saved_y;
-   int done=FALSE, abort=FALSE, xor_pixel;
+   int done=FALSE, abort=FALSE;
    char buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80];
-   XGCValues values;
    XEvent input, ev;
    XMotionEvent *motion_ev;
 
-   xor_pixel = xorColorPixels[colorIndex];
-
-   values.foreground = xor_pixel;
-   values.function = GXxor;
-   values.fill_style = FillSolid;
-#ifdef NO_THIN_LINE
-   values.line_width = 1;
-#else
-   values.line_width = 0;
-#endif
-   values.line_style = LineSolid;
-
-   XChangeGC(mainDisplay, drawGC,
-         GCForeground | GCFunction | GCFillStyle | GCLineWidth | GCLineStyle,
-         &values);
+   SetXorDrawGC(xorColorPixels[colorIndex]);
 
    saved_x = grid_x = OrigX;
    saved_y = grid_y = OrigY; 
@@ -429,6 +415,7 @@ void ContinueBox(OrigX, OrigY)
 
       if (input.type == Expose || input.type == VisibilityNotify) {
          ExposeEventHandler(&input, TRUE);
+         SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if (input.type == ButtonRelease) {
          XUngrabPointer(mainDisplay, CurrentTime);
          MyBox(drawWindow, drawGC, OrigX, OrigY, saved_x, saved_y);
