@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.31 2000/01/15 22:23:30 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.33 2000/04/17 20:56:57 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOOSE_C_
@@ -51,6 +51,7 @@
 #include "raster.e"
 #include "rect.e"
 #include "remote.e"
+#include "ruler.e"
 #include "scroll.e"
 #include "setup.e"
 #include "strtbl.e"
@@ -2115,6 +2116,8 @@ int Names(win_name, pn_selected_index, selected_str, str_sz, p_void)
 
    if (!CreateNamesWindows(win_name)) return INVALID;
 
+   EndMeasureTooltip(FALSE);
+
    pni->userdata = p_void;
 
    if (pni->edit_style == NAMES_SELECT_FILE) {
