@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.41 2000/01/18 22:55:39 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.46 2000/05/20 22:37:57 william Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -378,7 +378,12 @@
 #define CMDID_FREEHANDMODEINSLIDESHOW      425
 #define CMDID_RECENTLYUSEDFILES            426
 #define CMDID_MOVEEDITTEXTBOX              427
+#define CMDID_GETPROPERTY                  428
+#define CMDID_REPLACEGRAPHIC               429
+#define CMDID_TOGGLESHOWMEASUREMENTINTT    430
+#define CMDID_IMPORTJPEGFILE               431
+#define CMDID_TOGGLEAUTOEPSPREVIEWBITMAP   432
 
-#define MAXCMDIDS 428
+#define MAXCMDIDS 433
 
 #endif /*_CMDIDS_H_*/
