@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/color.c,v 4.35 2000/01/15 22:23:32 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/color.c,v 4.37 2000/05/15 22:30:33 william Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -586,15 +586,17 @@ void InitColor()
       for (i=0; i < maxColors; i++) {
          sprintf(buf, "Color%1d", i);
          if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, buf)) != NULL) {
-            strcpy(colorMenuItems[i], c_ptr);
+            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN, c_ptr);
          } else if (i < MAXCOLORS) {
-            strcpy(colorMenuItems[i], defaultColorMenuItems[i]);
+            UtilStrCpyN(colorMenuItems[i], COLORSTRLEN,
+                  defaultColorMenuItems[i]);
          } else {
             fprintf(stderr, TgLoadString(STID_CANNOT_GET_X_DEFAULT),
                   TOOL_NAME, buf);
             fprintf(stderr, "\n");
             exit(-1);
          }
+         UtilTrimBlanks(colorMenuItems[i]);
          colorLayerOn[i] = TRUE;
       }
    }
@@ -603,7 +605,7 @@ void InitColor()
       static int initVryLtPixel=FALSE;
 
       myBlackPixel = myWhitePixel = myLtGryPixel = myDkGryPixel = INVALID;
-      myVryLtPixel = INVALID;
+      myVryLtPixel = myYellowPixel = INVALID;
       memset(&color, 0, sizeof(XColor));
       if (ParseAndAllocColorByName("black", &color, NULL, NULL, NULL)) {
          myBlackPixel = color.pixel;
@@ -611,6 +613,9 @@ void InitColor()
       if (ParseAndAllocColorByName("white", &color, NULL, NULL, NULL)) {
          myWhitePixel = color.pixel;
       }
+      if (ParseAndAllocColorByName("#ffff00", &color, NULL, NULL, NULL)) {
+         myYellowPixel = color.pixel;
+      }
       if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"LightGrayColor")) != NULL) {
          if (ParseAndAllocColorByName(c_ptr, &color, NULL, NULL, NULL)) {
             myLtGryPixel = color.pixel;
@@ -649,7 +654,8 @@ void InitColor()
       }
       if (myBlackPixel == INVALID || myWhitePixel == INVALID ||
             myLtGryPixel == INVALID || myDkGryPixel == INVALID ||
-            (initVryLtPixel && myVryLtPixel == INVALID)) {
+            (initVryLtPixel && myVryLtPixel == INVALID) ||
+            myYellowPixel == INVALID) {
          fprintf(stderr, "%s\n", TgLoadString(STID_FAIL_TO_ALLOC_3D_COLORS));
          threeDLook = FALSE;
       }
