@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.38 2000/01/15 22:25:18 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.40 2000/05/10 17:29:53 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -463,8 +463,9 @@
 #define PDF_FILE 8
 #define TIFFEPSI_FILE 9
 #define PNG_FILE 10
+#define JPEG_FILE 11
 
-#define MAXDEFWHERETOPRINT 11
+#define MAXDEFWHERETOPRINT 12
 
 /* measurement */
 
@@ -966,5 +967,6 @@
 #define PROP_MASK_TEXT_STYLE	0x00008000 /* ChangeEditTextProperty() only */
 #define PROP_MASK_VSPACE	0x00010000
 #define PROP_MASK_UNDERLINE_ON	0x00020000
+#define PROP_MASK_WIDTH_INDEX	0x00100000 /* GetProperty() only */
 
 #endif /*_TGIF_CONST_H_*/
