@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,23 +28,18 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/cursor.c,v 3.12 1998/11/05 22:56:08 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cursor.c,v 4.8 2000/01/15 22:23:34 william Exp $
  */
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/cursorfont.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_CURSOR_C_
+
+#include "tgifdefs.h"
 
 #include "choice.e"
 #include "color.e"
-#ifndef _NO_EXTERN
 #include "cursor.e"
-#endif
 #include "setup.e"
+#include "strtbl.e"
 #include "xbitmap.e"
 
 #include "xbm/null.xbm"
@@ -65,6 +60,7 @@ Cursor handCursor=(Cursor)0;
 Cursor defaultCursor=(Cursor)0;
 Cursor watchCursor=(Cursor)0;
 Cursor drawCursor=(Cursor)0;
+Cursor freehandCursor=(Cursor)0;
 Cursor vertexCursor=(Cursor)0;
 Cursor rotateCursor=(Cursor)0;
 Cursor rotatingCursor=(Cursor)0;
@@ -93,6 +89,7 @@ static Pixmap dripPixmap=None;
 static Pixmap dripMaskPixmap=None;
 
 static char *cursorName[] = {
+   /* do not translate -- program constants */
    "X_cursor",
    "arrow",
    "based_arrow_down",
@@ -279,6 +276,12 @@ void SetDrawCursor(window)
    XDefineCursor(mainDisplay, window, drawCursor);
 }
 
+void SetFreeHandCursor(window)
+   Window window;
+{
+   XDefineCursor(mainDisplay, window, freehandCursor);
+}
+
 void SetVertexCursor(window)
    Window window;
 {
@@ -348,6 +351,8 @@ void ShowCursor()
       SetTextCursor(drawWindow);
    } else if (curChoice == NOTHING) {
       SetDefaultCursor(drawWindow);
+   } else if (curChoice == FREEHAND) {
+      SetFreeHandCursor(drawWindow);
    } else if (curChoice == VERTEXMODE) {
       SetVertexCursor(drawWindow);
    } else if (curChoice == ROTATEMODE) {
@@ -364,13 +369,13 @@ void CreateCursor()
    XColor color, fg_color, bg_color;
    char *c_ptr;
    int default_cursor_id=XC_arrow, rotate_shear_cursor_id;
-   int draw_cursor_id, drag_cursor_id, vertex_cursor_id;
+   int draw_cursor_id, drag_cursor_id, freehand_cursor_id, vertex_cursor_id;
 
    textPixmap = XCreateBitmapFromData(mainDisplay, mainWindow,
          (char *)text_cur_bits, text_cur_width, text_cur_height);
 
    values.foreground = myFgPixel;
-   values.background = myBgPixel;
+   values.background = (threeDLook ? myLtGryPixel : myBgPixel);
    values.fill_style = FillOpaqueStippled;
    values.stipple = textPixmap;
    textCursorGC = XCreateGC(mainDisplay, mainWindow,
@@ -393,8 +398,9 @@ void CreateCursor()
          }
       }
       if (*cursorName[i] == '\0') {
-         fprintf(stderr, "Can not find DefaultCursor %s, %s used instead.\n",
-               c_ptr, cursorName[default_cursor_id>>1]);
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "DefaultCursor", c_ptr, cursorName[default_cursor_id>>1]);
+         fprintf(stderr, "\n");
       }
    }
    defaultCursor = XCreateFontCursor(mainDisplay, default_cursor_id);
@@ -408,8 +414,9 @@ void CreateCursor()
          }
       }
       if (*cursorName[i] == '\0') {
-         fprintf(stderr, "Can not find DrawCursor %s, %s used instead.\n",
-               c_ptr, cursorName[draw_cursor_id>>1]);
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "DrawCursor", c_ptr, cursorName[draw_cursor_id>>1]);
+         fprintf(stderr, "\n");
       }
    }
    drawCursor = XCreateFontCursor(mainDisplay, draw_cursor_id);
@@ -423,12 +430,29 @@ void CreateCursor()
          }
       }
       if (*cursorName[i] == '\0') {
-         fprintf(stderr, "Can not find DragCursor %s, %s used instead.\n",
-               c_ptr, cursorName[drag_cursor_id>>1]);
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "DragCursor", c_ptr, cursorName[drag_cursor_id>>1]);
+         fprintf(stderr, "\n");
       }
    }
    handCursor = XCreateFontCursor(mainDisplay, drag_cursor_id);
 
+   freehand_cursor_id = XC_pencil;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"FreeHandCursor")) != NULL) {
+      for (i = 0; *cursorName[i] != '\0'; i++) {
+         if (strcmp(c_ptr, cursorName[i]) == 0) {
+            freehand_cursor_id = cursorID[i];
+            break;
+         }
+      }
+      if (*cursorName[i] == '\0') {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "FreeHandCursor", c_ptr, cursorName[freehand_cursor_id>>1]);
+         fprintf(stderr, "\n");
+      }
+   }
+   freehandCursor = XCreateFontCursor(mainDisplay, freehand_cursor_id);
+
    vertex_cursor_id = XC_plus;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"VertexCursor")) != NULL) {
       for (i = 0; *cursorName[i] != '\0'; i++) {
@@ -438,8 +462,9 @@ void CreateCursor()
          }
       }
       if (*cursorName[i] == '\0') {
-         fprintf(stderr, "Can not find VertexCursor %s, %s used instead.\n",
-               c_ptr, cursorName[vertex_cursor_id>>1]);
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "VertexCursor", c_ptr, cursorName[vertex_cursor_id>>1]);
+         fprintf(stderr, "\n");
       }
    }
    vertexCursor = XCreateFontCursor(mainDisplay, vertex_cursor_id);
@@ -453,8 +478,9 @@ void CreateCursor()
          }
       }
       if (*cursorName[i] == '\0') {
-         fprintf(stderr, "Can not find RotateCursor %s, %s %s.\n",
-               c_ptr, cursorName[rotate_shear_cursor_id>>1], "used instead");
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR), TOOL_NAME,
+               "RotateCursor", c_ptr, cursorName[rotate_shear_cursor_id>>1]);
+         fprintf(stderr, "\n");
       }
    }
    rotateCursor = XCreateFontCursor(mainDisplay, rotate_shear_cursor_id);
@@ -470,6 +496,7 @@ void CreateCursor()
    XRecolorCursor(mainDisplay, watchCursor, &fg_color, &bg_color);
    XRecolorCursor(mainDisplay, defaultCursor, &fg_color, &bg_color);
    XRecolorCursor(mainDisplay, drawCursor, &fg_color, &bg_color);
+   XRecolorCursor(mainDisplay, freehandCursor, &fg_color, &bg_color);
    XRecolorCursor(mainDisplay, vertexCursor, &fg_color, &bg_color);
    XRecolorCursor(mainDisplay, rotateCursor, &fg_color, &bg_color);
    XRecolorCursor(mainDisplay, rotatingCursor, &fg_color, &bg_color);
@@ -553,6 +580,7 @@ void CleanUpCursors()
    XFreeCursor(mainDisplay, defaultCursor);
    XFreeCursor(mainDisplay, watchCursor);
    XFreeCursor(mainDisplay, drawCursor);
+   XFreeCursor(mainDisplay, freehandCursor);
    XFreeCursor(mainDisplay, vertexCursor);
    XFreeCursor(mainDisplay, rotateCursor);
    XFreeCursor(mainDisplay, rotatingCursor);
