@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.20 2000/01/15 22:23:38 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.22 2000/05/20 22:36:13 william Exp $
  */
 
 #define _INCLUDE_FROM_EPS_C_
@@ -70,17 +70,22 @@ typedef struct EPSLineRec {
 float	defaultEPSScaling=1.0;
 char	defaultEPSScalingStr[80];
 
+int	autoEPSPreviewBitmap=FALSE;
+
 static struct EPSLineRec *topLine=NULL, *botLine=NULL;
 static int numLines=0;
 
 static char tiffToXbmCmd[MAXSTRING+1];
 static int tiffToXbmCmdInitialized=FALSE;
+static char psToXbmCmd[MAXSTRING+1];
 
 /* do not translate -- program constants */
 static char hexValue[]="0123456789abcdef";
 static char flippedHexValue[]="084c2a6e195d3b7f";
 
 static char defTiffToXbmCmd[]="tifftopnm %s | pgmtopbm | pbmtoxbm";
+static char defPsToXbmCmd[] =
+      "gs -q -dNOPAUSE -sDEVICE=pbm -sOutputFile=- -- \"%s\" | pbmtoxbm";
 
 void InitEPS()
 {
@@ -125,6 +130,27 @@ void InitEPS()
          strcpy(tiffToXbmCmd, defTiffToXbmCmd);
       }
    }
+   /* do not translate -- program constants */
+   strcpy(psToXbmCmd, defPsToXbmCmd);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PsToXbm")) != NULL) {
+      int count=0;
+
+      strcpy(psToXbmCmd, c_ptr);
+      for (c_ptr=strstr(psToXbmCmd,"%s"); c_ptr!=NULL;
+            c_ptr=strstr(++c_ptr,"%s")) {
+         count++;
+      }
+      if (count != 1) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "PsToXbm", psToXbmCmd, defPsToXbmCmd);
+         strcpy(psToXbmCmd, defPsToXbmCmd);
+      }
+   }
+   autoEPSPreviewBitmap = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"AutoEPSPreviewBitmap")) !=
+         NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      autoEPSPreviewBitmap = TRUE;
+   }
 }
 
 static
@@ -666,6 +692,116 @@ void GetEPSFullPath(file_name, full_path, buf_sz)
    }
 }
 
+static
+int ConvertPsToXbm(pszPsPath, pszXbmPath)
+   char *pszPsPath, *pszXbmPath;
+{
+   FILE *pFile=NULL, *pPipe=NULL;
+   char *pszCmd=NULL, szBuf[MAXSTRING];
+   int bytes_read=0;
+
+   sprintf(pszXbmPath, "%sTgifXXXXXX", TMP_DIR);
+   mktemp(pszXbmPath);
+   unlink(pszXbmPath);
+   pszCmd = (char*)malloc(
+         (strlen(psToXbmCmd)+strlen(pszPsPath)+10)*sizeof(char));
+   if (pszCmd == NULL) {
+      FailAllocMessage();
+      return FALSE;
+   }
+   if ((pFile=fopen(pszXbmPath,"w")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
+            pszXbmPath);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(pszCmd);
+      return FALSE;
+   }
+   sprintf(pszCmd, psToXbmCmd, pszPsPath);
+   sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
+         pszCmd);
+   SetStringStatus(gszMsgBox);
+   XSync(mainDisplay, False);
+   if ((pPipe=(FILE*)popen(pszCmd,"r")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_FAIL_TO_EXECUTE_CMD), pszCmd);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(pszCmd);
+      fclose(pFile);
+      unlink(pszXbmPath);
+      return FALSE;
+   }
+   writeFileFailed = FALSE;
+   while ((bytes_read=fread(szBuf, sizeof(char), sizeof(szBuf), pPipe)) > 0) {
+      if ((int)fwrite(szBuf, sizeof(char), bytes_read, pFile) <= 0) {
+         writeFileFailed = TRUE;
+         break;
+      }
+   }
+   pclose(pPipe);
+   SetStringStatus(TgLoadCachedString(CSTID_DOTS_DONE));
+   free(pszCmd);
+   fclose(pFile);
+   if (writeFileFailed) {
+      FailToWriteFileMessage(pszXbmPath);
+      unlink(pszXbmPath);
+      return FALSE;
+   }
+   return TRUE;
+}
+
+static
+int AutoGeneratePreviewBitmap(pszPsPath, pBitmap, pImage, pnImageW, pnImageH)
+   char *pszPsPath;
+   Pixmap *pBitmap;
+   XImage **pImage;
+   int *pnImageW, *pnImageH;
+{
+   char xbm_fname[MAXPATHLENGTH+1];
+   unsigned int tmp_w=0, tmp_h=0;
+   int rc=0, x_hot=0, y_hot=0, empty_image=FALSE, dont_free_orig_bitmap=FALSE;
+   Pixmap orig_bitmap=None, bitmap=None;
+   XImage *image=NULL;
+
+   SetWatchCursor(drawWindow);
+   SetWatchCursor(mainWindow);
+   SaveStatusStrings();
+   rc = ConvertPsToXbm(pszPsPath, xbm_fname);
+   RestoreStatusStrings();
+   SetDefaultCursor(mainWindow);
+   ShowCursor();
+   if (!rc) {
+      return FALSE;
+   }
+   SetWatchCursor(drawWindow);
+   SetWatchCursor(mainWindow);
+   rc = XReadBitmapFile(mainDisplay, mainWindow, xbm_fname, &tmp_w, &tmp_h,
+         &orig_bitmap, &x_hot, &y_hot);
+   SetDefaultCursor(mainWindow);
+   ShowCursor();
+   unlink(xbm_fname);
+   if (rc != BitmapSuccess) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANT_GEN_PREVIEW_FOR_PS),
+            pszPsPath);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return FALSE;
+   }
+   *pnImageW = (int)tmp_w;
+   *pnImageH = (int)tmp_h;
+   rc = AutoTrimBitmap(orig_bitmap, pnImageW, pnImageH, pBitmap, pImage,
+         &empty_image, &dont_free_orig_bitmap);
+   if (!dont_free_orig_bitmap) {
+      XFreePixmap(mainDisplay, orig_bitmap);
+   }
+   if (!rc) {
+      if (empty_image) {
+         sprintf(gszMsgBox, TgLoadString(STID_CANT_GEN_PREVIEW_FOR_EMPTY_PS),
+               pszPsPath);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
+      return FALSE;
+   }
+   return rc;
+}
+
 static
 int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
       pn_num_lines, ppsz_epsflines, pn_epsf_level, pf_llx, pf_lly,
@@ -918,6 +1054,12 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
                real_fname);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
+      if (autoEPSPreviewBitmap && image == NULL && bitmap == None &&
+            !preview_found && !(*tiff_eps_fname != '\0' && tiff_preview_ok)) {
+         if (AutoGeneratePreviewBitmap(real_fname, &bitmap, &image,
+               &image_w, &image_h)) {
+         }
+      }
       if (stat(real_fname, &stat_buf) == 0) {
          strcpy(loc_time, ctime (&(stat_buf.st_mtime)));
          loc_time[24] = '\0';
