@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,22 +28,18 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/expr.c,v 3.14 1998/11/05 22:56:12 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/expr.c,v 4.8 2000/01/15 22:23:40 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <math.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_EXPR_C_
+
+#include "tgifdefs.h"
 
 #include "dialog.e"
-#ifndef _NO_EXTERN
 #include "expr.e"
-#endif /* ~_NO_EXTERN */
 #include "msg.e"
 #include "setup.e"
+#include "strtbl.e"
 #include "util.e"
 
 #define OP_BEGIN 0
@@ -74,14 +70,6 @@
 #define OP_OPEN 25
 #define OP_END 26
 
-static char *opName[] = {
-   "BEGIN", "?", ":", "||", "&&", "|", "^", "&", "==", "!=", ">", "<", ">=",
-   "<=", "<<", ">>", "+", "-", "*", "/", "//", "%", "!", "~", ")", "(", "END",
-   NULL
-};
-
-#define MAXOP 27
-
 struct OpRec {
    int op_code;
    int prec;
@@ -98,15 +86,16 @@ static char *pszExpr=NULL;
 static struct OpRec *topOpStk=NULL;
 static struct ValRec *topValStk=NULL;
 
+#ifdef _TGIF_DBG
 static int gnExprDebug=FALSE;
+#endif /* _TGIF_DBG */
 
 static
 int BadOperandType(operator_str)
    char *operator_str;
 {
-   sprintf(gszMsgBox, "%s '%s'.\n\n%s '%s' %s.",
-         "Can not evaluate", pszCurExpr, "Invalid operand type for the",
-         operator_str, "operator");
+   sprintf(gszMsgBox, TgLoadString(STID_CANNOT_EVAL_INVALID_OP_TYPE),
+         pszCurExpr, operator_str);
    MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    return FALSE;
 }
@@ -115,8 +104,8 @@ static
 int DivByZero(operator_str)
    char *operator_str;
 {
-   sprintf(gszMsgBox, "%s '%s' %s.",
-         "Divide by zero encountered for the", operator_str, "operator");
+   sprintf(gszMsgBox, TgLoadString(STID_DIVIDE_BY_ZERO_FOR_OP_TYPE),
+         operator_str);
    MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    return FALSE;
 }
@@ -192,7 +181,7 @@ int OpArity(op_code)
    case OP_OPEN: return 0; /* '(' */
    case OP_END: return INVALID;
    }
-   sprintf(gszMsgBox, "Illegal expression: %s\n\nInvalid operator code: '%1d'.",
+   sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_INVALID_OP_CODE),
          pszCurExpr, op_code);
    MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    return INVALID;
@@ -279,6 +268,7 @@ int ExprAtomType(expr)
 
    c_ptr = dup_str;
    if (*c_ptr == '-') c_ptr++;
+   /* do not translate -- program constants */
    if (strtok(c_ptr, "0123456789") == NULL) rc = INT_VAL;
    free(dup_str);
    if (rc != INVALID) return rc;
@@ -288,13 +278,12 @@ int ExprAtomType(expr)
 
    c_ptr = dup_str;
    if (*c_ptr == '-') c_ptr++;
+   /* do not translate -- program constants */
    rc = (strtok(c_ptr, ".0123456789") == NULL ? DBL_VAL : STR_VAL);
    free(dup_str);
    return rc;
 }
 
-#define EQ_TOL (1.0e-8)
-
 static
 int EvalEquality(op_code, v, result_ptr)
    int op_code;
@@ -425,6 +414,8 @@ int EvalInequality(op_code, v, result_ptr)
          if (equal != 0) {
             greater = (equal > 0);
             equal = FALSE;
+         } else {
+            equal = TRUE;
          }
          break;
       }
@@ -447,6 +438,16 @@ int EvalMod(left, right)
    return (left-(right*((int)(left/right))));
 }
 
+static
+int MultipleOf(left, right)
+   double left, right;
+{
+   double dmod=(left-(right*((int)(left/right))));
+
+   if (fabs(dmod) < EQ_TOL) return TRUE;
+   return FALSE;
+}
+
 static
 int EvalArithmaticOp(op_code, op_str, v, result_ptr)
    int op_code;
@@ -579,12 +580,12 @@ int EvalArithmaticOp(op_code, op_str, v, result_ptr)
             break;
          case OP_DIV:
             if (fabs(v[0].val.d) < EQ_TOL) { DivByZero(op_str); return FALSE; }
-            if (EvalMod(v[1].val.d, v[0].val.d) == 0) {
+            if (MultipleOf(v[1].val.d, v[0].val.d)) {
                result_ptr->vtype = INT_VAL;
-               result_ptr->val.d = v[1].val.d / v[0].val.d;
+               result_ptr->val.i = round(v[1].val.d / v[0].val.d);
             } else {
                result_ptr->vtype = DBL_VAL;
-               result_ptr->val.d = ((double)v[1].val.d) / ((double)v[0].val.d);
+               result_ptr->val.d = v[1].val.d / v[0].val.d;
             }
             break;
          case OP_INTDIV:
@@ -610,7 +611,7 @@ int Operate()
    struct VRec v[3], result;
    
    if (topOpStk == NULL) {
-      sprintf(gszMsgBox, "Illegal expression: %s\n\nEmpty operator stack.",
+      sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_EMPTY_OP_STACK),
             pszCurExpr);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
@@ -624,13 +625,14 @@ int Operate()
 
    for (i=0; i < arity; i++) {
       if (topValStk == NULL) {
-         sprintf(gszMsgBox, "Illegal expression: %s\n\nEmpty operand stack.",
+         sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_EMPTY_OP_STACK),
                pszCurExpr);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          return FALSE;
       }
       PopValStk(&v[i]);
    }
+   /* do not translate -- program constants */
    switch (op_code) {
    case OP_IFTHEN:
       if (v[2].vtype != INT_VAL) return BadOperandType("?");
@@ -809,8 +811,8 @@ int ParseForNumericVal(ppsz_token, v)
       v->vtype = DBL_VAL;
       if (sscanf(token, "%lf", &(v->val.d)) != 1) {
          **ppsz_token = saved_ch;
-         sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-               pszCurExpr, "Bad numeric value", token);
+         sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_NUM_VALUE),
+               pszCurExpr, token);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          return FALSE;
       }
@@ -818,8 +820,8 @@ int ParseForNumericVal(ppsz_token, v)
       v->vtype = INT_VAL;
       if (sscanf(token, "%d", &(v->val.i)) != 1) {
          **ppsz_token = saved_ch;
-         sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-               pszCurExpr, "Bad numeric value", token);
+         sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_NUM_VALUE),
+               pszCurExpr, token);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          return FALSE;
       }
@@ -843,8 +845,8 @@ int ParseForStringVal(ppsz_token, v)
       }
    }
    if (**ppsz_token == '\0') {
-      sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-            pszCurExpr, "Bad string value", token);
+      sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_STR_VALUE),
+            pszCurExpr, token);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
@@ -855,8 +857,8 @@ int ParseForStringVal(ppsz_token, v)
    **ppsz_token = '"';
    (*ppsz_token)++;
    if (v->val.s == NULL) {
-      sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-            pszCurExpr, "Bad string value", token);
+      sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_STR_VALUE),
+            pszCurExpr, token);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
@@ -885,19 +887,28 @@ int ParseForIDVal(ppsz_token, v)
    v->val.s = UtilStrDup(token);
    **ppsz_token = saved_ch;
    if (v->val.s == NULL) {
-      sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-            pszCurExpr, "Bad identifier", token);
+      sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_IDENTIFIER),
+            pszCurExpr, token);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
    return TRUE;
 }
 
+#ifdef _TGIF_DBG /* debug, do not translate */
+static char *opName[] = {
+   "BEGIN", "?", ":", "||", "&&", "|", "^", "&", "==", "!=", ">", "<", ">=",
+   "<=", "<<", ">>", "+", "-", "*", "/", "//", "%", "!", "~", ")", "(", "END",
+   NULL
+};
+#define MAXOP 27
+
 static
 void DebugOpStk()
 {
-   struct OpRec *op_ptr;
+   struct OpRec *op_ptr=NULL;
    
+   if (gnExprDebug) { }
    fprintf(stderr, "top of op stack -->\n");
    for (op_ptr=topOpStk; op_ptr != NULL; op_ptr=op_ptr->next) {
       fprintf(stderr, "                   | %2s, %2d |\n",
@@ -905,12 +916,15 @@ void DebugOpStk()
    }
    fprintf(stderr, "                   |________|\n");
 }
+#endif /* _TGIF_DBG */
 
+#ifdef _TGIF_DBG /* debug, do not translate */
 static
 void DebugValStk()
 {
-   struct ValRec *v_ptr;
+   struct ValRec *v_ptr=NULL;
    
+   if (gnExprDebug) { }
    fprintf(stderr, "top of val stack ->\n");
    for (v_ptr=topValStk; v_ptr != NULL; v_ptr=v_ptr->next) {
       switch (v_ptr->v.vtype) {
@@ -933,6 +947,7 @@ void DebugValStk()
    }
    fprintf(stderr, "                   |________...\n");
 }
+#endif /* _TGIF_DBG */
 
 static
 int ProcessOp(prec)
@@ -1059,8 +1074,8 @@ int EvalExpr(orig_expr, v_ptr)
             op_code = OP_EQ;
             token = &token[2];
          } else {
-            sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-                  pszCurExpr, "Bad operator", "=");
+            sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_OPERATOR),
+                  pszCurExpr, "=");
             MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             ok = FALSE;
          }
@@ -1129,8 +1144,8 @@ int EvalExpr(orig_expr, v_ptr)
                ok = FALSE;
             }
          } else {
-            sprintf(gszMsgBox, "Illegal expression: '%s'.\n\n%s: '%s'.",
-                  pszCurExpr, "Bad identifier", token);
+            sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_BAD_IDENTIFIER),
+                  pszCurExpr, token);
             MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
             ok = FALSE;
          }
@@ -1141,8 +1156,9 @@ int EvalExpr(orig_expr, v_ptr)
          switch (op_code) {
          case OP_CLOSE:
             if (cur_depth == 0) {
-               sprintf(gszMsgBox, "Illegal expression: %s\n\n%s.",
-                     pszCurExpr, "Too many ')'");
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_ILLEGAL_EXPR_TOO_MANY_SYMBOL),
+                     pszCurExpr, ')');
                MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
                ok = FALSE;
             } else {
@@ -1168,26 +1184,25 @@ int EvalExpr(orig_expr, v_ptr)
    }
    if (ok) {
       if (cur_depth != 0) {
-         sprintf(gszMsgBox, "Illegal expression: %s\n\n%s.",
-               pszCurExpr, "Too many '('");
+         sprintf(gszMsgBox, TgLoadString(STID_ILLEGAL_EXPR_TOO_MANY_SYMBOL),
+               pszCurExpr, '(');
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          ok = FALSE;
       } else if (!ProcessOp((-1))) {
          ok = FALSE;
       } else if (topOpStk != NULL) {
-         sprintf(gszMsgBox, "Illegal expression: %s\n\n%s.",
-               pszCurExpr, "Non empty operator stack after evaluation");
+         sprintf(gszMsgBox, TgLoadString(STID_ILL_EXPR_OP_STACK_NON_EMPTY),
+               pszCurExpr);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          ok = FALSE;
       } else if (topValStk == NULL) {
-         sprintf(gszMsgBox, "Illegal expression: %s\n\n%s.",
-               pszCurExpr, "Non empty value stack after evaluation");
+         sprintf(gszMsgBox, TgLoadString(STID_ILL_EXPR_VAL_STACK_NON_EMPTY),
+               pszCurExpr);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          ok = FALSE;
       } else if (topValStk->next != NULL) {
-         sprintf(gszMsgBox, "Illegal expression: %s\n\n%s.",
-               pszCurExpr,
-               "Too many values left on the value stack after evaluation");
+         sprintf(gszMsgBox, TgLoadString(STID_ILL_EXPR_TOO_MANY_VALS_LEFT),
+               pszCurExpr);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          ok = FALSE;
       } else {
