@@ -1,11 +1,11 @@
 /*
- * Code in this file is derived from the public domain code in
+ * Some code in this file is derived from the public domain code in
  *              WWW/Library/Implementation/HTFTP.c distributed with lynx-2.2,
  *              whose original author is Tim Berners-lee <timbl@info.cern.ch>.
  *
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -22,7 +22,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -32,32 +32,19 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/ftp.c,v 3.13 1998/11/05 22:56:14 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ftp.c,v 4.8 2000/01/15 22:23:44 william Exp $
  */
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#ifdef isc /* SunSoft/Interactive UNIX */
-#include <net/errno.h>
-#endif
-#include <X11/Xlib.h>
+#define _INCLUDE_FROM_FTP_C_
 
-#include "const.h"
-#include "types.h"
+#include "tgifdefs.h"
 
+#include "dialog.e"
 #include "file.e"
-#ifndef _NO_EXTERN
 #include "ftp.e"
-#endif /* !_NO_EXTERN */
+#include "msg.e"
 #include "remote.e"
+#include "strtbl.e"
 #include "tcp.e"
 #include "util.e"
 
@@ -85,18 +72,19 @@ int FtpDoConnect(psz_host, us_port, pn_socket)
    char *msg=(char*)malloc((len+1)*sizeof(char));
 
    if (msg == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
-   sprintf(msg, "Making an FTP connection to \"%s\"...", psz_host);
+   sprintf(msg, TgLoadCachedString(CSTID_MAKING_CONN_TO_HOST), "FTP", psz_host);
    ShowRemoteStatus(msg);
 
    rc = TcpDoConnect(psz_host, us_port, pn_socket);
 
    if (rc == TG_REMOTE_STATUS_OK) {
-      sprintf(msg, "FTP: connection to \"%s\" established.", psz_host);
+      sprintf(msg, TgLoadCachedString(CSTID_CONN_TO_HOST_ESTABLISHED), "FTP",
+            psz_host);
    } else {
-      sprintf(msg, "Fail to connect to FTP server on \"%s\".", psz_host);
+      sprintf(msg, TgLoadString(STID_FAIL_TO_CONN_TO_HOST), "FTP", psz_host);
    }
    ShowRemoteStatus(msg);
    free(msg);
@@ -153,6 +141,7 @@ void FtpDumpResponse(func_name, buf)
    char *c_ptr=strchr(buf, '\n'), *line_ptr=buf;
    FILE *fp=stdout;
 
+   /* debug, do not translate */
    if (debugFtp > 0) fprintf(fp, "In %s:\n", func_name);
    while (c_ptr != NULL) {
       char *prev_ptr=c_ptr;
@@ -200,16 +189,19 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
    if (buf == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
+   CleanUpDownloadStats();
    do {
       int bytes_read;
+      char progress_buf[MAXSTRING];
 
+      *progress_buf = '\0';
       if (buf_sz - len < MIN_READ_SIZE) {
          buf_sz += 0x400;
          if ((buf=(char*)realloc(buf, buf_sz)) == NULL) {
-            fprintf(stderr, "Memory allocation failed.\n");
+            FailAllocMessage();
             if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus &&
                   pn_buf_sz != NULL) {
                fprintf(stderr, "\n");
@@ -218,12 +210,14 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
          }
       }
       bytes_read = read(n_socket, &buf[len], buf_sz-len-1);
+      /* debug, do not translate */
       if (debugFtp >= 3) fprintf(stderr, "      read %1d bytes\n", bytes_read);
       if (bytes_read <= 0) {
          if (bytes_read < 0 && (errno == ENOTCONN || errno == ECONNRESET ||
                errno == EPIPE)) {
             free(buf);
-            fprintf(stderr, "Network read error.\n");
+            fprintf(stderr, TgLoadString(STID_NETWORK_READ_ERROR), "FTP");
+            fprintf(stderr, "\n");
             if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus &&
                   pn_buf_sz != NULL) {
                fprintf(stderr, "\n");
@@ -231,34 +225,46 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
             return TG_REMOTE_STATUS_READ;
          } else if (bytes_read < 0) {
             free(buf);
-            fprintf(stderr, "Network error.\n");
+            fprintf(stderr, TgLoadString(STID_NETWORK_ERROR), "FTP");
+            fprintf(stderr, "\n");
             if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus &&
                   pn_buf_sz != NULL) {
                fprintf(stderr, "\n");
             }
             return TG_REMOTE_STATUS_NET;
          }
+         if (!UpdateDownloadStats(0, progress_buf)) {
+            *progress_buf = '\0';
+         }
          end_of_file = TRUE;
       } else {
+         if (!UpdateDownloadStats(bytes_read, progress_buf)) {
+            *progress_buf = '\0';
+         }
          len += bytes_read;
       }
       if (!end_of_file && UserAbortComm()) {
          if (buf != NULL) free(buf);
-         ShowRemoteStatus("FTP: aborted by the user.");
+         sprintf(gszMsgBox, TgLoadString(STID_CONN_ABORT_BY_USER), "FTP");
+         ShowRemoteStatus(gszMsgBox);
          if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus &&
                pn_buf_sz != NULL) {
             fprintf(stderr, "\n");
          }
          return TG_REMOTE_STATUS_INTR;
       } else {
-         char msg[40], *c_ptr;
+         char msg[MAXSTRING], *c_ptr;
          int cont_code=(-1);
 
-         sprintf(msg, "FTP: %1d bytes...", len);
+         /* do not translate -- program constants */
+         sprintf(msg, "FTP: %1d bytes %s...", len,
+               (*progress_buf=='\0' ? "" : progress_buf));
          ShowRemoteStatus(msg);
          if (PRTGIF && cmdLineDumpURL && cmdLineDumpURLShowStatus &&
                pn_buf_sz != NULL) {
-            fprintf(stderr, "FTP: %1d bytes...\r", len);
+            /* do not translate -- program constants */
+            fprintf(stderr, "FTP: %1d bytes %s...\r", len,
+                  (*progress_buf=='\0' ? "" : progress_buf));
          }
          buf[len] = '\0';
          if (!ftpReadData) {
@@ -289,15 +295,17 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
          pn_buf_sz != NULL) {
       fprintf(stderr, "\n");
    }
+   /* debug, do not translate */
    if (debugFtp >= 2) fprintf(stderr, "    rc = %1d in FtpDoRead().\n", rc);
    if (rc == 421) {
       free(buf);
-      fprintf(stderr, "Connection terminated by the FTP server.\n");
+      fprintf(stderr, TgLoadString(STID_CONN_TERM_BY_SERVER), "FTP");
+      fprintf(stderr, "\n");
       return TG_REMOTE_STATUS_TERM;
    } else {
       buf[len] = '\0';
       *ppsz_buf = buf;
-      if (pn_buf_sz != NULL) *pn_buf_sz = (len+1);
+      if (pn_buf_sz != NULL) *pn_buf_sz = len;
       return TG_REMOTE_STATUS_OK;
    }
 }
@@ -311,7 +319,7 @@ int FtpSendSimpleCmd(n_socket, value)
    char *cmd;
 
    if ((cmd=AppendSimpleString(NULL, value)) == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    status = FtpDoWrite(n_socket, cmd);
@@ -351,11 +359,13 @@ int FtpSendRetrieveCmd(n_socket, psz_path)
    int status=TG_REMOTE_STATUS_OK;
    char *cmd;
 
+   /* do not translate -- program constants */
    cmd = (char*)malloc((strlen("RETR")+1+strlen(psz_path)+2+1)*sizeof(char));
    if (cmd == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
+   /* do not translate -- program constants */
    sprintf(cmd, "RETR %s\r\n", psz_path);
    status = FtpDoWrite(n_socket, cmd);
    free(cmd);
@@ -366,6 +376,7 @@ static
 int FtpSendListCmd(n_socket)
    int n_socket;
 {
+   /* do not translate -- program constants */
    return FtpSendSimpleCmd(n_socket, "NLST");
 }
 
@@ -377,11 +388,13 @@ int FtpSendCWDCmd(n_socket, psz_path)
    int status=TG_REMOTE_STATUS_OK;
    char *cmd;
 
+   /* do not translate -- program constants */
    cmd = (char*)malloc((strlen("CWD")+1+strlen(psz_path)+2+1)*sizeof(char));
    if (cmd == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
+   /* do not translate -- program constants */
    sprintf(cmd, "CWD %s\r\n", psz_path);
    status = FtpDoWrite(n_socket, cmd);
    free(cmd);
@@ -398,7 +411,8 @@ int FtpSendPortCmd(n_socket, pn_data_socket)
 
    *pn_data_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (*pn_data_socket < 0) {
-      fprintf(stderr, "Fail to open a socket for FTP data port.\n");
+      fprintf(stderr, TgLoadString(STID_FAIL_TO_OPEN_DATA_SOCKET), "FTP");
+      fprintf(stderr, "\n");
       return TG_REMOTE_STATUS_FILE;
    }
 
@@ -432,6 +446,7 @@ int FtpSendPortCmd(n_socket, pn_data_socket)
          return TG_REMOTE_STATUS_HOST;
       }
    }
+   /* do not translate -- program constants */
    sprintf(port_cmd, "PORT %d,%d,%d,%d,%d,%d",
          (int)*((unsigned char *)(&sin->sin_addr)+0),
          (int)*((unsigned char *)(&sin->sin_addr)+1),
@@ -456,7 +471,7 @@ int FtpReadCmd(n_socket, ppsz_buf, pn_cmd)
    int status;
 
    if ((status=FtpDoRead(n_socket, ppsz_buf, NULL)) == TG_REMOTE_STATUS_OK) {
-      status = TG_REMOTE_STATUS_HOST;
+      status = TG_REMOTE_STATUS_FORMAT;
       if (*ppsz_buf != NULL) {
          if (sscanf(*ppsz_buf, "%d", pn_cmd) == 1) {
             *pn_cmd = (int)(*pn_cmd / 100);
@@ -479,9 +494,11 @@ int FtpLogin(n_socket)
    int status, ftp_cmd=(-1);
 
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpLogin", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd == 2) {
+         /* do not translate -- program constants */
          status = FtpSendUserName(n_socket, "USER anonymous");
       }
       FtpFreeBuf(buf);
@@ -503,6 +520,7 @@ void InitPassword()
 
    GetUserID(user_name, sizeof(user_name));
 
+   /* do not translate -- program constants */
    sprintf(SZ_PASSWORD, "PASS %s", user_name);
 }
 
@@ -514,6 +532,7 @@ int FtpPassword(n_socket)
    int status, ftp_cmd=(-1);
 
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpPassword", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd == 3) {
@@ -533,15 +552,18 @@ int FtpPort(n_socket, pn_data_socket)
    int status, ftp_cmd=(-1);
 
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpPort", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd == 2) {
          status = FtpSendPortCmd(n_socket, pn_data_socket);
       } else if (ftp_cmd == 3) {
          FtpFreeBuf(buf);
+         /* do not translate -- program constants */
          status = FtpSendPassword(n_socket, "ACCT noaccount");
          if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) ==
                TG_REMOTE_STATUS_OK) {
+            /* debug, do not translate */
             FtpDumpResponse("FtpPort", buf);
             status = TG_REMOTE_STATUS_FORMAT;
             if (ftp_cmd == 2) {
@@ -564,9 +586,11 @@ int FtpType(n_socket)
    int status, ftp_cmd=(-1);
 
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpType", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd == 2) {
+         /* do not translate -- program constants */
          status = FtpSendType(n_socket, "TYPE I");
       }
       FtpFreeBuf(buf);
@@ -583,6 +607,7 @@ int FtpRetr(n_socket, psz_path)
    int status, ftp_cmd=(-1);
 
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpRetr", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd == 2) {
@@ -603,6 +628,7 @@ int FtpCwd(n_socket, psz_path, pn_is_dir)
 
    *pn_is_dir = FALSE;
    if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) == TG_REMOTE_STATUS_OK) {
+      /* debug, do not translate */
       FtpDumpResponse("FtpCwd", buf);
       status = TG_REMOTE_STATUS_FORMAT;
       if (ftp_cmd != 1) {
@@ -610,11 +636,12 @@ int FtpCwd(n_socket, psz_path, pn_is_dir)
          status = FtpSendCWDCmd(n_socket, psz_path);
          if ((status=FtpReadCmd(n_socket, &buf, &ftp_cmd)) ==
                TG_REMOTE_STATUS_OK) {
+            /* debug, do not translate */
             FtpDumpResponse("FtpCwd", buf);
             if (ftp_cmd == 2) {
                *pn_is_dir = TRUE;
                gnReadyToReceiveData = TRUE;
-               status = FtpSendListCmd(n_socket, psz_path);
+               status = FtpSendListCmd(n_socket);
             }
             FtpFreeBuf(buf);
             buf = NULL;
@@ -680,7 +707,7 @@ int FtpDoTalk(n_socket, psz_path, ppsz_buf, pn_buf_sz)
       return (FTP_LOGGED_IN|status);
    }
    if (gnReadyToReceiveData) {
-      sprintf(msg, "FTP: login successful; retrieving data...");
+      sprintf(msg, TgLoadCachedString(CSTID_LOGIN_SUCC_RETRIEVE_DATA), "FTP");
       ShowRemoteStatus(msg);
 
       status = FtpGetContent(n_socket, data_socket, is_dir, ppsz_buf,
