@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.34 2000/01/19 20:25:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.38 2000/05/10 17:34:00 william Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -460,6 +460,7 @@ void ToggleUseGray()
    } else {
       Msg(TgLoadString(STID_NO_GRAY_SCALE_IN_BW_PRINTING));
    }
+   SetFileModified(TRUE);
    UpdatePinnedMenu(MENU_LAYOUT);
 }
 
@@ -544,6 +545,7 @@ void WhereToPrintMsg()
          Msg(TgLoadString(STID_WILL_EXPORT_TIFFEPSI_FILE));
          break;
       case PNG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_PNG_FILE)); break;
+      case JPEG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_JPEG_FILE)); break;
       }
    }
 }
@@ -1346,6 +1348,12 @@ int RefreshShowHideMenu(menu)
    /* ShowMeasurement */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMEASUREMENT,
          showMeasurement);
+   if (!showMeasurement) {
+      ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWMEASUREMENTINTT, FALSE);
+   }
+   /* ShowMeasurementInTooltip */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMEASUREMENTINTT,
+         showMeasurementInTooltip);
    /* ShowMenubar */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMENUBAR, !noMenubar);
    /* ShowStatus */
