@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,30 +28,25 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/help.c,v 3.28 1998/11/05 23:46:11 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/help.c,v 4.9 2000/01/15 22:23:45 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <math.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_HELP_C_
+
+#include "tgifdefs.h"
 #include "patchlvl.h"
 
 #include "color.e"
 #include "dialog.e"
 #include "file.e"
-#ifndef _NO_EXTERN
 #include "help.e"
-#endif
 #include "menu.e"
+#include "menuinfo.e"
 #include "msg.e"
 #include "navigate.e"
 #include "remote.e"
 #include "setup.e"
+#include "strtbl.e"
 #include "util.e"
 #include "version.e"
 
@@ -63,19 +58,6 @@
 
 int numHelp=MAXHELPS;
 
-char *helpMenuStr[] = {
-      "About             ",
-      "Copyright         ",
-      "LatestReleaseInfo ",
-      NULL
-};
-static char *helpMenuDesc[] = {
-      "Version information, where to send bug reports, etc.",
-      "Copyright information",
-      "Find out what is the version number of the most recently released tgif",
-      NULL
-};
-
 /* ----------------------- About ----------------------- */
 
 void About()
@@ -84,34 +66,34 @@ void About()
 
    if (TGIF_PATCHLEVEL == 0) {
       if (*specialBuild == '\0') {
-         sprintf(c_ptr, "%s Version %s", TOOL_NAME, versionString);
+         sprintf(c_ptr, TgLoadString(STID_TOOL_VERSION),
+               TOOL_NAME, versionString);
       } else {
-         sprintf(c_ptr, "%s Version %s (%s)", TOOL_NAME, versionString,
-               specialBuild);
+         sprintf(c_ptr, TgLoadString(STID_TOOL_VERSION_SPC_BUILD),
+               TOOL_NAME, versionString, specialBuild);
       }
    } else {
       if (*specialBuild == '\0') {
-         sprintf(c_ptr, "%s Version %s (patchlevel %1d)", TOOL_NAME,
-               versionString, TGIF_PATCHLEVEL);
+         sprintf(c_ptr, TgLoadString(STID_TOOL_VERSION_PATCH),
+               TOOL_NAME, versionString, TGIF_PATCHLEVEL);
       } else {
-         sprintf(c_ptr, "%s Version %s (patchlevel %1d - %s)", TOOL_NAME,
-               versionString, TGIF_PATCHLEVEL, specialBuild);
+         sprintf(c_ptr, TgLoadString(STID_TOOL_VERSION_PATCH_SPC_BUILD),
+               TOOL_NAME, versionString, TGIF_PATCHLEVEL, specialBuild);
       }
    }
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    sprintf(c_ptr, "\n\n%s", copyrightString);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
-   sprintf(c_ptr, "\n\nWWW Hypertext Home Page: %s", homePageURL);
+   sprintf(c_ptr, TgLoadString(STID_HYPER_TEXT_HOME), homePageURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
-   sprintf(c_ptr, "\n\nLatest Release Information: %s", currentReleaseURL);
+   sprintf(c_ptr, TgLoadString(STID_LATEST_REL_INFO), currentReleaseURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
-   sprintf(c_ptr, "\n\nWWW Hyper-Graphics Home Page: %s", hyperGraphicsURL);
+   sprintf(c_ptr, TgLoadString(STID_HYPER_GRAPHICS_INFO), hyperGraphicsURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
-   sprintf(c_ptr, "\n\n%s %s.  %s %s.",
-         "There is a tgif mailing maintained by Kragen at", mailingList,
-         "To join, please send an empty message to", joinMailingList);
+   sprintf(c_ptr, TgLoadString(STID_MAILING_LIST_INFO),
+         mailingList, joinMailingList);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
-   sprintf(c_ptr, "\n\nPlease send bug reports to %s.", "bill.cheng@acm.org");
+   sprintf(c_ptr, TgLoadString(STID_SEND_BUG_REPORT_TO), "bill.cheng@acm.org");
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
 
@@ -119,6 +101,7 @@ void About()
 
 /* ----------------------- Copyright ----------------------- */
 
+/* do not translate -- program constants */
 static char *gszCopyright[] = {
    "",
    "William Cheng (\"Author\") grants to the party hereby receiving \"tgif\"",
@@ -183,20 +166,22 @@ void LatestReleaseInfo()
 
       *version_str = '\0';
       if (fp == NULL) {
-         sprintf(gszMsgBox, "%s '%s' %s.\n\n%s %s %s.",
-               "Fail to open", currentReleaseURL, "for reading",
-               "Please view", currentReleaseURL, "with a web browser");
+         sprintf(gszMsgBox, TgLoadString(STID_OPEN_URL_FAIL_USE_BROWSER),
+               currentReleaseURL, currentReleaseURL);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       } else {
+         /* do not translate -- program constants */
          int len=strlen("The current release of tgif is"), scanning=TRUE;
          char *buf;
 
          while (scanning && (buf=UtilGetALine(fp)) != NULL) {
+            /* do not translate -- program constants */
             if (strncmp(buf, "The current release of tgif is", len) == 0) {
                char *buf1=UtilGetALine(fp);
 
                scanning = FALSE;
                if (buf1 != NULL) {
+                  /* do not translate -- program constants */
                   char *version_ptr=strstr(buf1, "Version"), *c_ptr=NULL;
 
                   if (version_ptr != NULL &&
@@ -213,52 +198,39 @@ void LatestReleaseInfo()
          fclose(fp);
 
          if (*version_str == '\0') {
-            sprintf(gszMsgBox, "%s '%s'.\n\n%s %s %s.",
-                  "Fail to find version information in", currentReleaseURL,
-                  "Please view", currentReleaseURL, "with a web browser");
+            sprintf(gszMsgBox, TgLoadString(STID_FIND_VER_FAIL_USE_BROWSER),
+                  currentReleaseURL, currentReleaseURL);
          } else if (TGIF_PATCHLEVEL == 0) {
             if (*specialBuild == '\0') {
-               sprintf(gszMsgBox, "%s %s %s %s %s %s %s.\n\n%s %s %s.",
-                     TOOL_NAME, "is currently at", version_str,
-                     "and this", TOOL_NAME, "is Version", versionString,
-                     "For download information, please visit",
-                     (*final_url=='\0' ? currentReleaseURL : final_url),
-                     "with a web browser");
+               sprintf(gszMsgBox, TgLoadString(STID_TOOL_CUR_VER_IS_DIFF_INFO),
+                     TOOL_NAME, version_str, TOOL_NAME, versionString,
+                     (*final_url=='\0' ? currentReleaseURL : final_url));
             } else {
-               sprintf(gszMsgBox, "%s %s %s %s %s %s %s (%s).\n\n%s %s %s.",
-                     TOOL_NAME, "is currently at", version_str,
-                     "and this", TOOL_NAME, "is Version", versionString,
-                     specialBuild, "For download information, please visit",
-                     (*final_url=='\0' ? currentReleaseURL : final_url),
-                     "with a web browser");
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_TOOL_CUR_VER_IS_DIFF_SPC_INFO),
+                     TOOL_NAME, version_str, TOOL_NAME, versionString,
+                     specialBuild,
+                     (*final_url=='\0' ? currentReleaseURL : final_url));
             }
          } else {
             if (*specialBuild == '\0') {
-               sprintf(gszMsgBox,
-                     "%s %s %s %s %s %s %s %s %1d.\n\n%s %s %s.",
-                     TOOL_NAME, "is currently at", version_str,
-                     "and this", TOOL_NAME, "is Version", versionString,
-                     "Patchlevel", TGIF_PATCHLEVEL,
-                     "For download information, please visit",
-                     (*final_url=='\0' ? currentReleaseURL : final_url),
-                     "with a web browser");
+               sprintf(gszMsgBox, TgLoadString(STID_TOOL_CUR_VER_PATCH_INFO),
+                     TOOL_NAME, version_str, TOOL_NAME, versionString,
+                     TGIF_PATCHLEVEL,
+                     (*final_url=='\0' ? currentReleaseURL : final_url));
             } else {
                sprintf(gszMsgBox,
-                     "%s %s %s %s %s %s %s %s %1d (%s).\n\n%s %s %s.",
-                     TOOL_NAME, "is currently at", version_str,
-                     "and this", TOOL_NAME, "is Version", versionString,
-                     "Patchlevel", TGIF_PATCHLEVEL, specialBuild,
-                     "For download information, please visit",
-                     (*final_url=='\0' ? currentReleaseURL : final_url),
-                     "with a web browser");
+                     TgLoadString(STID_TOOL_CUR_VER_PATCH_SPC_INFO),
+                     TOOL_NAME, version_str, TOOL_NAME, versionString,
+                     TGIF_PATCHLEVEL, specialBuild,
+                     (*final_url=='\0' ? currentReleaseURL : final_url));
             }
          }
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
    } else {
-      sprintf(gszMsgBox, "%s '%s' %s.\n\n%s %s %s.",
-            "Cannot download", currentReleaseURL, "to get version information",
-            "Please view", currentReleaseURL, "with a web browser");
+      sprintf(gszMsgBox, TgLoadString(STID_CANT_DOWNLOAD_VER_USE_BROWSER),
+            currentReleaseURL, currentReleaseURL);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    }
    if (*tmp_fname != '\0') unlink(tmp_fname);
@@ -276,27 +248,23 @@ void InitHelp()
 
 /* ----------------------- Menu Functions ----------------------- */
 
-void HelpSubMenu(nIndex)
-   int nIndex;
+void RefreshHelpMenu(menu)
+   TgMenu *menu;
 {
-   switch (nIndex) {
-   case HELP_ABOUT: About(); break;
-   case HELP_COPYRIGHT: Copyright(); break;
-   case HELP_CURRENT: LatestReleaseInfo(); break;
-   }
 }
 
 int HelpMenu(X, Y, TrackMenubar)
    int X, Y, TrackMenubar;
 {
-   int index, *fore_colors, *valid, *init_rv;
-
-   DefaultColorArrays(MAXHELPS, &fore_colors, &valid, &init_rv, NULL);
+   int rc=INVALID;
+   TgMenu *menu=(helpMenuInfo.create_proc)(NULL, X, Y, &helpMenuInfo, FALSE);
 
    activeMenu = MENU_HELP;
-   index = TextMenuLoop(X, Y, helpMenuStr, MAXHELPS, fore_colors,
-         valid, init_rv, helpMenuDesc, SINGLECOLOR, TrackMenubar);
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
 
-   if (index >= 0) HelpSubMenu(index);
-   return index;
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
 }
