@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.53 2000/01/18 21:25:46 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.56 2000/05/20 22:40:56 william Exp $
  */
 
 #define _INCLUDE_FROM_IMPORT_C_
@@ -689,10 +689,12 @@ void ImportEPSFile(Embed)
 
 static char gifToXpmCmd[MAXSTRING+1];
 static char pngToXpmCmd[MAXSTRING+1];
+static char jpegToXpmCmd[MAXSTRING+1];
 static char gifAnimExplodeCmd[MAXSTRING+1];
 
 static char defGifToXpm[]="giftopnm %s | ppmtoxpm";
 static char defPngToXpm[]="pngtopnm %s | ppmquant 222 | ppmtoxpm";
+static char defJpegToXpm[]="djpeg -gif -color 222 %s | giftopnm | ppmtoxpm";
 
 static
 void InitGifToXpm()
@@ -737,6 +739,23 @@ void InitGifToXpm()
       } else {
          strcpy(pngToXpmCmd, defPngToXpm);
       }
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"JpegToXpm")) != NULL) {
+         int count=0;
+
+         strcpy(jpegToXpmCmd, c_ptr);
+         for (c_ptr=strstr(jpegToXpmCmd,"%s"); c_ptr!=NULL;
+               c_ptr=strstr(++c_ptr,"%s")) {
+            count++;
+         }
+         if (count != 1) {
+            sprintf(gszMsgBox, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+                  TOOL_NAME, "JpegToXpm", jpegToXpmCmd, defJpegToXpm);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            strcpy(jpegToXpmCmd, defJpegToXpm);
+         }
+      } else {
+         strcpy(jpegToXpmCmd, defJpegToXpm);
+      }
       if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"GifAnimExplode")) !=
             NULL) {
          strcpy(gifAnimExplodeCmd, c_ptr);
@@ -1020,6 +1039,139 @@ void ImportPNGFile()
    importingFile = FALSE;
 }
 
+int ConvertJpegToXpm(pszJpegPath, pszXpmPath)
+   char *pszJpegPath, *pszXpmPath;
+{
+   FILE *pFile=NULL, *pPipe=NULL;
+   char *pszCmd=NULL, szBuf[MAXSTRING];
+   int bytes_read;
+
+   InitGifToXpm();
+   sprintf(pszXpmPath, "%sTgifXXXXXX", TMP_DIR);
+   mktemp(pszXpmPath);
+   unlink(pszXpmPath);
+   pszCmd = (char*)malloc(
+         (strlen(jpegToXpmCmd)+strlen(pszJpegPath)+10)*sizeof(char));
+   if (pszCmd == NULL) {
+      FailAllocMessage();
+      return FALSE;
+   }
+   if ((pFile=fopen(pszXpmPath,"w")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
+            pszXpmPath);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(pszCmd);
+      return FALSE;
+   }
+   sprintf(pszCmd, jpegToXpmCmd, pszJpegPath);
+   sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
+         pszCmd);
+   SetStringStatus(gszMsgBox);
+   XSync(mainDisplay, False);
+   if ((pPipe=(FILE*)popen(pszCmd,"r")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_FAIL_TO_EXECUTE_CMD), pszCmd);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(pszCmd);
+      fclose(pFile);
+      unlink(pszXpmPath);
+      return FALSE;
+   }
+   writeFileFailed = FALSE;
+   while ((bytes_read=fread(szBuf, sizeof(char), sizeof(szBuf), pPipe)) > 0) {
+      if ((int)fwrite(szBuf, sizeof(char), bytes_read, pFile) <= 0) {
+         writeFileFailed = TRUE;
+         break;
+      }
+   }
+   pclose(pPipe);
+   SetStringStatus(TgLoadCachedString(CSTID_DOTS_DONE));
+   free(pszCmd);
+   fclose(pFile);
+   if (writeFileFailed) {
+      FailToWriteFileMessage(pszXpmPath);
+      unlink(pszXpmPath);
+      return FALSE;
+   }
+   return TRUE;
+}
+
+void ImportJPEGFile()
+{
+   char file_name[MAXPATHLENGTH+1], *rest=NULL, tmp_fname[MAXPATHLENGTH+1];
+   char szJpegPath[MAXPATHLENGTH+1];
+   int rc, short_name=FALSE, image_w=0, image_h=0, remote_file=FALSE;
+   XEvent ev;
+
+   MakeQuiescent();
+
+   importingFile = TRUE;
+   *szJpegPath = *tmp_fname = '\0';
+   if (importFromLibrary) {
+      char name[MAXSTRING+1], path[MAXSTRING+1];
+
+      if (SelectFromLibrary(TgLoadString(STID_SEL_JPEG_FILE_TO_IMPORT),
+            "jpeg", name, path) == INVALID) {
+         importingFile = FALSE;
+         return;
+      }
+      sprintf(szJpegPath, "%s%c%s", path, DIR_SEP, name);
+   } else if (SelectFileNameToImport(TgLoadString(STID_SEL_JPEG_FILE_TO_IMPORT),
+         "jpeg", szJpegPath) == INVALID) {
+      importingFile = FALSE;
+      return;
+   } else if (FileIsRemote(szJpegPath)) {
+      int is_html=FALSE;
+
+      if (!DownloadRemoteFile(szJpegPath, NULL, NULL, &is_html, tmp_fname, NULL,
+           0) || *tmp_fname == '\0') {
+         importingFile = FALSE;
+         return;
+      }
+      remote_file = TRUE;
+   }
+   XSync(mainDisplay, False);
+   if (XCheckMaskEvent(mainDisplay, ExposureMask, &ev)) {
+      ExposeEventHandler(&ev, TRUE);
+   }
+   SetWatchCursor(drawWindow);
+   SetWatchCursor(mainWindow);
+   SaveStatusStrings();
+   rc = ConvertJpegToXpm(remote_file ? tmp_fname : szJpegPath, file_name);
+   RestoreStatusStrings();
+   SetDefaultCursor(mainWindow);
+   ShowCursor();
+   if (!rc) {
+      if (remote_file) unlink(tmp_fname);
+      importingFile = FALSE;
+      return;
+   }
+   if (remote_file) {
+      short_name = FALSE;
+   } else {
+      if ((short_name=IsPrefix(bootDir, szJpegPath, &rest))) ++rest;
+   }
+   if (!ImportGivenXPixmapFile(FALSE, NULL, file_name, INVALID, INVALID,
+         &image_w, &image_h)) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_IMPORT_GIVEN_JPEG),
+            (short_name ? rest : szJpegPath));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      unlink(file_name);
+      if (remote_file) unlink(tmp_fname);
+      importingFile = FALSE;
+      return;
+   }
+   unlink(file_name);
+
+   if (!importFromLibrary && !remote_file) SetCurImportDir(szJpegPath);
+
+   sprintf(gszMsgBox, TgLoadString(STID_GIVEN_JPEG_SIZE_FILE_IMPORTED),
+         image_w, image_h, (short_name ? rest : szJpegPath));
+   Msg(gszMsgBox);
+
+   if (remote_file) unlink(tmp_fname);
+   importingFile = FALSE;
+}
+
 DspList *ImportFilterListing(pnEntries)
    int *pnEntries;
 {
@@ -2314,3 +2466,41 @@ TgMenu *CreateScreenCaptureMenu(parent_menu, x, y, menu_info, status_str_xlated)
    }
    return menu;
 }
+
+void ToggleAutoEPSPreviewBitmap()
+{
+   autoEPSPreviewBitmap = !autoEPSPreviewBitmap;
+   sprintf(gszMsgBox, TgLoadString(autoEPSPreviewBitmap ?
+         STID_WILL_AUTO_GEN_PREVIEW_BITMAP : STID_WONT_AUTO_GEN_PREVIEW_BITMAP),
+         TOOL_NAME);
+   Msg(gszMsgBox);
+}
+
+int RefreshImportMenu(menu)
+   TgMenu *menu;
+{
+   int ok=TRUE;
+
+   /* Auto EPS Preview Bitmap */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLEAUTOEPSPREVIEWBITMAP,
+         autoEPSPreviewBitmap);
+
+   return ok;
+}
+
+TgMenu *CreateImportMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      if (!RefreshImportMenu(menu)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+      menu->refresh_proc = ((RefreshMenuFunc*)RefreshImportMenu);
+   }
+   return menu;
+}
