@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.57 2000/01/15 22:23:50 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.62 2000/05/10 23:55:38 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -121,6 +121,7 @@ int	exitNormally=FALSE;
 char	geometrySpec[80];
 char	*displayName=NULL;
 int	initializingMain=FALSE;
+int	cmdLineUseXlib=FALSE;
 
 int			numExtraWins=0;
 struct WinInfoRec	*extraWinInfo=NULL;
@@ -133,6 +134,8 @@ static int	quitDraw=TRUE;
 static char	*xEnvStr=NULL;
 #endif /* USE_XAPPLRESDIR */
 
+static int	useXlib=FALSE;
+
 int AddExtraWinInfo(win, mapped, raise, expose_handler, ev_handler,
       cleanup_routine, userdata)
    Window win;
@@ -343,6 +346,7 @@ void CleanUp()
    CleanUpShortCut();
    CleanUpHtml();
    CleanUpRecentFiles();
+   CleanUpMeasureTooltip();
 
    if (iconWindowCreated) {
       XDestroyWindow(mainDisplay, iconBaseWindow);
@@ -629,25 +633,29 @@ void InitLocale()
       fprintf(stderr, "%s\n", TgLoadString(STID_CANNOT_SET_LOCALE_ENV_VARS));
    }
    if (x11R6OrAbove) {
-#ifdef USE_XT_INITIALIZE
-      if (XtSetLanguageProc(NULL, NULL, NULL) == NULL) {
-         fprintf(stderr, "%s\n",
-               TgLoadString(STID_XT_CANNOT_SET_LOCALE_ENV_VARS));
-         MySetLocale(LC_ALL, "C");
-      }
-#else /* ~USE_XT_INITIALIZE */
-      if (!XSupportsLocale()) {
-         fprintf(stderr, "%s\n",
-               TgLoadString(STID_XLIB_CANT_SET_LOCALE_ENV_VARS));
-         MySetLocale(LC_ALL, "C");
-      }
-      if (!XSetLocaleModifiers("")) {
-         if (getenv("LANG") != NULL) {
+      if (useXlib) {
+         if (!XSupportsLocale()) {
             fprintf(stderr, "%s\n",
-                  TgLoadString(STID_CANT_SET_LOCALE_MODIFIERS));
+                  TgLoadString(STID_XLIB_CANT_SET_LOCALE_ENV_VARS));
+            MySetLocale(LC_ALL, "C");
+         }
+         if (!XSetLocaleModifiers("")) {
+            if (getenv("LANG") != NULL) {
+               fprintf(stderr, "%s\n",
+                     TgLoadString(STID_CANT_SET_LOCALE_MODIFIERS));
+            }
+         }
+      } else {
+#ifndef USE_XAPPLRESDIR
+#ifdef USE_XT_INITIALIZE
+         if (XtSetLanguageProc(NULL, NULL, NULL) == NULL) {
+            fprintf(stderr, "%s\n",
+                  TgLoadString(STID_XT_CANNOT_SET_LOCALE_ENV_VARS));
+            MySetLocale(LC_ALL, "C");
          }
-      }
 #endif /* USE_XT_INITIALIZE */
+#endif /* ~USE_XAPPLRESDIR */
+      }
    }
    MySetLocale(LC_ALL, NULL);
 
@@ -1010,6 +1018,7 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    int *pnNeedToCheckAutoExec;
 {
    char *c_ptr=NULL;
+   int cannot_opendisplay=FALSE;
 #ifdef USE_XAPPLRESDIR
    FILE *xenv_fp=NULL;
    char tmp_buf[MAXPATHLENGTH+1];
@@ -1057,14 +1066,22 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
 #else /* ~USE_XAPPLRESDIR */
    c_ptr = NULL;
 #ifdef USE_XT_INITIALIZE
-   toplevel = XtInitialize(TOOL_NAME, TOOL_NAME, NULL, 0, &origArgC,
-         origArgV);
-   if (toplevel == NULL || (mainDisplay=XtDisplay(toplevel)) == 0)
+   useXlib = cmdLineUseXlib;
 #else /* ~USE_XT_INITIALIZE */
-   if ((mainDisplay=XOpenDisplay(displayName)) == 0)
+   useXlib = TRUE;
 #endif /* USE_XT_INITIALIZE */
+   if (useXlib) {
+      cannot_opendisplay = ((mainDisplay=XOpenDisplay(displayName)) == 0);
+   } else {
+#ifdef USE_XT_INITIALIZE
+      toplevel = XtInitialize(TOOL_NAME, TOOL_NAME, NULL, 0, &origArgC,
+            origArgV);
+      cannot_opendisplay = (toplevel == NULL ||
+            (mainDisplay=XtDisplay(toplevel)) == 0);
+#endif /* USE_XT_INITIALIZE */
+   }
 #endif /* USE_XAPPLRESDIR */
-   {
+   if (cannot_opendisplay) {
       if (displayName == NULL) {
          fprintf(stderr, "%s\n", TgLoadString(STID_CANT_OPEN_DISPLAY_ABORT));
          exit(-1);
@@ -1360,7 +1377,7 @@ int TryProcessAnAbortXEvent(esc_to_abort, pf_abort_callback, p_void)
       has_ch = XLookupString(key_ev, s, sizeof(s), &key_sym, &c_stat);
       if (has_ch > 0) s[has_ch] = '\0';
       TranslateKeys(s, &key_sym);
-      if (esc_to_abort && has_ch && s[0] == '\033') {
+      if (esc_to_abort && CharIsESC(key_ev, s, key_sym, &has_ch)) {
          if (pf_abort_callback == NULL || (pf_abort_callback)(p_void)) {
             return TRUE;
          }
@@ -1461,13 +1478,17 @@ void MainLoop(Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
       quitDraw = TRUE;
       XFlush(mainDisplay);
       XSync(mainDisplay, True);
+      if (useXlib) {
+         XCloseDisplay(mainDisplay);
+         mainDisplay = NULL;
+      } else {
+#ifndef USE_XAPPLRESDIR
 #ifdef USE_XT_INITIALIZE
-      XtDestroyWidget(toplevel);
-      toplevel = (Widget)0;
-#else /* ~USE_XT_INITIALIZE */
-      XCloseDisplay(mainDisplay);
-      mainDisplay = NULL;
+         XtDestroyWidget(toplevel);
+         toplevel = (Widget)0;
 #endif /* USE_XT_INITIALIZE */
+#endif /* ~USE_XAPPLRESDIR */
+      }
       exitNormally = TRUE;
       AllocStrings(FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3);
       strcpy(*FuncStr, "Quit");
