@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.79 2000/01/19 19:31:02 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.85 2000/05/20 22:52:01 william Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -503,18 +503,24 @@ static TgMenuItemInfo importMenuItemInfo[] = {
      N_("Embed/Import an X11 bitmap file"), NULL, CMDID_IMPORTXBITMAP },
    { N_("Import X Pixmap..."), "Alt+Cntrl+,",
      N_("Embed/Import an X11 pixmap file"), NULL, CMDID_IMPORTXPIXMAP },
-   { N_("Import EPS File..."), "Alt+(", N_("Link to an PS/EPS file"), NULL,
+   { N_("Import EPS File..."), "Alt+(", N_("Link to a PS/EPS file"), NULL,
      CMDID_IMPORTEPSFILE },
    { N_("Import GIF File..."), NULL, N_("Embed/Import a GIF file"), NULL,
      CMDID_IMPORTGIFFILE },
    { N_("Import PNG File..."), NULL, N_("Embed/Import a PNG file"), NULL,
      CMDID_IMPORTPNGFILE },
+   { N_("Import JPEG File..."), NULL, N_("Embed/Import a JPEG file"), NULL,
+     CMDID_IMPORTJPEGFILE },
    { N_("Import Other File..."), NULL, N_("Embed/Import another type of file"),
      NULL, CMDID_IMPORTOTHERFILE },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
-   { N_("Embed EPS File..."), NULL, N_("Embed/Import an PS/EPS file"), NULL,
+   { N_("Embed EPS File..."), NULL, N_("Embed/Import a PS/EPS file"), NULL,
      CMDID_EMBEDEPSFILE },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Auto EPS Preview Bitmap"), NULL,
+     N_("Auto-generate a preview bitmap for a PS/EPS file when embedding/importing it (if it does not have one already)."), NULL,
+     CMDID_TOGGLEAUTOEPSPREVIEWBITMAP },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Browse"), TGMUITEM_SUBMENU,
      N_("Browse submenu >>>"), &browseMenuInfo, INVALID },
    { N_("Screen Capture"), TGMUITEM_SUBMENU,
@@ -522,7 +528,7 @@ static TgMenuItemInfo importMenuItemInfo[] = {
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo importMenuInfo={ TGMUTYPE_TEXT, importMenuItemInfo,
-                                   TgCreateMenuFromMenuInfo };
+                                   CreateImportMenu };
 
 static TgMenuItemInfo domainMenuItemInfo[] = {
    { N_("Change Domain..."), "Alt+*", N_("Change to a different domain"), NULL,
@@ -1028,6 +1034,9 @@ static TgMenuItemInfo showHideMenuItemInfo[] = {
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Show Measurement"), NULL,
      N_("Show cursor measurements"), NULL, CMDID_TOGGLESHOWMEASUREMENT },
+   { N_("Show Measurement in Tooltip"), NULL,
+     N_("Show cursor measurements in tooltip"), NULL,
+     CMDID_TOGGLESHOWMEASUREMENTINTT },
    { N_("Set Measure Unit..."), NULL, N_("Set the unit of measurements"), NULL,
      CMDID_SETMEASUREUNIT },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
@@ -1357,6 +1366,9 @@ static TgMenuItemInfo specialMenuItemInfo[] = {
    { N_("Save Sym In Library..."), NULL,
      N_("Save symbol file in a library path within the current domain"), NULL,
      CMDID_SAVESYMINLIBRARY },
+   { N_("Replace Graphic"), NULL,
+     N_("Replace the graphical part of selected objects by the graphical object in the cut buffer"), NULL,
+     CMDID_REPLACEGRAPHIC },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Domain"), TGMUITEM_SUBMENU,
      N_("Domain submenu >>>"), &domainMenuInfo, INVALID },
@@ -1401,6 +1413,8 @@ TgMenuInfo helpMenuInfo={ TGMUTYPE_TEXT, helpMenuItemInfo,
 TgMenuInfo editAttrInEditorMenuInfo={ TGMUTYPE_TEXT, NULL,
                                       CreateEditAttrInEditorMenu };
 
+TgMenuInfo getPropertyMenuInfo={ TGMUTYPE_TEXT, NULL, CreateGetPropertyMenu };
+
 static TgMenuItemInfo baseContextMenuItemInfo[] = {
    { N_("Mode"), TGMUITEM_PINNABLESUBMENU, N_("Mode Menu"), &modeMenuInfo,
      MENU_MODE },
@@ -1414,10 +1428,14 @@ static TgMenuItemInfo baseContextMenuItemInfo[] = {
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Properties"), TGMUITEM_PINNABLESUBMENU,
      N_("Properties Menu"), &propertiesMenuInfo, MENU_PROPERTIES },
+   { N_("Image Proc"), TGMUITEM_PINNABLESUBMENU,
+     N_("ImageProc Menu"), &imageProcMenuInfo, MENU_IMAGEPROC },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Edit Attribute In Editor"), TGMUITEM_SUBMENU,
      N_("Edit attribute in editor submenu >>>"), &editAttrInEditorMenuInfo,
      INVALID },
+   { N_("Get Property"), TGMUITEM_SUBMENU,
+     N_("Get property submenu >>>"), &getPropertyMenuInfo, INVALID },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo baseContextMenuInfo={ TGMUTYPE_TEXT, baseContextMenuItemInfo,
