@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.42 2000/02/26 22:55:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.44 2000/05/10 19:20:44 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -1375,6 +1375,7 @@ void Usage(tool_name)
    fprintf(stderr, "\t[-geometry <geom>] [=<geom>] \\\n");
    fprintf(stderr, "\t[-exec <file>] \\\n");
    fprintf(stderr, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
+   fprintf(stderr, "\t[-usexlib] \\\n");
    fprintf(stderr, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT, OBJ_FILE_EXT);
    fprintf(stderr, "or:\n");
@@ -1528,6 +1529,10 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             cmdLineWhereToPrint = PNG_FILE;
             cmdLineColor = TRUE;
             cmdLineOpenDisplay = TRUE;
+         } else if (strcmp(*argv,"-jpeg") == 0) {
+            cmdLineWhereToPrint = JPEG_FILE;
+            cmdLineColor = TRUE;
+            cmdLineOpenDisplay = TRUE;
          } else if (strcmp(*argv,"-xpm") == 0) {
             cmdLineWhereToPrint = XBM_FILE;
             cmdLineColor = TRUE;
@@ -1806,6 +1811,8 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          }
          strcpy(cmdLineInputMethod, argv[i]);
          cmdLineHasInputMethod = TRUE;
+      } else if (strcmp(argv[i], "-usexlib") == 0) {
+         cmdLineUseXlib = TRUE;
       } else if (strcmp(argv[i], "-merge") == 0) {
          cmdLineMerge = TRUE;
       } else if (strcmp(argv[i], "-a4") == 0) {
