@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.44 2000/01/15 22:23:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.45 2000/05/02 15:03:59 william Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -969,6 +969,8 @@ int DirNames(TopStr, ExtStr, SelStr, JustSetDir)
       strcpy(dirNamesInfo.other_ext_str, ".GIF");
    } else if (ExtStr != NULL && strcmp(ExtStr, "png") == 0) {
       strcpy(dirNamesInfo.other_ext_str, ".PNG");
+   } else if (ExtStr != NULL && strcmp(ExtStr, "jpeg") == 0) {
+      strcpy(dirNamesInfo.other_ext_str, ".JPEG;.jpg;.JPG");
    } else {
       *dirNamesInfo.other_ext_str = '\0';
    }
