@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,17 +28,13 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/navigate.c,v 3.31 1998/11/05 22:56:19 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.34 2000/02/22 00:32:49 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <sys/types.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/keysym.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_NAVIGATE_C_
+
+#include "tgifdefs.h"
+#include "cmdids.h"
 
 #include "auxtext.e"
 #include "button.e"
@@ -47,6 +43,7 @@
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
+#include "cutpaste.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "file.e"
@@ -54,77 +51,36 @@
 #include "mainloop.e"
 #include "mainmenu.e"
 #include "menu.e"
+#include "menuinfo.e"
 #include "msg.e"
 #include "names.e"
-#ifndef _NO_EXTERN
 #include "navigate.e"
-#endif
 #include "obj.e"
 #include "page.e"
+#include "pattern.e"
+#include "raster.e"
 #include "rect.e"
 #include "remote.e"
 #include "scroll.e"
 #include "select.e"
 #include "setup.e"
 #include "stk.e"
+#include "strtbl.e"
 #include "util.e"
-
-#ifndef XK_KP_Left
-#define XK_KP_Left	0xFF96
-#define XK_KP_Up	0xFF97
-#define XK_KP_Right	0xFF98
-#define XK_KP_Down	0xFF99
-#endif /* ~XK_KP_LEFT */
-
-extern int atoi ARGS_DECL((char*));
-extern char *getenv ARGS_DECL((char*));
+#include "xpixmap.e"
 
 int navigatingBackAndForth=FALSE;
 int inHyperSpace=FALSE;
+int inSlideShow=FALSE;
+int visibleGridInSlideShow=FALSE;
+int slideShowInfoValid=FALSE;
+char *slideShowBorderColor=NULL;
+int slideShowXOffset=0;
+int slideShowYOffset=0;
 int autoHyperSpaceOnRemote=TRUE;
 int allowLaunchInHyperSpace=FALSE;
 int navigateRefresh=FALSE;
-
-char * navigateMenuStr[] = {
-      "GoBack             ",
-      "GoForward          ",
-      "RefreshCurrent     ",
-      "HotList            ",
-      "AddCurrentToHotList",
-      "SessionHistory     ",
-      "GoHyperSpace       ",
-      NULL
-};
-char * navigateMenuStrInHyperSpace[] = {
-      "GoBack             ",
-      "GoForward          ",
-      "RefreshCurrent     ",
-      "HotList            ",
-      "AddCurrentToHotList",
-      "SessionHistory     ",
-      "LeaveHyperSpace    ",
-      NULL
-};
-static char * navigateMenuDescription[] = {
-      "Go back one file",
-      "Go forward one file",
-      "Reload the current file",
-      "Navigate using the hot-list",
-      "Add the current file to the hot-list",
-      "Go to a file visited during this session",
-      "Go into HyperSpace -- cursor will indicate hot spots",
-      NULL
-};
-static char * navigateMenuDescriptionInHyperSpace[] = {
-      "Go back one file",
-      "Go forward one file",
-      "Reload the current file",
-      "Navigate using the hot-list",
-      "Add the current file to the hot-list",
-      "Go to a file visited during this session",
-      "Get out of HyperSpace",
-      NULL
-};
+int goHyperSpaceInSlideShow=FALSE;
 
 static int validHotListFileName=FALSE;
 static char *hotListFileName=NULL;
@@ -145,6 +101,103 @@ static struct NavigateRec *firstSessionHistory=NULL, *lastSessionHistory=NULL;
 static struct URLCacheRec *firstURLCache=NULL, *lastURLCache=NULL;
 static int maxURLCache=(-1), curURLCache=0;
 
+static int lineWidthIndexInSlideShow=4;
+
+int ParseXYSpec(spec, pnx, pny)
+   char *spec;
+   int *pnx, *pny;
+{
+   char *psz_dx=NULL, *psz_dy=NULL, *psz_dup_spec=NULL;
+   int dx, dy;
+
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return FALSE;
+   if ((psz_dup_spec=UtilStrDup(spec)) == NULL) return FailAllocMessage();
+
+   if ((psz_dx=strtok(psz_dup_spec, " ,\t\n\r")) == NULL ||
+         sscanf(psz_dx, "%d", &dx) != 1) {
+      sprintf(gszMsgBox, TgLoadString(STID_ERR_IN_PARSE_STR_FOR_2_VAL), spec);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(psz_dup_spec);
+      return FALSE;
+   }
+   psz_dy = strtok(NULL, " ,\t\n\r");
+   if (psz_dy == NULL) {
+      dy = dx;
+   } else {
+      if (sscanf(psz_dy, "%d", &dy) != 1) {
+         sprintf(gszMsgBox, TgLoadString(STID_ERR_IN_PARSE_STR_FOR_2_VAL),
+               spec);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         free(psz_dup_spec);
+         return FALSE;
+      }
+   }
+   if (pnx != NULL) *pnx = dx;
+   if (pny != NULL) *pny = dy;
+
+   free(psz_dup_spec);
+   return TRUE;
+}
+
+static
+int ParseWHSpecError(spec, psz_dup_spec, rc)
+   char *spec;
+   char *psz_dup_spec;
+   int rc;
+{
+   if (!rc) {
+      sprintf(gszMsgBox, TgLoadString(STID_ERR_IN_PARSE_STR_FOR_2_VAL), spec);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+   }
+   UtilFree(psz_dup_spec);
+
+   return rc;
+}
+
+int ParseWHSpec(spec, pnw, pnh)
+   char *spec;
+   int *pnw, *pnh;
+{
+   char *psz_h=NULL, *psz_dup_spec=NULL;
+   int w=0, h=0;
+
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return FALSE;
+   if ((psz_dup_spec=UtilStrDup(spec)) == NULL) return FailAllocMessage();
+
+   UtilStrLower(psz_dup_spec);
+   if ((psz_h=strchr(psz_dup_spec, 'x')) == NULL) {
+      if ((psz_h=strchr(psz_dup_spec, ' ')) == NULL) {
+         return ParseWHSpecError(spec, psz_dup_spec, FALSE);
+      }
+      *psz_h++ = '\0';
+   } else {
+      *psz_h++ = '\0';
+   }
+   UtilTrimBlanks(psz_dup_spec);
+   UtilTrimBlanks(psz_h);
+   if (!GetDimension(psz_dup_spec, FALSE, &w)) {
+      return ParseWHSpecError(spec, psz_dup_spec, FALSE);
+   }
+   if (!GetDimension(psz_h, FALSE, &h)) {
+      return ParseWHSpecError(spec, psz_dup_spec, FALSE);
+   }
+   if (pnw != NULL) *pnw = w;
+   if (pnh != NULL) *pnh = h;
+
+   free(psz_dup_spec);
+   return TRUE;
+}
+
+void CleanUpSlideShowInfo()
+{
+   if (slideShowBorderColor != NULL) UtilFree(slideShowBorderColor);
+   slideShowBorderColor = NULL;
+   slideShowXOffset = slideShowYOffset = 0;
+   slideShowInfoValid = FALSE;
+}
+
 static
 void InsertNavigate(pnr_prev, pnr_next, pnr)
    struct NavigateRec *pnr_prev, *pnr_next, *pnr;
@@ -236,6 +289,35 @@ void CleanUpNavigate()
    ClearSessionHistory();
 
    CleanUpURLCache();
+   CleanUpSlideShowInfo();
+}
+
+void InitNavigate()
+{
+   char *c_ptr=NULL;
+
+   visibleGridInSlideShow = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "VisibleGridInSlideShow")) != NULL &&
+         UtilStrICmp(c_ptr, "true") == 0) {
+      visibleGridInSlideShow = TRUE;
+   }
+   goHyperSpaceInSlideShow = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "GoHyperSpaceInSlideShow")) !=
+         NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      goHyperSpaceInSlideShow = TRUE;
+   }
+   lineWidthIndexInSlideShow = 4;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "LineWidthIndexInSlideShow")) != NULL) {
+      lineWidthIndexInSlideShow = atoi(c_ptr);
+      if (lineWidthIndexInSlideShow < 0 ||
+            lineWidthIndexInSlideShow >= maxLineWidths) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+               TOOL_NAME, "LineWidthIndexInSlideShow", c_ptr, 4);
+         lineWidthIndexInSlideShow = 4;
+      }
+   }
 }
 
 static
@@ -248,8 +330,8 @@ void InitURLCache()
       if (c_ptr != NULL) {
          maxURLCache = atoi(c_ptr);
          if (maxURLCache < 0) {
-            fprintf(stderr, "Invalid %s*%s: '%s', 40 is used.\n",
-                  TOOL_NAME, "MaxNavigateCacheBuffers", c_ptr);
+            fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_VALUE),
+                  TOOL_NAME, "MaxNavigateCacheBuffers", c_ptr, 40);
             maxURLCache = 40;
          }
       }
@@ -479,12 +561,13 @@ void CommitNavigate()
 
    curNavigate->full_fname = (char*)malloc((len+1)*sizeof(char));
    if (curNavigate->full_fname == NULL) FailAllocMessage();
-   sprintf(curNavigate->full_fname, "%s/%s", curDir, curFileName);
+   sprintf(curNavigate->full_fname, "%s%c%s", curDir, DIR_SEP, curFileName);
    if (firstPage != NULL && firstPage->name != NULL) {
       curNavigate->doc_name = UtilStrDup(firstPage->name);
    } else {
       curNavigate->doc_name = NULL;
    }
+   AddARecentlyUsedFile(curNavigate->full_fname);
    curNavigate->stk = topStk;
 
    InsertNavigate(lastNavigate, NULL, curNavigate);
@@ -499,10 +582,12 @@ void PostNavigate(pnr)
    struct AttrRec *exec_attr=FindFileAttrWithName("auto_exec=");
 
    if (exec_attr != NULL) {
+      ClearObjCachesInAllPages();
       DoExecLoop(NULL, exec_attr);
    } else if (pnr != NULL) {
       ScrollToSpecifiedOrigin(pnr->cur_page_num, pnr->orig_x, pnr->orig_y,
             pnr->zoom_scale, pnr->zoomed_in);
+      ClearObjCachesInAllPages();
    }
 }
 
@@ -542,8 +627,12 @@ void NavigateTo(full_fname, do_not_save, force_load)
       if (content_type != NULL) FreeRemoteBuf(content_type);
       if (buf != NULL) FreeRemoteBuf(buf);
    } else {
+      int obj_file=FALSE, gzipped=FALSE;
+
       navigatingBackAndForth = TRUE;
-      if (!LoadFile(full_fname, TRUE)) {
+      obj_file = FileNameHasExtension(full_fname, OBJ_FILE_TYPE, &gzipped,
+            NULL);
+      if (!LoadFile(full_fname, obj_file, obj_file && gzipped)) {
          if (do_not_save) {
             SetFileModified(TRUE);
          }
@@ -566,18 +655,22 @@ void NavigateBack()
       PopIcon();
       return;
    }
-   while (!DirIsRemote(curDir) && fileModified) {
-      switch (MsgBox(
-            "File modified, save file before going back? [ync](y)",
-            TOOL_NAME, YNC_MB)) {
+   while (!DirIsRemote(curDir) && fileModified && !IsFiletUnSavable()) {
+      XBell(mainDisplay, 0);
+      switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_BACK), TOOL_NAME,
+            YNC_MB)) {
       case MB_ID_YES: SaveFile(); break;
       case MB_ID_NO: do_not_save=TRUE; SetFileModified(FALSE); break;
       case MB_ID_CANCEL: return;
       }
    }
+   if (!DirIsRemote(curDir) && fileModified && IsFiletUnSavable()) {
+      do_not_save = TRUE;
+      SetFileModified(FALSE);
+   }
    MakeQuiescent();
    if (curNavigate == NULL || (curFileDefined && curNavigate->prev == NULL)) {
-      MsgBox("Nothing to go back to!", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_NOTHING_TO_GO_BACK_TO), TOOL_NAME, INFO_MB);
       if (do_not_save) SetFileModified(TRUE);
       return;
    }
@@ -586,6 +679,7 @@ void NavigateBack()
    memcpy(&nr, curNavigate, sizeof(struct NavigateRec));
    NavigateTo(curNavigate->full_fname, do_not_save, FALSE);
    PostNavigate(&nr);
+   SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
 void NavigateForward()
@@ -593,18 +687,22 @@ void NavigateForward()
    int do_not_save=FALSE;
    struct NavigateRec nr;
 
-   while (!DirIsRemote(curDir) && fileModified) {
-      switch (MsgBox(
-            "File modified, save file before going forward? [ync](y)",
-            TOOL_NAME, YNC_MB)) {
+   while (!DirIsRemote(curDir) && fileModified && !IsFiletUnSavable()) {
+      XBell(mainDisplay, 0);
+      switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_FORWARD), TOOL_NAME,
+            YNC_MB)) {
       case MB_ID_YES: SaveFile(); break;
       case MB_ID_NO: do_not_save=TRUE; SetFileModified(FALSE); break;
       case MB_ID_CANCEL: return;
       }
    }
+   if (!DirIsRemote(curDir) && fileModified && IsFiletUnSavable()) {
+      do_not_save = TRUE;
+      SetFileModified(FALSE);
+   }
    MakeQuiescent();
    if (curNavigate == lastNavigate) {
-      MsgBox("Nothing to go forward to!", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_NOTHING_TO_GO_FORWARD_TO), TOOL_NAME, INFO_MB);
       if (do_not_save) SetFileModified(TRUE);
       return;
    }
@@ -613,6 +711,7 @@ void NavigateForward()
    memcpy(&nr, curNavigate, sizeof(struct NavigateRec));
    NavigateTo(curNavigate->full_fname, do_not_save, FALSE);
    PostNavigate(&nr);
+   SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
 void AdjustNavigate()
@@ -630,32 +729,31 @@ void AdjustNavigate()
          curNavigate = curNavigate->prev;
       }
    }
-   Msg("Possible fatal error.  Please read the terminal error output.");
-   fprintf(stderr, "Warning:  fail to find record in AdjustNavigate().\n");
-   fprintf(stderr, "\nNavigating may crash %s.\n", TOOL_NAME);
-   fprintf(stderr, "Safest thing to do is to save and exit.\n");
-   fprintf(stderr, "Please try to reproduce this error and\n");
-   fprintf(stderr, "\tsend bug report to william@cs.ucla.edu.\n");
-   MsgBox("Possible fatal error.  Please read the terminal error output.",
-         TOOL_NAME, STOP_MB);
-   XFlush(mainDisplay);
-   XSync(mainDisplay, False);
+   sprintf(gszMsgBox, TgLoadString(STID_FAIL_TO_FIND_RECORD_IN_FUNC),
+         "AdjustNavigate()");
+   FatalUnexpectedError(gszMsgBox,
+         TgLoadString(STID_FURTHER_NAVIGATE_MAY_CRASH));
 }
 
 void NavigateRefresh()
 {
    if (!curFileDefined) {
-      MsgBox("Can not refresh an undefined file.", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_CANT_REFRESH_UNDEFINED_FILE), TOOL_NAME,
+            INFO_MB);
       return;
    }
-   while (!DirIsRemote(curDir) && fileModified) {
-      switch (MsgBox("File modified, save file before reloading? [ync](y)",
-         TOOL_NAME, YNC_MB)) {
+   while (!DirIsRemote(curDir) && fileModified && !IsFiletUnSavable()) {
+      XBell(mainDisplay, 0);
+      switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_RELOAD), TOOL_NAME,
+            YNC_MB)) {
       case MB_ID_YES: SaveFile(); break;
       case MB_ID_NO: SetFileModified(FALSE); break;
       case MB_ID_CANCEL: return;
       }
    }
+   if (!DirIsRemote(curDir) && fileModified && IsFiletUnSavable()) {
+      SetFileModified(FALSE);
+   }
    NavigateTo(curNavigate->full_fname, TRUE, TRUE);
    PostNavigate(NULL);
 }
@@ -670,9 +768,8 @@ char **ReadHotListFile(pn_count)
 
    if (pn_count != NULL) *pn_count = 0;
    if (!InitHotListFileName() || hotListFileName==NULL) {
-      sprintf(gszMsgBox, "%s.  %s %s*%s.",
-            "Hot list file undefined",
-            "Please specify it in", TOOL_NAME, "HotListFileName");
+      sprintf(gszMsgBox, TgLoadString(STID_HOT_LIST_FILE_UNDEF_PLS_SPEC),
+            TOOL_NAME, "HotListFileName");
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return NULL;
    }
@@ -695,7 +792,8 @@ char **ReadHotListFile(pn_count)
    ppsz_buf[num_lines] = NULL;
    if ((num_lines & 0x1) != 0) {
       ppsz_buf[num_lines-1] = NULL;
-      sprintf(gszMsgBox, "Malformed hot list file '%s'.", hotListFileName);
+      sprintf(gszMsgBox, TgLoadString(STID_MALFORMED_NAMED_HOT_LIST_FILE),
+            hotListFileName);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
    }
    fclose(fp);
@@ -722,13 +820,13 @@ DspList *HotListListing(ppsz_buf, pn_count)
    for (i=(*pn_count)-1, dsp_ptr=pdl, s_ptr=ppsz_buf; *s_ptr != NULL;
          i--, dsp_ptr++, s_ptr=(&s_ptr[2])) {
       if (**s_ptr == '\0') {
-         UtilStrCpy(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), s_ptr[1]);
+         UtilStrCpyN(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), s_ptr[1]);
          dsp_ptr->directory = FALSE; /* use file name as title */
       } else {
-         UtilStrCpy(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), s_ptr[0]);
+         UtilStrCpyN(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), s_ptr[0]);
          dsp_ptr->directory = TRUE; /* file has a title */
       }
-      UtilStrCpy(dsp_ptr->pathstr, sizeof(dsp_ptr->pathstr), s_ptr[1]);
+      UtilStrCpyN(dsp_ptr->pathstr, sizeof(dsp_ptr->pathstr), s_ptr[1]);
       dsp_ptr->next = (i==0 ? NULL : &dsp_ptr[1]);
    }
    return pdl;
@@ -753,14 +851,14 @@ DspList *HistoryListing(pn_count)
    for (i=0, dsp_ptr=pdl, pnr=firstSessionHistory; pnr != NULL;
          i++, dsp_ptr++, pnr=pnr->next) {
       if (pnr->doc_name == NULL) {
-         UtilStrCpy(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr),
+         UtilStrCpyN(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr),
                pnr->full_fname);
          dsp_ptr->directory = FALSE; /* use file name as title */
       } else {
-         UtilStrCpy(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), pnr->doc_name);
+         UtilStrCpyN(dsp_ptr->itemstr, sizeof(dsp_ptr->itemstr), pnr->doc_name);
          dsp_ptr->directory = TRUE; /* file has a title */
       }
-      UtilStrCpy(dsp_ptr->pathstr, sizeof(dsp_ptr->pathstr), pnr->full_fname);
+      UtilStrCpyN(dsp_ptr->pathstr, sizeof(dsp_ptr->pathstr), pnr->full_fname);
       dsp_ptr->next = (i==(*pn_count)-1 ? NULL : &dsp_ptr[1]);
    }
    return pdl;
@@ -776,10 +874,11 @@ struct HotListInfoRec {
 
 static
 int GetHotListEntries(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
-      cur_buf, p_void)
+      pp_check_array, cur_buf, p_void)
    DspList **p_dsp_ptr;
    char ***p_entries, *cur_buf;
    int *pn_num_entries, *pn_marked_index;
+   struct CheckArrayRec **pp_check_array;
    void *p_void;
 {
    struct HotListInfoRec *p_hlinfo=(struct HotListInfoRec *)p_void;
@@ -789,7 +888,7 @@ int GetHotListEntries(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
    if (p_hlinfo->ppsz_buf == NULL) {
       return FALSE;
    } else if (*p_hlinfo->ppsz_buf == NULL) {
-      MsgBox("Hot list file empty!", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_HOT_LIST_FILE_IS_EMPTY), TOOL_NAME, INFO_MB);
       free(p_hlinfo->ppsz_buf);
       p_hlinfo->ppsz_buf = NULL;
       return FALSE;
@@ -805,10 +904,11 @@ int GetHotListEntries(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
 
 static
 int HotListAfterLoop(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
-      cur_buf, btn_id, selected_index, p_void)
+      pp_check_array, cur_buf, btn_id, selected_index, p_void)
    DspList **p_dsp_ptr;
    char ***p_entries, *cur_buf;
    int *pn_num_entries, *pn_marked_index, btn_id, selected_index;
+   struct CheckArrayRec **pp_check_array;
    void *p_void;
 {
    int i, modified=FALSE, something_deleted=FALSE, goto_something=FALSE;
@@ -822,7 +922,7 @@ int HotListAfterLoop(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
    }
    for (s_ptr=p_hlinfo->ppsz_buf, i=0; *s_ptr != NULL; s_ptr=(&s_ptr[2]), i++) {
       if (something_deleted && i == (*pn_marked_index)) {
-         sprintf(gszMsgBox, "Delete '%s' from the hot list? [ync](y)",
+         sprintf(gszMsgBox, TgLoadString(STID_Q_DEL_NAMED_FROM_HOT_LIST),
                (**s_ptr == '\0' ? s_ptr[1] : *s_ptr));
          if (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)==MB_ID_YES) {
             *s_ptr[0] = *s_ptr[1] = '\0';
@@ -852,7 +952,7 @@ int HotListAfterLoop(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
       FILE *fp;
 
       if ((fp=fopen(hotListFileName, "w")) == NULL) {
-         sprintf(gszMsgBox, "Can not open '%s' for write.",
+         sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
                hotListFileName);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       } else {
@@ -872,16 +972,20 @@ int HotListAfterLoop(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
    if (goto_something) {
       int do_not_save=FALSE, canceled=FALSE;
 
-      while (!canceled && !DirIsRemote(curDir) && fileModified)
-      {
-         sprintf(gszMsgBox, "%s, %s? [ync](y)",
-               "File modified", "save file before opening another file");
-         switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
+      while (!canceled && !DirIsRemote(curDir) && fileModified &&
+            !IsFiletUnSavable()) {
+         switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_OPEN_ANO),
+               TOOL_NAME, YNC_MB)) {
          case MB_ID_YES: SaveFile(); break;
          case MB_ID_NO: do_not_save=TRUE; SetFileModified(FALSE); break;
          case MB_ID_CANCEL: canceled=TRUE; break;
          }
       }
+      if (!canceled && !DirIsRemote(curDir) && fileModified &&
+            IsFiletUnSavable()) {
+         do_not_save = TRUE;
+         SetFileModified(FALSE);
+      }
       if (!canceled) {
          for (i=0, s_ptr=p_hlinfo->ppsz_buf; *s_ptr != NULL;
                s_ptr=(&s_ptr[2]), i++) {
@@ -923,8 +1027,8 @@ int GetHistoryEntries(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
 {
    *p_dsp_ptr = HistoryListing(pn_num_entries);
    if (*p_dsp_ptr == NULL) {
-      MsgBox("Sesion history has not been established yet!",
-            TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_SESSION_HIST_NOT_ESTABLISHED), TOOL_NAME,
+            INFO_MB);
       return FALSE;
    } else {
       ignoreDirectoryFlag = TRUE;
@@ -955,15 +1059,20 @@ int HistoryAfterLoop(p_dsp_ptr, p_entries, pn_num_entries, pn_marked_index,
    if (goto_something) {
       int do_not_save=FALSE, canceled=FALSE;
 
-      while (!canceled && !DirIsRemote(curDir) && fileModified) {
-         sprintf(gszMsgBox, "%s, %s? [ync](y)",
-               "File modified", "save file before opening another file");
-         switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
+      while (!canceled && !DirIsRemote(curDir) && fileModified &&
+            !IsFiletUnSavable()) {
+         switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_OPEN_ANO),
+               TOOL_NAME, YNC_MB)) {
          case MB_ID_YES: SaveFile(); break;
          case MB_ID_NO: do_not_save=TRUE; SetFileModified(FALSE); break;
          case MB_ID_CANCEL: canceled=TRUE; break;
          }
       }
+      if (!canceled && !DirIsRemote(curDir) && fileModified &&
+            IsFiletUnSavable()) {
+         do_not_save = TRUE;
+         SetFileModified(FALSE);
+      }
       if (!canceled) {
          int i;
          struct NavigateRec *pnr;
@@ -998,24 +1107,24 @@ void SelectForNavigate(TopStr, Which)
    ResetNamesInfo();
    NamesSetTitle(TopStr);
    NamesSetDefaultBtnId(GOTO_BUTTON, GOTO_BUTTON);
-   NamesSetEntries(NULL, NULL, 0, TRUE, INVALID, 0);
+   NamesSetEntries(NULL, NULL, 0, NULL, TRUE, INVALID, 0);
    switch (Which) {
    case NAVIGATE_HOTLIST:
-      NamesAddButton("GOTO", GOTO_BUTTON);
-      NamesAddButton("DELETE", DELETE_BUTTON);
-      NamesAddButton("CLOSE", BUTTON_CANCEL);
+      NamesAddButton(TgLoadCachedString(CSTID_GOTO), GOTO_BUTTON);
+      NamesAddButton(TgLoadCachedString(CSTID_DELETE), DELETE_BUTTON);
+      NamesAddButton(TgLoadCachedString(CSTID_CLOSE), BUTTON_CANCEL);
       NamesSetStyle(NAMES_EDIT_NAME, NAMES_LOOP_MANY);
       NamesSetCallback((GetEntriesFunc*)GetHotListEntries,
-            (AfterLoopFunc*)HotListAfterLoop);
-      sprintf(win_name, "%s - Hot List", TOOL_NAME);
+            (AfterLoopFunc*)HotListAfterLoop, NULL);
+      sprintf(win_name, TgLoadString(STID_TOOL_HOT_LIST), TOOL_NAME);
       break;
    case NAVIGATE_HISTORY:
-      NamesAddButton("GOTO", GOTO_BUTTON);
-      NamesAddButton("CLOSE", BUTTON_CANCEL);
+      NamesAddButton(TgLoadCachedString(CSTID_GOTO), GOTO_BUTTON);
+      NamesAddButton(TgLoadCachedString(CSTID_CLOSE), BUTTON_CANCEL);
       NamesSetStyle(NAMES_COMPLEX_SELECT_NAME, NAMES_LOOP_MANY);
       NamesSetCallback((GetEntriesFunc*)GetHistoryEntries,
-            (AfterLoopFunc*)HistoryAfterLoop);
-      sprintf(win_name, "%s - History", TOOL_NAME);
+            (AfterLoopFunc*)HistoryAfterLoop, NULL);
+      sprintf(win_name, TgLoadString(STID_TOOL_HOT_LIST), TOOL_NAME);
       break;
    default: return;
    }
@@ -1024,15 +1133,15 @@ void SelectForNavigate(TopStr, Which)
 
 void NavigateHotList()
 {
-   MakeQuiescent ();
+   MakeQuiescent();
    if (!InitHotListFileName() || hotListFileName==NULL) {
-      sprintf(gszMsgBox, "%s.  %s %s*%s.",
-            "Hot list file undefined",
-            "Please specify it in", TOOL_NAME, "HotListFileName");
+      sprintf(gszMsgBox, TgLoadString(STID_HOT_LIST_FILE_UNDEF_PLS_SPEC),
+            TOOL_NAME, "HotListFileName");
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return;
    }
-   SelectForNavigate("Hot list...", NAVIGATE_HOTLIST);
+   SelectForNavigate(TgLoadString(STID_HOT_LIST_DOTS), NAVIGATE_HOTLIST);
+   SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
 void NavigateAddToHotList()
@@ -1042,17 +1151,18 @@ void NavigateAddToHotList()
    FILE *fp;
 
    if (!curFileDefined) {
-      MsgBox("Can not add an undefined file to hot list.", TOOL_NAME, INFO_MB);
+      MsgBox(TgLoadString(STID_CANT_ADD_UNDEF_TO_HOT_LIST), TOOL_NAME, INFO_MB);
       return;
    }
    if ((full_fname=(char*)malloc((len+1)*sizeof(char))) == NULL) {
       FailAllocMessage();
       return;
    }
-   sprintf(full_fname, "%s/%s", curDir, curFileName);
+   sprintf(full_fname, "%s%c%s", curDir, DIR_SEP, curFileName);
    if ((ppsz_buf=ReadHotListFile(&num_entries)) == NULL) return;
    if ((fp=fopen(hotListFileName, "w")) == NULL) {
-      sprintf(gszMsgBox, "Can not open '%s' for write.", hotListFileName);
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
+            hotListFileName);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       free(full_fname);
       return;
@@ -1090,15 +1200,14 @@ void NavigateAddToHotList()
    free(ppsz_buf);
    fclose(fp);
    free(full_fname);
-   sprintf(gszMsgBox, "Hot list file updated (%1d entrie%s).", num_entries,
-         (num_entries>1 ? "s" : ""));
-   Msg(gszMsgBox);
+   Msg(TgLoadString(STID_HOT_LIST_FILE_UPDATED));
 }
 
 void NavigateSessionHistory()
 {
-   MakeQuiescent ();
-   SelectForNavigate("Session history...", NAVIGATE_HISTORY);
+   MakeQuiescent();
+   SelectForNavigate(TgLoadString(STID_SESSION_HISTORY_DOTS), NAVIGATE_HISTORY);
+   SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
 void ToggleHyperSpace(KeepSelected)
@@ -1107,46 +1216,406 @@ void ToggleHyperSpace(KeepSelected)
    inHyperSpace = !inHyperSpace;
    if (inHyperSpace) {
       if (!KeepSelected) MakeQuiescent();
-      Msg("Entering hyperspace...");
+      Msg(TgLoadString(STID_ENTER_HYPERSPACE_DOTS));
    } else {
       ShowCursor();
-      Msg("Leaving hyperspace...");
+      Msg(TgLoadString(STID_LEAVING_HYPERSPACE_DOTS));
    }
-   UpdateSubMenu(MENU_NAVIGATE);
+   UpdatePinnedMenu(MENU_NAVIGATE);
    RedrawDummyWindow1();
 }
 
-void NavigateSubMenu(index)
-   int index;
+struct SlideShowInfoRec {
+   int saved_in_hyperspace;
+   int saved_fill, saved_pen;
+   int saved_line_width, saved_line_style, saved_cur_dash;
+};
+static struct SlideShowInfoRec gstSlideShow;
+
+static
+void StartSlideShow()
 {
-   switch (index) {
-   case NAVIGATE_BACK: NavigateBack(); break;
-   case NAVIGATE_FORWARD: NavigateForward(); break;
-   case NAVIGATE_REFRESH: NavigateRefresh(); break;
-   case NAVIGATE_HOTLIST: NavigateHotList(); break;
-   case NAVIGATE_ADD: NavigateAddToHotList(); break;
-   case NAVIGATE_HISTORY: NavigateSessionHistory(); break;
-   case NAVIGATE_HYPERSPACE: ToggleHyperSpace(FALSE); break;
+   struct AttrRec *exec_attr=FindFileAttrWithName("start_slide_show=");
+
+   gstSlideShow.saved_in_hyperspace = inHyperSpace;
+   gstSlideShow.saved_fill = objFill;
+   gstSlideShow.saved_pen = penPat;
+   gstSlideShow.saved_line_width = lineWidth;
+   gstSlideShow.saved_line_style = lineStyle;
+   gstSlideShow.saved_cur_dash = curDash;
+   objFill = NONEPAT;
+   penPat = SOLIDPAT;
+   lineStyle = LS_PLAIN;
+   curDash = 0;
+   lineWidth = lineWidthIndexInSlideShow;
+
+   if (!goHyperSpaceInSlideShow) {
+      SetCurChoice(FREEHAND);
    }
+   if (slideShowBorderColor != NULL) {
+      int new_alloc=FALSE, index=0;
+
+      index = QuickFindColorIndex(NULL, slideShowBorderColor, &new_alloc,
+            FALSE);
+      if (index != INVALID) {
+         XSetWindowBackground(mainDisplay, mainWindow, colorPixels[index]);
+      }
+   } else if (myFileBgColorStr != NULL && myFileBgPixel != INVALID) {
+      XSetWindowBackground(mainDisplay, mainWindow, myFileBgPixel);
+   }
+   if (goHyperSpaceInSlideShow && !inHyperSpace) {
+      ToggleHyperSpace(FALSE);
+   }
+   FixPageNumbersForSlideShow();
+   ClearObjCachesInAllPages();
+   HidePopupMenusForSlideShow();
+
+   if (exec_attr != NULL) {
+      DoExecLoop(NULL, exec_attr);
+   }
+}
+
+static
+void EndSlideShow()
+{
+   XSetWindowBackground(mainDisplay, mainWindow, myBgPixel);
+
+   ShowPopupMenusForSlideShow();
+   ClearObjCachesInAllPages();
+   if (gstSlideShow.saved_in_hyperspace != inHyperSpace) {
+      ToggleHyperSpace(FALSE);
+   }
+   objFill = gstSlideShow.saved_fill;
+   penPat = gstSlideShow.saved_pen;
+   lineWidth = gstSlideShow.saved_line_width;
+   lineStyle = gstSlideShow.saved_line_style;
+   curDash = gstSlideShow.saved_cur_dash;
+   ShowFill();
+   ShowPen();
+   ShowLineWidth();
+   ShowLineStyle();
+   ShowDash();
+   UpdatePinnedMenu(MENU_FILL);
+   UpdatePinnedMenu(MENU_PEN);
+   UpdatePinnedMenu(MENU_LINEWIDTH);
+   UpdatePinnedMenu(MENU_LINESTYLE);
+   UpdatePinnedMenu(MENU_LINEDASH);
+   MakeQuiescent();
+}
+
+void ToggleSlideShow()
+{
+   int need_to_restore_cur_choice=FALSE;
+
+   if (!inSlideShow) {
+      if (pageLayoutMode == PAGE_TILE) {
+         switch (MsgBox(TgLoadString(STID_CANT_SLIDESHOW_IN_TILED_PAGE),
+               TOOL_NAME, YNC_MB)) {
+         case MB_ID_YES: break;
+         case MB_ID_NO: return;
+         case MB_ID_CANCEL: return;
+         }
+         PageLayoutSubMenu(PAGE_STACK);
+         if (pageLayoutMode != PAGE_STACK) return;
+      } else {
+         if (fileModified && !IsFiletUnSavable() &&
+               ((curFileDefined && !DirIsRemote(curDir)) || !curFileDefined)) {
+            XBell(mainDisplay, 0);
+            switch (MsgBox(TgLoadString(STID_FILE_MOD_SAVE_BEFORE_SLIDE),
+                  TOOL_NAME, YNC_MB)) {
+            case MB_ID_YES: SaveFile(); break;
+            case MB_ID_NO: break;
+            case MB_ID_CANCEL: return;
+            }
+         }
+         if (!IsFiletUnSavable() && NeedToFixPageNumbersForSlideShow()) {
+            sprintf(gszMsgBox,
+                  TgLoadString(STID_Q_PAGE_NUM_MOD_IN_SLIDE_YNC), TOOL_NAME);
+            switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
+            case MB_ID_YES: SetUnSavableFile(TRUE); break;
+            case MB_ID_NO: return;
+            case MB_ID_CANCEL: return;
+            }
+         }
+         MakeQuiescent();
+         need_to_restore_cur_choice = TRUE;
+      }
+   }
+   inSlideShow = !inSlideShow;
+
+   if (inSlideShow) {
+      Msg(TgLoadString(STID_ENTERING_SLIDESHOW_DOTS));
+      StartSlideShow();
+   } else {
+      ShowCursor();
+      Msg(TgLoadString(STID_LEAVING_SLIDESHOW_DOTS));
+      EndSlideShow();
+   }
+   RedrawDummyWindow1();
+   if (need_to_restore_cur_choice) SetCurChoice(curChoiceBeforeMakeQuiescent);
+}
+
+void SetSlideShowBorderColor()
+{
+   char spec[MAXSTRING];
+   int new_alloc=0;
+
+   *spec = '\0';
+   if (Dialog(TgLoadString(STID_ENTER_A_COLOR_FOR_SLIDE_BRDR),
+         TgLoadString(STID_PRESS_ENTER_FOR_DEF_COLOR), spec) == INVALID) {
+      return;
+   }
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') {
+      if (slideShowBorderColor != NULL) UtilFree(slideShowBorderColor);
+      slideShowBorderColor = NULL;
+      Msg(TgLoadString(STID_SLIDE_WILL_USE_DEF_BRDR_COLOR));
+      if (slideShowXOffset == 0 && slideShowYOffset == 0) {
+         slideShowInfoValid = FALSE;
+      }
+      return;
+   }
+   if (QuickFindColorIndex(NULL, spec, &new_alloc, FALSE) == INVALID) {
+      sprintf(gszMsgBox, TgLoadString(STID_FAIL_ALLOC_NAMED_COLOR), spec);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return;
+   }
+   if (slideShowBorderColor != NULL) UtilFree(slideShowBorderColor);
+   slideShowBorderColor = UtilStrDup(spec);
+   if (slideShowBorderColor == NULL) FailAllocMessage();
+   slideShowInfoValid = TRUE;
+
+   sprintf(gszMsgBox, TgLoadString(STID_SLIDE_BRDR_COLOR_SET_TO_NAMED),
+         slideShowBorderColor);
+   Msg(gszMsgBox);
+}
+
+void SetSlideShowWindowOffsets()
+{
+   char spec[MAXSTRING];
+
+   *spec = '\0';
+   sprintf(gszMsgBox, TgLoadString(STID_ENTER_XY_OFFSET_FOR_SLIDE_WIN),
+         slideShowXOffset, slideShowYOffset);
+   if (Dialog(gszMsgBox, NULL, spec) == INVALID) return;
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return;
+
+   if (ParseXYSpec(spec, &slideShowXOffset, &slideShowYOffset)) {
+      sprintf(gszMsgBox, TgLoadString(STID_SLIDE_WIN_XY_OFFSETS_SET_TO),
+            slideShowXOffset, slideShowYOffset);
+      Msg(gszMsgBox);
+   }
+   slideShowInfoValid = TRUE;
+}
+
+void ToggleVisibleGridInSlideShow()
+{
+   visibleGridInSlideShow = (!visibleGridInSlideShow);
+   Msg(TgLoadString(visibleGridInSlideShow ? STID_GRID_BE_VISIBLE_IN_SLIDE :
+         STID_GRID_NOT_BE_VISIBLE_IN_SLIDE));
+}
+
+void GoHyperSpaceInSlideShow()
+{
+   if (goHyperSpaceInSlideShow) return;
+
+   goHyperSpaceInSlideShow = TRUE;
+   Msg(TgLoadString(STID_GO_HYPERSPACE_IN_SLIDE));
+   if (inSlideShow) {
+      if (!inHyperSpace) {
+         inSlideShow = FALSE;
+         ToggleHyperSpace(FALSE);
+         inSlideShow = TRUE;
+      }
+   }
+}
+
+void FreehandModeInSlideShow()
+{
+   if (!goHyperSpaceInSlideShow) return;
+
+   goHyperSpaceInSlideShow = FALSE;
+   Msg(TgLoadString(STID_DONT_GO_HYPERSPACE_IN_SLIDE));
+   if (inSlideShow) {
+      if (inHyperSpace) {
+         ToggleHyperSpace(FALSE);
+      }
+      SetCurChoice(FREEHAND);
+   }
+}
+
+int RefreshSlideShowModeMenu(menu)
+   TgMenu *menu;
+{
+   int ok=TRUE;
+
+   /* GoHyperSpaceInSlideShow */
+   ok &= TgSetMenuItemRadioById(menu, CMDID_GOHYPERSPACEINSLIDESHOW,
+         goHyperSpaceInSlideShow);
+   /* FreehandModeInSlideShow */
+   ok &= TgSetMenuItemRadioById(menu, CMDID_FREEHANDMODEINSLIDESHOW,
+         !goHyperSpaceInSlideShow);
+
+   /* ColorMenu */
+   if (goHyperSpaceInSlideShow) {
+      ok &= TgEnableMenuItemById(menu, MENU_COLOR, FALSE);
+   }
+   return ok;
+}
+
+TgMenu *CreateSlideShowModeMenu(parent_menu, X, Y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int X, Y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, X, Y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      if (!RefreshSlideShowModeMenu(menu)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+      menu->refresh_proc =
+            ((RefreshMenuFunc*)RefreshSlideShowModeMenu);
+   }
+   return menu;
+}
+
+int SlideShowModeMenu(X, Y, TrackMenubar)
+   int X, Y, TrackMenubar;
+{
+   int rc=INVALID;
+   TgMenu *menu=(slideShowModeMenuInfo.create_proc)(NULL, X, Y,
+         &slideShowModeMenuInfo, INVALID);
+
+   activeMenu = INVALID;
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
+
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
+}
+
+int RefreshSlideShowMenu(menu)
+   TgMenu *menu;
+{
+   int ok=TRUE;
+
+   /* SlideShow */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESLIDESHOW, inSlideShow);
+   /* VisibleGridInSlideShow */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_VISIBLEGRIDINSLIDESHOW,
+         visibleGridInSlideShow);
+   /* GoHyperSpaceInSlideShow */
+   ok &= TgSetMenuItemRadioById(menu, CMDID_GOHYPERSPACEINSLIDESHOW,
+         goHyperSpaceInSlideShow);
+   /* FreehandModeInSlideShow */
+   ok &= TgSetMenuItemRadioById(menu, CMDID_FREEHANDMODEINSLIDESHOW,
+         !goHyperSpaceInSlideShow);
+   if (inSlideShow) {
+      ok &= TgEnableMenuItemById(menu, CMDID_GOHYPERSPACEINSLIDESHOW, FALSE);
+      ok &= TgEnableMenuItemById(menu, CMDID_FREEHANDMODEINSLIDESHOW, FALSE);
+   }
+   return ok;
+}
+
+TgMenu *CreateSlideShowMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      if (!RefreshSlideShowMenu(menu)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+   }
+   return menu;
+}
+
+int ReadSlideShowInfo(buf)
+   char *buf;
+{
+   if (!importingFile) {
+      int x=0, y=0;
+      char color_str[40], *psz=FindChar((int)'(', buf);
+
+      *color_str = '\0';
+      psz = ParseStr(psz, (int)',', color_str, sizeof(color_str));
+      InitScan(psz, "\t\n, []");
+      if (GETINT("slideshow_info", x, "x offset") == INVALID ||
+          GETINT("slideshow_info", y, "y offset") == INVALID) {
+         return FALSE;
+      }
+      CleanUpSlideShowInfo();
+      UtilTrimBlanks(color_str);
+      if (*color_str != '\0') {
+         slideShowBorderColor = UtilStrDup(color_str);
+         if (slideShowBorderColor == NULL) FailAllocMessage();
+      }
+      slideShowXOffset = x;
+      slideShowYOffset = y;
+
+      slideShowInfoValid = TRUE;
+   }
+   return TRUE;
+}
+
+void RefreshNavigateMenu(menu)
+   TgMenu *menu;
+{
+   TgMenuItem *menu_item=FindMenuItemByCmdId(menu, CMDID_TOGGLEHYPERSPACE);
+   TgMenuItem stMenuItem;
+
+   if (menu_item == NULL) return;
+
+   memset(&stMenuItem, 0, sizeof(TgMenuItem));
+   stMenuItem.checked = inHyperSpace;
+   TgSetMenuItemInfo(menu_item, TGMU_MASK_CHECK, &stMenuItem);
+}
+
+TgMenu *CreateNavigateMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   TgMenu *menu=TgCreateMenuFromMenuInfo(parent_menu, x, y, menu_info, FALSE);
+
+   if (menu != NULL) {
+      TgMenuItem *menu_item=FindMenuItemByCmdId(menu, CMDID_TOGGLEHYPERSPACE);
+      TgMenuItem stMenuItem;
+
+      if (menu_item == NULL) return TgDestroyMenu(menu, TRUE);
+
+      memset(&stMenuItem, 0, sizeof(TgMenuItem));
+      stMenuItem.checked = inHyperSpace;
+
+      if (!TgSetMenuItemInfo(menu_item, TGMU_MASK_CHECK, &stMenuItem)) {
+         return TgDestroyMenu(menu, TRUE);
+      }
+   }
+   return menu;
 }
 
 int NavigateMenu(X, Y, TrackMenubar)
    int X, Y, TrackMenubar;
 {
-   int index;
-   int *fore_colors, *valid, *init_rv;
+   int rc=INVALID;
+   TgMenu *menu=(navigateMenuInfo.create_proc)(NULL, X, Y, &navigateMenuInfo,
+         INVALID);
 
-   DefaultColorArrays(MAXNAVIGATEMENUS, &fore_colors, &valid, &init_rv, NULL);
    activeMenu = MENU_NAVIGATE;
-   if (inHyperSpace) {
-      index = TextMenuLoop(X, Y, navigateMenuStrInHyperSpace, MAXNAVIGATEMENUS,
-            fore_colors, valid, init_rv, navigateMenuDescriptionInHyperSpace,
-            SINGLECOLOR, TrackMenubar);
-   } else {
-      index = TextMenuLoop(X, Y, navigateMenuStr, MAXNAVIGATEMENUS, fore_colors,
-            valid, init_rv, navigateMenuDescription, SINGLECOLOR, TrackMenubar);
-   }
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
 
-   if (index >= 0) NavigateSubMenu(index);
-   return index;
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
 }
