@@ -1,5 +1,5 @@
 /*
- * @(#)$Header: /mm/src/tgif/v4/RCS/nkf.c,v 4.4 1999/01/07 15:39:27 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/nkf.c,v 4.5 2000/04/18 01:16:51 william Exp $
  */
 
 /** Network Kanji Filter. (PDS Version)
@@ -1499,12 +1499,12 @@ base64decode(c)
     int            c;
 {
     int             i;
-    if (c > '@')
+    if (c > '@') {
 	if (c < '[')
 	    i = c - 'A';	/* A..Z 0-25 */
 	else
 	    i = c - 'G' /* - 'a' + 26 */ ;	/* a..z 26-51 */
-    else if (c > '/')
+    } else if (c > '/')
 	i = c - '0' + '4' /* - '0' + 52 */ ;	/* 0..9 52-61 */
     else if (c == '+')
 	i = '>' /* 62 */ ;	/* +  62 */
