@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,14 +28,12 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/obj.c,v 3.19 1998/11/05 22:56:19 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/obj.c,v 4.9 2000/01/15 22:24:00 william Exp $
  */
 
-#include <stdio.h>
-#include <math.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_OBJ_C_
+
+#include "tgifdefs.h"
 
 #include "arc.e"
 #include "attr.e"
@@ -45,11 +43,10 @@
 #include "cursor.e"
 #include "group.e"
 #include "msg.e"
-#ifndef _NO_EXTERN
 #include "obj.e"
-#endif
 #include "oval.e"
 #include "page.e"
+#include "pin.e"
 #include "poly.e"
 #include "polygon.e"
 #include "rcbox.e"
@@ -61,38 +58,40 @@
 #include "xbitmap.e"
 #include "xpixmap.e"
 
-struct ObjRec	* topObj = NULL, * botObj = NULL;
+struct ObjRec *topObj=NULL, *botObj=NULL;
 
-void AddObj (PrevPtr, NextPtr, ObjPtr)
-   struct ObjRec	* PrevPtr, * NextPtr, * ObjPtr;
+void AddObj(PrevPtr, NextPtr, ObjPtr)
+   struct ObjRec *PrevPtr, *NextPtr, *ObjPtr;
    /* add ObjPtr between PrevPtr and NextPtr */
 {
    ObjPtr->prev = PrevPtr;
    ObjPtr->next = NextPtr;
 
-   if (PrevPtr == NULL)
+   if (PrevPtr == NULL) {
       curPage->top = topObj = ObjPtr;
-   else
+   } else {
       PrevPtr->next = ObjPtr;
-
-   if (NextPtr == NULL)
+   }
+   if (NextPtr == NULL) {
       curPage->bot = botObj = ObjPtr;
-   else
+   } else {
       NextPtr->prev = ObjPtr;
+   }
 }
 
-void UnlinkObj (ObjPtr)
-   struct ObjRec	*ObjPtr;
+void UnlinkObj(ObjPtr)
+   struct ObjRec *ObjPtr;
 {
-   if (topObj == ObjPtr)
+   if (topObj == ObjPtr) {
       curPage->top = topObj = ObjPtr->next;
-   else
+   } else {
       ObjPtr->prev->next = ObjPtr->next;
-
-   if (botObj == ObjPtr)
+   }
+   if (botObj == ObjPtr) {
       curPage->bot = botObj = ObjPtr->prev;
-   else
+   } else {
       ObjPtr->next->prev = ObjPtr->prev;
+   }
 }
 
 void ReplaceObj(old_obj, new_obj)
@@ -121,6 +120,10 @@ void FreeObj(ObjPtr)
       free(ObjPtr->ctm);
       ObjPtr->ctm = NULL;
    }
+   if (ObjPtr->creator_full_id != NULL) {
+      free(ObjPtr->creator_full_id);
+      ObjPtr->creator_full_id = NULL;
+   }
    switch (ObjPtr->type) {
    case OBJ_POLY: DelAllAttrs(ObjPtr->fattr); FreePolyObj(ObjPtr); break;
    case OBJ_BOX: DelAllAttrs(ObjPtr->fattr); FreeBoxObj(ObjPtr); break;
@@ -133,13 +136,26 @@ void FreeObj(ObjPtr)
    case OBJ_XPM: DelAllAttrs(ObjPtr->fattr); FreeXPmObj(ObjPtr); break;
    case OBJ_GROUP:
    case OBJ_ICON:
-   case OBJ_SYM: DelAllAttrs(ObjPtr->fattr); FreeGroupObj(ObjPtr); break;
+   case OBJ_SYM:
+   case OBJ_PIN: DelAllAttrs(ObjPtr->fattr); FreeGroupObj(ObjPtr); break;
    }
 }
 
 void DelObj(ObjPtr)
    struct ObjRec *ObjPtr;
 {
+   if (ObjPtr->type == OBJ_POLY) {
+      if (ObjPtr->detail.p->start_conn != NULL) {
+         DelConnFromPoly(ObjPtr->detail.p->start_conn);
+      }
+      if (ObjPtr->detail.p->end_conn != NULL) {
+         DelConnFromPoly(ObjPtr->detail.p->end_conn);
+      }
+   } else if (ObjPtr->type == OBJ_PIN) {
+      while (ObjPtr->detail.r->pin_connected > 0) {
+         DelConnFromPin(ObjPtr->detail.p->end_conn);
+      }
+   }
    if (topObj == ObjPtr) {
       curPage->top = topObj = ObjPtr->next;
    } else {
@@ -153,16 +169,15 @@ void DelObj(ObjPtr)
    FreeObj(ObjPtr);
 }
 
-void DelAllObj ()
+void DelAllObj()
 {
-   register struct ObjRec	* ptr, * next_obj;
+   register struct ObjRec *ptr, *next_obj;
 
    if (topObj == NULL) return;
 
-   for (ptr = topObj; ptr != NULL; ptr = next_obj)
-   {
+   for (ptr=topObj; ptr != NULL; ptr=next_obj) {
       next_obj = ptr->next;
-      FreeObj (ptr);
+      FreeObj(ptr);
    }
    if (curPage != NULL) curPage->top = curPage->bot = NULL;
    topObj = botObj = NULL;
@@ -179,24 +194,11 @@ void AdjObjOBBox(ObjPtr)
       CalcArcOBBox(ObjPtr);
       break;
    case OBJ_TEXT:
-      switch (ObjPtr->detail.t->rotate) {
-      case ROTATE0:
-      case ROTATE180:
-         SetTextOBBox(ObjPtr, ObjPtr->detail.t->just,
-               ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
-               ObjPtr->obbox.rby-ObjPtr->obbox.lty,
-               ObjPtr->detail.t->min_lbearing,
-               ObjPtr->detail.t->max_rextra, ObjPtr->detail.t->rotate);
-         break;
-      case ROTATE90:
-      case ROTATE270:
-         SetTextOBBox(ObjPtr, ObjPtr->detail.t->just,
-               ObjPtr->obbox.rby-ObjPtr->obbox.lty,
-               ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
-               ObjPtr->detail.t->min_lbearing,
-               ObjPtr->detail.t->max_rextra, ObjPtr->detail.t->rotate);
-         break;
-      }
+      SetTextOBBox(ObjPtr, ObjPtr->detail.t->minilines.just,
+            ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
+            ObjPtr->obbox.rby-ObjPtr->obbox.lty,
+            ObjPtr->detail.t->min_lbearing,
+            ObjPtr->detail.t->max_rextra, ROTATE0);
       break;
    case OBJ_GROUP:
    case OBJ_ICON:
@@ -213,6 +215,13 @@ void AdjObjOBBox(ObjPtr)
       ObjPtr->obbox.ltx = oltx; ObjPtr->obbox.lty = olty;
       ObjPtr->obbox.rbx = orbx; ObjPtr->obbox.rby = orby;
       break;
+   case OBJ_PIN:
+      obj_ptr = GetPinObj(ObjPtr);
+      oltx = obj_ptr->obbox.ltx; olty = obj_ptr->obbox.lty;
+      orbx = obj_ptr->obbox.rbx; orby = obj_ptr->obbox.rby;
+      ObjPtr->obbox.ltx = oltx; ObjPtr->obbox.lty = olty;
+      ObjPtr->obbox.rbx = orbx; ObjPtr->obbox.rby = orby;
+      break;
    }
 }
 
@@ -255,6 +264,13 @@ void AdjObjBBox(ObjPtr)
       ObjPtr->bbox.ltx = ltx; ObjPtr->bbox.lty = lty;
       ObjPtr->bbox.rbx = rbx; ObjPtr->bbox.rby = rby;
       break;
+   case OBJ_PIN:
+      obj_ptr = GetPinObj(ObjPtr);
+      ltx = obj_ptr->bbox.ltx; lty = obj_ptr->bbox.lty;
+      rbx = obj_ptr->bbox.rbx; rby = obj_ptr->bbox.rby;
+      ObjPtr->bbox.ltx = ltx; ObjPtr->bbox.lty = lty;
+      ObjPtr->bbox.rbx = rbx; ObjPtr->bbox.rby = rby;
+      break;
    }
    switch (ObjPtr->type) {
    case OBJ_POLY:
@@ -272,24 +288,11 @@ void AdjObjBBox(ObjPtr)
       ObjPtr->bbox.rby = ObjPtr->obbox.rby + w;
       break;
    case OBJ_TEXT:
-      switch (ObjPtr->detail.t->rotate) {
-      case ROTATE0:
-      case ROTATE180:
-         SetTextBBox(ObjPtr, ObjPtr->detail.t->just,
-               ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
-               ObjPtr->obbox.rby-ObjPtr->obbox.lty,
-               ObjPtr->detail.t->min_lbearing,
-               ObjPtr->detail.t->max_rextra, ObjPtr->detail.t->rotate);
-         break;
-      case ROTATE90:
-      case ROTATE270:
-         SetTextBBox(ObjPtr, ObjPtr->detail.t->just,
-               ObjPtr->obbox.rby-ObjPtr->obbox.lty,
-               ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
-               ObjPtr->detail.t->min_lbearing,
-               ObjPtr->detail.t->max_rextra, ObjPtr->detail.t->rotate);
-         break;
-      }
+      SetTextBBox(ObjPtr, ObjPtr->detail.t->minilines.just,
+            ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
+            ObjPtr->obbox.rby-ObjPtr->obbox.lty,
+            ObjPtr->detail.t->min_lbearing,
+            ObjPtr->detail.t->max_rextra, ROTATE0);
       break;
    }
 
@@ -328,15 +331,50 @@ void AdjObjBBox(ObjPtr)
    }
 }
 
+static
+void RecursivelyAdjAnObjBBox(target_obj, owner_obj)
+   struct ObjRec *target_obj, *owner_obj;
+{
+   if (target_obj == owner_obj) {
+      AdjObjBBox(target_obj);
+   } else {
+      struct ObjRec *obj_ptr=NULL;
+
+      switch (owner_obj->type) {
+      case OBJ_GROUP:
+      case OBJ_ICON:
+      case OBJ_SYM:
+      case OBJ_PIN:
+         for (obj_ptr=owner_obj->detail.r->first; obj_ptr!=NULL;
+               obj_ptr=obj_ptr->next) {
+            RecursivelyAdjAnObjBBox(target_obj, obj_ptr);
+         }
+         AdjObjBBox(owner_obj);
+         break;
+      default: break;
+      }
+   }
+}
+
+void RecursivelyAdjObjBBox(sub_obj, owner_obj, top_owner)
+   struct ObjRec *sub_obj, *owner_obj, *top_owner;
+{
+   if (owner_obj == NULL) {
+      RecursivelyAdjAnObjBBox(sub_obj, sub_obj);
+   } else {
+      RecursivelyAdjAnObjBBox(sub_obj, top_owner);
+   }
+}
+
 static
 void InvalidateObjCache(ObjPtr)
    struct ObjRec *ObjPtr;
 {
-   int w, h, num_rows, num_cols;
-   struct TextRec *text_ptr;
-   struct XBmRec *xbm_ptr;
-   struct XPmRec *xpm_ptr;
-   struct ObjRec *obj_ptr;
+   int w=0, h=0, num_rows=0, num_cols=0;
+   struct TextRec *text_ptr=NULL;
+   struct XBmRec *xbm_ptr=NULL;
+   struct XPmRec *xpm_ptr=NULL;
+   struct ObjRec *obj_ptr=NULL;
 
    w = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
    h = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
@@ -401,11 +439,17 @@ void InvalidateObjCache(ObjPtr)
          if (text_ptr->cached_bitmap != None) {
             XFreePixmap(mainDisplay, text_ptr->cached_bitmap);
          }
+         if (text_ptr->cached_pixmap != None) {
+            XFreePixmap(mainDisplay, text_ptr->cached_pixmap);
+         }
+         if (text_ptr->cached_bg_bitmap != None) {
+            XFreePixmap(mainDisplay, text_ptr->cached_bg_bitmap);
+         }
          text_ptr->cached_bitmap = None;
+         text_ptr->cached_pixmap = None;
+         text_ptr->cached_bg_bitmap = None;
          if (zoomScale != 0) {
             text_ptr->cached_zoom = 0;
-         } else {
-            text_ptr->cached_rotate = INVALID;
          }
       }
       break;
@@ -420,8 +464,6 @@ void InvalidateObjCache(ObjPtr)
          xbm_ptr->cached_bitmap = None;
          if (zoomScale != 0) {
             xbm_ptr->cached_zoom = 0;
-         } else {
-            xbm_ptr->cached_rotate = INVALID;
          }
       }
       break;
@@ -444,8 +486,6 @@ void InvalidateObjCache(ObjPtr)
          xpm_ptr->clip_mask = None;
          if (zoomScale != 0) {
             xpm_ptr->cached_zoom = 0;
-         } else {
-            xpm_ptr->cached_rotate = INVALID;
          }
          xpm_ptr->cached_color = (-1);
       }
@@ -454,6 +494,7 @@ void InvalidateObjCache(ObjPtr)
    case OBJ_GROUP:
    case OBJ_ICON:
    case OBJ_SYM:
+   case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->first; obj_ptr != NULL;
             obj_ptr=obj_ptr->next) {
          InvalidateObjCache(obj_ptr);
@@ -462,8 +503,8 @@ void InvalidateObjCache(ObjPtr)
    }
 }
 
-void AdjObjCache (ObjPtr)
-   struct ObjRec	* ObjPtr;
+void AdjObjCache(ObjPtr)
+   struct ObjRec *ObjPtr;
 {
    register struct ObjRec *obj_ptr;
    register struct AttrRec *attr_ptr;
@@ -473,6 +514,7 @@ void AdjObjCache (ObjPtr)
    case OBJ_GROUP:
    case OBJ_ICON:
    case OBJ_SYM:
+   case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->first; obj_ptr != NULL;
             obj_ptr=obj_ptr->next) {
          AdjObjCache(obj_ptr);
@@ -489,14 +531,14 @@ void AdjObjCache (ObjPtr)
    }
 }
 
-void AdjCaches ()
+void AdjCaches()
 {
    register struct ObjRec *obj_ptr, *ptr;
    register struct AttrRec *attr_ptr;
 
    if (topObj == NULL) return;
 
-   for (obj_ptr = topObj; obj_ptr != NULL; obj_ptr = obj_ptr->next) {
+   for (obj_ptr = topObj; obj_ptr != NULL; obj_ptr=obj_ptr->next) {
       switch (obj_ptr->type) {
       case OBJ_TEXT:
       case OBJ_XBM:
@@ -506,6 +548,7 @@ void AdjCaches ()
       case OBJ_GROUP:
       case OBJ_ICON:
       case OBJ_SYM:
+      case OBJ_PIN:
          for (ptr=obj_ptr->detail.r->first; ptr != NULL; ptr=ptr->next) {
             AdjObjCache(ptr);
          }
@@ -610,11 +653,9 @@ void AdjPolyCache(ObjPtr)
       break;
    }
    if (poly_ptr->style != LS_PLAIN) {
-      IntPoint		*v=poly_ptr->vlist, *v_copy;
-      int			aw = poly_ptr->aw;
-      int			i, n = poly_ptr->n, dx, dy;
-      int			retracted_arrow;
-      double		len, sin_val, cos_val;
+      IntPoint *v=poly_ptr->vlist, *v_copy;
+      int i, n=poly_ptr->n, dx, dy, aw=poly_ptr->aw, retracted_arrow;
+      double len, sin_val, cos_val;
 
       retracted_arrow = (RetractedArrowAttr(ObjPtr) ||
             AutoRetractedArrowAttr(ObjPtr, TRUE));
@@ -742,6 +783,7 @@ void AdjObjSplineVs(ObjPtr)
    case OBJ_GROUP:
    case OBJ_ICON:
    case OBJ_SYM:
+   case OBJ_PIN:
       for (obj_ptr=ObjPtr->detail.r->first; obj_ptr != NULL;
             obj_ptr=obj_ptr->next) {
          AdjObjSplineVs(obj_ptr);
@@ -769,11 +811,12 @@ void AdjSplineVs()
       case OBJ_RCBOX:
       case OBJ_XBM:
       case OBJ_XPM:
-         AdjObjSplineVs (obj_ptr);
+         AdjObjSplineVs(obj_ptr);
          break;
       case OBJ_GROUP:
       case OBJ_ICON:
       case OBJ_SYM:
+      case OBJ_PIN:
          for (ptr=obj_ptr->detail.r->first; ptr != NULL; ptr=ptr->next) {
             AdjObjSplineVs(ptr);
          }
@@ -784,90 +827,100 @@ void AdjSplineVs()
    dontAutoRetractArrow = FALSE;
 }
 
-void MoveRotatedObjCache (ObjPtr, AbsDx, AbsDy)
-   struct ObjRec	* ObjPtr;
-   int			AbsDx, AbsDy;
+void ClearObjCachesInAllPages()
 {
-   register int	i;
-   int		dx_off, dy_off, n=0;
-   XPoint	* v=NULL;
+   struct ObjRec *saved_top_obj=topObj, *saved_bot_obj=botObj;
+   struct PageRec *saved_cur_page=curPage;
+ 
+   for (curPage=firstPage; curPage != NULL; curPage=curPage->next) {
+      topObj = curPage->top;
+      botObj = curPage->bot;
+ 
+      AdjCaches();
+      AdjSplineVs();
+   }
+   topObj = saved_top_obj;
+   botObj = saved_bot_obj;
+   curPage = saved_cur_page;
+}
+
+void MoveRotatedObjCache(ObjPtr, AbsDx, AbsDy)
+   struct ObjRec *ObjPtr;
+   int AbsDx, AbsDy;
+{
+   register int i;
+   int dx_off, dy_off, n=0;
+   XPoint *v=NULL;
 
    if (ObjPtr->ctm == NULL) return;
 
    dx_off = ZOOMED_SIZE(AbsDx);
    dy_off = ZOOMED_SIZE(AbsDy);
-   for (i=0; i < 5; i++)
-   {
+   for (i=0; i < 5; i++) {
       ObjPtr->rotated_obbox[i].x += dx_off;
       ObjPtr->rotated_obbox[i].y += dy_off;
    }
    ObjPtr->orig_obbox.ltx += AbsDx; ObjPtr->orig_obbox.lty += AbsDy;
    ObjPtr->orig_obbox.rbx += AbsDx; ObjPtr->orig_obbox.rby += AbsDy;
-   switch (ObjPtr->type)
-   {
+   switch (ObjPtr->type) {
+   case OBJ_POLY:
+   case OBJ_POLYGON:
+   case OBJ_OVAL:
+   case OBJ_ARC:
+   case OBJ_RCBOX:
+      switch (ObjPtr->type) {
       case OBJ_POLY:
+         n = ObjPtr->detail.p->rotated_n;
+         v = ObjPtr->detail.p->rotated_vlist;
+         break;
       case OBJ_POLYGON:
+         n = ObjPtr->detail.g->rotated_n;
+         v = ObjPtr->detail.g->rotated_vlist;
+         break;
       case OBJ_OVAL:
+         n = ObjPtr->detail.o->rotated_n;
+         v = ObjPtr->detail.o->rotated_vlist;
+         break;
       case OBJ_ARC:
+         n = ObjPtr->detail.a->rotated_n;
+         v = ObjPtr->detail.a->rotated_vlist;
+         break;
       case OBJ_RCBOX:
-         switch (ObjPtr->type)
-         {
-            case OBJ_POLY:
-               n = ObjPtr->detail.p->rotated_n;
-               v = ObjPtr->detail.p->rotated_vlist;
-               break;
-            case OBJ_POLYGON:
-               n = ObjPtr->detail.g->rotated_n;
-               v = ObjPtr->detail.g->rotated_vlist;
-               break;
-            case OBJ_OVAL:
-               n = ObjPtr->detail.o->rotated_n;
-               v = ObjPtr->detail.o->rotated_vlist;
-               break;
-            case OBJ_ARC:
-               n = ObjPtr->detail.a->rotated_n;
-               v = ObjPtr->detail.a->rotated_vlist;
-               break;
-            case OBJ_RCBOX:
-               n = ObjPtr->detail.rcb->rotated_n;
-               v = ObjPtr->detail.rcb->rotated_vlist;
-               break;
+         n = ObjPtr->detail.rcb->rotated_n;
+         v = ObjPtr->detail.rcb->rotated_vlist;
+         break;
+      }
+      if (v != NULL) {
+         for (i=0; i < n; i++) {
+            v[i].x += dx_off;
+            v[i].y += dy_off;
          }
-         if (v != NULL)
-         {
-            for (i=0; i < n; i++)
-            {
+      }
+      if (ObjPtr->type == OBJ_POLY) {
+         n = ObjPtr->detail.p->rotated_asn;
+         v = ObjPtr->detail.p->rotated_asvlist;
+         if (v != NULL) {
+            for (i=0; i < n; i++) {
                v[i].x += dx_off;
                v[i].y += dy_off;
             }
          }
-         if (ObjPtr->type == OBJ_POLY)
-         {
-            n = ObjPtr->detail.p->rotated_asn;
-            v = ObjPtr->detail.p->rotated_asvlist;
-            if (v != NULL)
-            {
-               for (i=0; i < n; i++)
-               {
-                  v[i].x += dx_off;
-                  v[i].y += dy_off;
-               }
-            }
-         }
-         break;
-      case OBJ_BOX:
-      case OBJ_GROUP:
-      case OBJ_ICON:
-      case OBJ_SYM:
-      case OBJ_XBM:
-      case OBJ_XPM:
-         break;
-      case OBJ_TEXT:
-         ObjPtr->detail.t->orig_bbox.ltx += AbsDx;
-         ObjPtr->detail.t->orig_bbox.lty += AbsDy;
-         ObjPtr->detail.t->orig_bbox.rbx += AbsDx;
-         ObjPtr->detail.t->orig_bbox.rby += AbsDy;
-         break;
+      }
+      break;
+   case OBJ_BOX:
+   case OBJ_GROUP:
+   case OBJ_ICON:
+   case OBJ_SYM:
+   case OBJ_PIN:
+   case OBJ_XBM:
+   case OBJ_XPM:
+      break;
+   case OBJ_TEXT:
+      ObjPtr->detail.t->orig_bbox.ltx += AbsDx;
+      ObjPtr->detail.t->orig_bbox.lty += AbsDy;
+      ObjPtr->detail.t->orig_bbox.rbx += AbsDx;
+      ObjPtr->detail.t->orig_bbox.rby += AbsDy;
+      break;
    }
 }
 
@@ -878,3 +931,27 @@ void AdjHotSpots()
 void AdjObjHotSpot()
 {
 }
+
+int IsTopLevelObject(ObjPtr)
+   struct ObjRec *ObjPtr;
+{
+   struct ObjRec *obj_ptr=NULL;
+
+   for (obj_ptr=topObj; obj_ptr != NULL; obj_ptr=obj_ptr->next) {
+      if (obj_ptr == ObjPtr) {
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
+struct ObjRec *GetTopOwner(ObjPtr)
+   struct ObjRec *ObjPtr;
+{
+   struct ObjRec *top_owner=ObjPtr;
+
+   while (top_owner->tmp_parent != NULL) {
+      top_owner = top_owner->tmp_parent;
+   }
+   return top_owner;
+}
