@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/oval.c,v 4.10 2000/01/15 22:24:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/oval.c,v 4.12 2000/05/01 17:51:00 william Exp $
  */
 
 #define _INCLUDE_FROM_OVAL_C_
@@ -546,30 +546,16 @@ static
 void ContinueOval(OrigX, OrigY)
    int OrigX, OrigY;
 {
-   int end_x, end_y, grid_x, grid_y, done=FALSE, abort=FALSE, xor_pixel;
+   int end_x, end_y, grid_x, grid_y, done=FALSE, abort=FALSE;
    char buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80];
    struct BBRec bbox;
-   XGCValues values;
    XEvent input, ev;
    XMotionEvent *motion_ev=NULL;
 
    bbox.ltx = bbox.rbx = OrigX;
    bbox.lty = bbox.rby = OrigY;
 
-   xor_pixel = xorColorPixels[colorIndex];
-
-   values.foreground = xor_pixel;
-   values.function = GXxor;
-   values.fill_style = FillSolid;
-#ifdef NO_THIN_LINE
-   values.line_width = 1;
-#else
-   values.line_width = 0;
-#endif
-   values.line_style = LineSolid;
-   XChangeGC(mainDisplay, drawGC,
-         GCForeground | GCFunction | GCFillStyle | GCLineWidth | GCLineStyle,
-         &values);
+   SetXorDrawGC(xorColorPixels[colorIndex]);
 
    grid_x = end_x = OrigX;
    grid_y = end_y = OrigY; 
@@ -580,15 +566,17 @@ void ContinueOval(OrigX, OrigY)
    sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
    StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
    BeginIntervalRulers(grid_x, grid_y, grid_x, grid_y);
-   XGrabPointer(mainDisplay, drawWindow, FALSE,
-         PointerMotionMask | ButtonReleaseMask,
-         GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
-   
+   if (!debugNoPointerGrab) {
+      XGrabPointer(mainDisplay, drawWindow, FALSE,
+            PointerMotionMask | ButtonReleaseMask,
+            GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
+   }
    while (!done) {
       XNextEvent(mainDisplay, &input);
 
       if (input.type == Expose || input.type == VisibilityNotify) {
          ExposeEventHandler(&input, TRUE);
+         SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if (input.type == ButtonRelease) {
          XUngrabPointer(mainDisplay, CurrentTime);
          MyContinueOval(drawWindow, drawGC, &bbox);
