@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.14 2000/01/15 22:24:04 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.15 2000/04/20 16:28:29 william Exp $
  */
 
 #define _INCLUDE_FROM_POLYGON_C_
@@ -549,7 +549,6 @@ void ContinuePolygon(OrigX, OrigY)
    /* OrigX and OrigY are screen coordinates (scaled and translated). */
 {
    register int i;
-   XGCValues values;
    XEvent input, ev;
    XButtonEvent *button_ev;
    XMotionEvent *motion_ev;
@@ -558,27 +557,13 @@ void ContinuePolygon(OrigX, OrigY)
    int start_polygon_x=OrigX, start_polygon_y=OrigY;
    int end_x, end_y, grid_x, grid_y, done=FALSE;
    int saved_x, saved_y, closed=FALSE, abort=FALSE;
-   int xor_pixel, num_pts=1, n=2, sn=0, max_n=40, intn=0;
+   int num_pts=1, n=2, sn=0, max_n=40, intn=0;
    int ltx=OrigX, lty=OrigY, rbx=OrigX, rby=OrigY;
    struct PtRec *pt_ptr;
    XPoint *sv=NULL;
    IntPoint *pv=NULL, *cntrlv=NULL;
 
-   xor_pixel = xorColorPixels[colorIndex];
-
-   values.foreground = xor_pixel;
-   values.function = GXxor;
-   values.fill_style = FillSolid;
-#ifdef NO_THIN_LINE
-   values.line_width = 1;
-#else
-   values.line_width = 0;
-#endif
-   values.line_style = LineSolid;
-
-   XChangeGC(mainDisplay, drawGC,
-         GCForeground | GCFunction | GCFillStyle | GCLineWidth | GCLineStyle,
-         &values);
+   SetXorDrawGC(xorColorPixels[colorIndex]);
 
    grid_x = end_x = saved_x = OrigX;
    grid_y = end_y = saved_y = OrigY;
@@ -622,6 +607,7 @@ void ContinuePolygon(OrigX, OrigY)
 
       if (input.type == Expose || input.type == VisibilityNotify) {
          ExposeEventHandler(&input, TRUE);
+         SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if ((!(shiftForDiagMouseMove && DiagEventCheck(&input))) &&
             (input.type == KeyPress || input.type == KeyRelease)) {
          int has_ch;
