@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,27 +28,18 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/prtgif.c,v 3.11 1998/11/05 22:56:22 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/prtgif.c,v 4.8 2000/01/15 22:24:05 william Exp $
  */
 
-#include <stdio.h>
+#define _INCLUDE_FROM_PRTGIF_C_
 
 #define PRTGIF_NO_TGIF_DBG 1
 
-#include "const.h"
+#include "tgifdefs.h"
 
-#ifndef _NO_EXTERN
 #include "prtgif.e"
-#endif
 
-extern char	* getenv ARGS_DECL((char *));
-#ifdef SYSV
-extern unsigned	sleep ARGS_DECL((unsigned));
-#else
-extern int	sleep ARGS_DECL((unsigned));
-#endif
-
-int main (argc, argv)
+int main(argc, argv)
    int	argc;
    char	* argv[];
 {
@@ -58,25 +49,27 @@ int main (argc, argv)
    char		tmp_str[MAXSTRING+1], **s_ptr;
    FILE		* fp;
 
-   if ((c_ptr = getenv ("TGIFPATH")) == NULL)
-      strcpy (tgif_path, TGIF_PATH);
-   else
-      if (strlen (c_ptr) >= MAXSTRING)
+   if ((c_ptr=getenv("TGIFPATH")) == NULL) {
+      strcpy(tgif_path, TGIF_PATH);
+   } else {
+      if (strlen(c_ptr) >= MAXSTRING) {
          /* must be an error */
-         strcpy (tgif_path, TGIF_PATH);
-      else
-         strcpy (tgif_path, c_ptr);
+         strcpy(tgif_path, TGIF_PATH);
+      } else {
+         strcpy(tgif_path, c_ptr);
+      }
+   }
 
 #ifdef PRINT_CMD
-   strcpy (print_cmd, PRINT_CMD);
+   strcpy(print_cmd, PRINT_CMD);
 #else
 #ifdef VMS
-   strcpy (print_cmd, "print");
+   strcpy(print_cmd, "print");
 #else
 #ifdef SYSV
-   strcpy (print_cmd, "lp -dpostscript");
+   strcpy(print_cmd, "lp -dpostscript");
 #else
-   strcpy (print_cmd, "lpr");
+   strcpy(print_cmd, "lpr");
 #endif /* SYSV */
 #endif /* VMS */
 #endif /* PRINT_CMD */
@@ -89,8 +82,7 @@ int main (argc, argv)
    }
    cmd = (char*)malloc((total_len+2)*sizeof(char));
    if (cmd == NULL) {
-      fprintf(stderr, "Out of virtual memory and can not malloc().\n");
-      fflush(stderr);
+      fprintf(stderr, "Out of virtual memory and cannot malloc().\n");
    }
    c_ptr = cmd;
    sprintf(c_ptr, "tgif "); c_ptr += strlen("tgif")+1;
@@ -99,25 +91,20 @@ int main (argc, argv)
    c_ptr += strlen(tgif_path)+strlen("-tgif_path")+4;
    sprintf(c_ptr, "-print_cmd \"%s\" ", print_cmd);
    c_ptr += strlen(print_cmd)+strlen("-print_cmd")+4;
-   for (argc--, argv++; argc > 0; argc--, argv++)
-   {
+   for (argc--, argv++; argc > 0; argc--, argv++) {
       sprintf(c_ptr, "%s ", *argv);
       c_ptr += strlen(*argv)+1;
    }
    *(--c_ptr) = '\0';
 
-   if ((fp = (FILE*) popen (cmd, "r")) == NULL)
-   {
-      fprintf(stderr, "Can not popen() to tgif.  Abort!\n");
+   if ((fp=(FILE*)popen(cmd, "r")) == NULL) {
+      fprintf(stderr, "Cannot popen() to tgif.  Abort!\n");
       exit (-1);
    }
-
-   while (fgets (tmp_str, MAXSTRING, fp) != NULL)
-   {
-      fprintf (stderr, "%s", tmp_str);
-      sleep (5);
+   while (fgets(tmp_str, MAXSTRING, fp) != NULL) {
+      fprintf(stderr, "%s", tmp_str);
    }
-   pclose (fp);
+   pclose(fp);
    free(cmd);
-   return (0);
+   return 0;
 }
