@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,25 +28,24 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/ps.c,v 3.17 1998/11/05 22:56:23 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ps.c,v 4.14 2000/01/15 22:24:05 william Exp $
  */
 
-#include <stdio.h>
-#include <X11/Xlib.h>
+#define _INCLUDE_FROM_PS_C_
 
-#include "const.h"
-#include "types.h"
+#include "tgifdefs.h"
 
 #include "msg.e"
-#ifndef _NO_EXTERN
 #include "ps.e"
-#endif
 #include "util.e"
 
 #define DEF_PS_DICT_COUNT 2
 
 int preDumpSetup=FALSE;
 int needsTgifOrigCTM=TRUE;
+int psUseShortHand=FALSE;
+
+char **gPsCmd=NULL;
 
 static int psDictCount=DEF_PS_DICT_COUNT;
 
@@ -61,8 +60,13 @@ static int psUseColorImage=FALSE;
 static int psUseColorPattern=FALSE;
 static int psUseCenterText=FALSE;
 static int psUseRightText=FALSE;
+static int psUseMiniLines=FALSE;
 static int psUseReencode=FALSE;
 
+#ifdef _PS_USE_EUC_FONT
+static int psUseEucFont=FALSE;
+#endif /* _PS_USE_EUC_FONT */
+
 void PSUsePSAdobe()
 {
    if (preDumpSetup && !psUsePSAdobe) {
@@ -153,15 +157,23 @@ void PSUseRightText()
    }
 }
 
+void PSUseMiniLines()
+{
+   if (preDumpSetup && !psUseMiniLines) {
+      psUseMiniLines = TRUE;
+      psDictCount += 3;
+   }
+}
+
 typedef struct ReEncodeRec {
-   char *font_name;
+   char *font_str;
    struct ReEncodeRec *next;
 } *ReEncodePtr;
 
 static struct ReEncodeRec *topReEncodeInfo=NULL;
 
-void PSUseReencode(font_name)
-   char *font_name;
+void PSUseReencode(font_str)
+   char *font_str;
 {
    if (preDumpSetup && !psUseReencode) {
       psUseReencode = TRUE;
@@ -172,19 +184,155 @@ void PSUseReencode(font_name)
 
       for (p_reencode=topReEncodeInfo; p_reencode != NULL;
             p_reencode=p_reencode->next) {
-         if (strcmp(p_reencode->font_name, font_name) == 0) {
+         if (strcmp(p_reencode->font_str, font_str) == 0) {
             return;
          }
       }
       p_reencode = (struct ReEncodeRec *)malloc(sizeof(struct ReEncodeRec));
       if (p_reencode == NULL) FailAllocMessage();
-      p_reencode->font_name = UtilStrDup(font_name);
+      p_reencode->font_str = UtilStrDup(font_str);
       p_reencode->next = topReEncodeInfo;
       topReEncodeInfo = p_reencode;
       psDictCount += 1;
    }
 }
 
+static char *gaszPsShortHand[] = {
+  "GS",   /* "gsave", */
+  "GR",   /* "grestore", */
+  "NP",   /* "newpath", */
+  "CP",   /* "closepath", */
+  "CHP",  /* "charpath", */
+  "CT",   /* "curveto", */
+  "L",    /* "lineto", */
+  "RL",   /* "rlineto", */
+  "M",    /* "moveto", */
+  "RM",   /* "rmoveto", */
+  "S",    /* "stroke", */
+  "F",    /* "fill", */
+  "TR",   /* "translate", */
+  "RO",   /* "rotate", */
+  "SC",   /* "scale", */
+  "MU",   /* "mul", */
+  "DI",   /* "div", */
+  "DU",   /* "dup", */
+  "NE",   /* "neg", */
+  "AD",   /* "add", */
+  "SU",   /* "sub", */
+  "PO",   /* "pop", */
+  "EX",   /* "exch", */
+  "CO",   /* "concat", */
+  "CL",   /* "clip", */
+  "EC",   /* "eoclip", */
+  "EF",   /* "eofill", */
+  "IM",   /* "image", */
+  "IMM",  /* "imagemask", */
+  "ARY",  /* "array", */
+  "SG",   /* "setgray", */
+  "RG",   /* "setrgbcolor", */
+  "SD",   /* "setdash", */
+  "W",    /* "setlinewidth", */
+  "SM",   /* "setmiterlimit", */
+  "SLC",  /* "setlinecap", */
+  "SLJ",  /* "setlinejoin", */
+  "SH",   /* "show", */
+  "FF",   /* "findfont", */
+  "MS",   /* "makefont setfont", */
+  "AR",   /* "arcto 4 {pop} repeat", */
+  "CURP", /* "currentpoint", */
+  "FLAT", /* "flattenpath strokepath clip newpath", */
+  "\tTGSM", /* "tgiforigctm setmatrix", */
+  "\tTGRM", /* "savematrix setmatrix", */
+  NULL
+};
+
+static char *gaszPsLongHand[] = {
+  /* "GS",   */ "gsave",
+  /* "GR",   */ "grestore",
+  /* "NP",   */ "newpath",
+  /* "CP",   */ "closepath",
+  /* "CHP",  */ "charpath",
+  /* "CT",   */ "curveto",
+  /* "L",    */ "lineto",
+  /* "RL",   */ "rlineto",
+  /* "M",    */ "moveto",
+  /* "RM",   */ "rmoveto",
+  /* "S",    */ "stroke",
+  /* "F",    */ "fill",
+  /* "TR",   */ "translate",
+  /* "RO",   */ "rotate",
+  /* "SC",   */ "scale",
+  /* "MU",   */ "mul",
+  /* "DI",   */ "div",
+  /* "DU",   */ "dup",
+  /* "NE",   */ "neg",
+  /* "AD",   */ "add",
+  /* "SU",   */ "sub",
+  /* "PO",   */ "pop",
+  /* "EX",   */ "exch",
+  /* "CO",   */ "concat",
+  /* "CL",   */ "clip",
+  /* "EC",   */ "eoclip",
+  /* "EF",   */ "eofill",
+  /* "IM",   */ "image",
+  /* "IMM",  */ "imagemask",
+  /* "ARY",  */ "array",
+  /* "SG",   */ "setgray",
+  /* "RG",   */ "setrgbcolor",
+  /* "SD",   */ "setdash",
+  /* "W",    */ "setlinewidth",
+  /* "SM",   */ "setmiterlimit",
+  /* "SLC",  */ "setlinecap",
+  /* "SLJ",  */ "setlinejoin",
+  /* "SH",   */ "show",
+  /* "FF",   */ "findfont",
+  /* "MS",   */ "makefont setfont",
+  /* "AR",   */ "arcto 4 {pop} repeat",
+  /* "CURP", */ "currentpoint",
+  /* "FLAT", */ "flattenpath strokepath clip newpath",
+  /* "TGSM", */ "\ttgiforigctm setmatrix",
+  /* "TGRM", */ "\tsavematrix setmatrix",
+  NULL
+};
+
+void PSDontUseShortHand()
+{
+   if (preDumpSetup && psUseShortHand) {
+      char **ppsz=NULL;
+
+      psUseShortHand = FALSE;
+      psDictCount -= 1;
+      for (ppsz=gaszPsShortHand; *ppsz != NULL; ppsz++) {
+         psDictCount--;
+      }
+      gPsCmd = gaszPsLongHand;
+   }
+}
+
+void PSUseShortHand()
+{
+   if (preDumpSetup && !psUseShortHand) {
+      char **ppsz=NULL;
+
+      psUseShortHand = TRUE;
+      psDictCount += 1;
+      for (ppsz=gaszPsShortHand; *ppsz != NULL; ppsz++) {
+         psDictCount++;
+      }
+      gPsCmd = gaszPsShortHand;
+   }
+}
+
+#ifdef _PS_USE_EUC_FONT
+void PSUseEucFont()
+{
+   if (preDumpSetup && !psUseEucFont) {
+      psUseEucFont = TRUE;
+      psDictCount += 4;
+   }
+}
+#endif /* _PS_USE_EUC_FONT */
+
 static
 void CleanUpReEncodeInfo()
 {
@@ -192,7 +340,7 @@ void CleanUpReEncodeInfo()
 
    for ( ; topReEncodeInfo != NULL; topReEncodeInfo=p_next_reencode) {
       p_next_reencode = topReEncodeInfo->next;
-      free(topReEncodeInfo->font_name);
+      if (topReEncodeInfo->font_str != NULL) free(topReEncodeInfo->font_str);
       free(topReEncodeInfo);
    }
 }
@@ -204,22 +352,27 @@ void ResetPSInfo()
 
    needsTgifOrigCTM = TRUE;
 
-   psUsePSAdobe=TRUE;
+   psUsePSAdobe = TRUE;
 #ifdef INVERT_CTM_BUG
    UseMinRadius();
 #else /* ~INVERT_CTM_BUG */
    psUseMinRadius = FALSE;
 #endif /* INVERT_CTM_BUG */
-   psUseEllipse=FALSE;
-   psUseArrow=FALSE;
-   psUseArc=FALSE;
-   psUsePattern=FALSE;
-   psUseBWPattern=FALSE;
-   psUseColorImage=FALSE;
-   psUseColorPattern=FALSE;
-   psUseCenterText=FALSE;
-   psUseRightText=FALSE;
-   psUseReencode=FALSE;
+   psUseEllipse = FALSE;
+   psUseArrow = FALSE;
+   psUseArc = FALSE;
+   psUsePattern = FALSE;
+   psUseBWPattern = FALSE;
+   psUseColorImage = FALSE;
+   psUseColorPattern = FALSE;
+   psUseCenterText = FALSE;
+   psUseRightText = FALSE;
+   psUseMiniLines = FALSE;
+   psUseReencode = FALSE;
+   psUseShortHand = FALSE;
+#ifdef _PS_USE_EUC_FONT
+   psUseEucFont = FALSE;
+#endif /* _PS_USE_EUC_FONT */
    CleanUpReEncodeInfo();
 }
 
@@ -258,7 +411,7 @@ static char *psEllipseMacro[] = {
   "/tgifellipsedict 6 dict def",
   "tgifellipsedict /mtrx matrix put",
   "",
-  "/tgifellipse",
+  "/TGEL % tgifellipse",
   " { tgifellipsedict begin",
   "      /yrad exch def",
   "      /xrad exch def",
@@ -279,7 +432,7 @@ static char *psArrowMacro[] = {
   "/tgifarrowtipdict 8 dict def",
   "tgifarrowtipdict /mtrx matrix put",
   "",
-  "/tgifarrowtip",
+  "/TGAT % tgifarrowtip",
   " { tgifarrowtipdict begin",
   "      /dy exch def",
   "      /dx exch def",
@@ -304,7 +457,7 @@ static char *psArcMacro[] = {
   "/tgifarcdict 8 dict def",
   "tgifarcdict /mtrx matrix put",
   "",
-  "/tgifarcn",
+  "/TGAN % tgifarcn",
   " { tgifarcdict begin",
   "      /endangle exch def",
   "      /startangle exch def",
@@ -320,7 +473,7 @@ static char *psArcMacro[] = {
   "   end",
   " } def",
   "",
-  "/tgifarc",
+  "/TGAR % tgifarc",
   " { tgifarcdict begin",
   "      /endangle exch def",
   "      /startangle exch def",
@@ -534,7 +687,7 @@ static char *psColorPatternMacro[] = {
   "   /i i 1 add def",
   " } def",
   "",
-  "/tgifpatfill",
+  "/TGPF % tgifpatfill",
   " { tgifpatdict begin",
   "      /h exch def",
   "      /w exch def",
@@ -592,13 +745,25 @@ static char *psPatternMacro[] = {
 };
 
 static char *psCenterTextMacro[] = {
-  "/tgifcentertext { dup stringwidth pop 2 div neg 0 rmoveto } def",
+  "/TGCTX { dup stringwidth pop 2 div neg 0 rmoveto } def",
+  "",
+  NULL
+};
+
+static char *psMiniLinesMacro[] = {
+  "/TGMAX",
+  " { exch dup 3 1 roll exch dup 3 1 roll gt { pop } { exch pop } ifelse",
+  " } def",
+  "/TGMIN",
+  " { exch dup 3 1 roll exch dup 3 1 roll lt { pop } { exch pop } ifelse",
+  " } def",
+  "/TGSW { stringwidth pop } def",
   "",
   NULL
 };
 
 static char *psRightTextMacro[] = {
-  "/tgifrighttext { dup stringwidth pop neg 0 rmoveto } def",
+  "/TGRTX { dup stringwidth pop neg 0 rmoveto } def",
   "",
   NULL
 };
@@ -643,113 +808,265 @@ static char *psReencodeMacro[] = {
   NULL
 };
 
-void DumpPSMacro (FP)
-   FILE	* FP;
-{
-   register int	i;
+static char *psShortHandMacro[] = {
+  "/bd { bind def } bind def",
+  "",
+  NULL
+};
 
-   fprintf (FP, "\n");
+#ifdef _PS_USE_EUC_FONT
+static char *psEucFontMacro[] = {
+  "%%Title: eucfont.ps",
+  "% old eucfont.ps(OCFeucfont) and new eucfont.ps(CIDeucfont) were combined.",
+  "%  1996-09, kawamata@sra.co.jp",
+  "% Sample Usage:",
+  "%   Operands are only font names permitted. Don't put fontdict itself.",
+  "% /Courier-Ryumin /Courier /Ryumin-Light-EUC-V eucfont",
+  "",
+  "%%Title: CIDeucfont.ps",
+  "%%Creator: yohtaki@cis.ibaraki.ac.jp",
+  "%%CreationDate: 1996-9-29",
+  "%%EndComments",
+  "%",
+  "% <newfont> <asciifont> <kanjifont> CIDeucfont",
+  "/CIDeucfont {",
+  "dup findfont /WMode get 1 eq 4 1 roll % push direction info.",
+  "12 dict begin",
+  "  /CIDInit /ProcSet findresource begin",
+  "  exch [ 3 1 roll ] beginrearrangedfont",
+  "      3 -1 roll {   % pop direction info",
+  "         /WMode 1 def	% vertical font",
+#ifdef _VERT_FONT_OFFSET_POINT_4
+  "         0 beginusematrix [ 0 1 -1 0 0 0.4 ] endusematrix",
+#else /* ~_VERT_FONT_OFFSET_POINT_4 */
+  "         0 beginusematrix [ 0 1 -1 0 0 0.3 ] endusematrix",
+#endif /* _VERT_FONT_OFFSET_POINT_4 */
+  "      } if",
+  "      1 usefont",
+  "      1 beginbfrange",
+  "         <00> <7F> <00>",
+  "       endbfrange",
+  "     endrearrangedfont",
+  "  end",
+  "end",
+  "} def",
+  "",
+  "%%Title: OCFeucfont.ps",
+  "%%Creator: shikida@cs.titech.ac.jp, ueno@cs.titech.ac.jp",
+  "%%CreationDate: 1991-2-27",
+  "%%EndComments",
+  "% <font> OCFcopyfont <font>",
+  "/OCFcopyfont {",
+  "    dup maxlength 1 add dict begin",
+  "    {",
+  "	1 index /FID ne 2 index /UniqueID ne and",
+  "	{def} {pop pop} ifelse",
+  "    } forall",
+  "    currentdict",
+  "    end",
+  "} bind def",
+  "% <newfont> <asciifont> <kanjifont> OCFeucfont",
+  "/OCFeucfont {",
+  "      12 dict begin",
+  "	dup type /nametype eq { findfont } if",
+  "	dup /WMode known {",
+  "	    dup /WMode get /WMode exch def",
+  "	    WMode 1 eq {",
+#ifdef _VERT_FONT_OFFSET_POINT_4
+  "		[0.0 1.0 -1.0 0.0 0.0 0.40] makefont",
+#else /* ~_VERT_FONT_OFFSET_POINT_4 */
+  "		[0.0 1.0 -1.0 0.0 0.0 0.30] makefont",
+#endif /* _VERT_FONT_OFFSET_POINT_4 */
+  "	    } if",
+  "	} if",
+  "	OCFcopyfont dup begin",
+  "	    /Encoding Encoding",
+  "	    FMapType dup 2 eq {",
+  "		pop 128 128",
+  "	    } { 5 eq {",
+  "		256 256",
+  "	    } {",
+  "		/compositefont errordict /invalidfont get exec",
+  "	    } ifelse",
+  "	    } ifelse",
+  "	    getinterval def",
+  "	end",
+  "	/kanjifont exch definefont",
+  "	exch",
+  "",
+  "	dup type /nametype eq { findfont } if",
+  "	exch",
+  "",
+  "	/FDepVector [ 4 2 roll ] def",
+  "	/FontType 0 def",
+  "	/FMapType 4 def",
+  "	/FontMatrix matrix def",
+  "	/Encoding [ 0 1 ] def",
+  "	/FontBBox {0 0 0 0} def",
+  "	dup /FontName exch def",
+  "	currentdict",
+  "    end",
+  "    definefont pop",
+  "} def",
+  "",
+  "% <newfont> <asciifont> <kanjifont> eucfont",
+  "/eucfont {",
+  "	dup findfont",
+  "	/FMapType get 9 eq",
+  "		{ CIDeucfont}",
+  "		{ OCFeucfont} ifelse",
+  "} def",
+  "",
+  NULL
+};
+#endif /* _PS_USE_EUC_FONT */
+
+void DumpPSMacro(FP)
+   FILE *FP;
+{
+   register int i;
+
+   fprintf(FP, "\n");
    if (!psUsePSAdobe) {
-      for (i = 0; psAdobeMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psAdobeMacro[i]);
+      for (i=0; psAdobeMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psAdobeMacro[i]);
       }
    }
 
-   fprintf (FP, "/tgifdict %1d dict def\n", psDictCount);
-   fprintf (FP, "tgifdict begin\n");
-   fprintf (FP, "\n");
+   fprintf(FP, "/tgifdict %1d dict def\n", psDictCount);
+   fprintf(FP, "tgifdict begin\n");
+   fprintf(FP, "\n");
 
    if (psUseMinRadius) {
-      for (i = 0; psMinRadiusMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psMinRadiusMacro[i]);
+      for (i=0; psMinRadiusMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psMinRadiusMacro[i]);
       }
    }
    if (psUseEllipse) {
-      for (i = 0; psEllipseMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psEllipseMacro[i]);
+      for (i=0; psEllipseMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psEllipseMacro[i]);
       }
    }
    if (psUseArrow) {
-      for (i = 0; psArrowMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psArrowMacro[i]);
+      for (i=0; psArrowMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psArrowMacro[i]);
       }
    }
    if (psUseArc) {
-      for (i = 0; psArcMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psArcMacro[i]);
+      for (i=0; psArcMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psArcMacro[i]);
       }
    }
    if (psUseBWPattern) {
-      for (i = 0; psBWPatternMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psBWPatternMacro[i]);
+      for (i=0; psBWPatternMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psBWPatternMacro[i]);
       }
    }
    if (psUseColorImage) {
-      for (i = 0; psColorImageMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psColorImageMacro[i]);
+      for (i=0; psColorImageMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psColorImageMacro[i]);
       }
    }
    if (psUseColorPattern) {
-      for (i = 0; psColorPatternMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psColorPatternMacro[i]);
+      for (i=0; psColorPatternMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psColorPatternMacro[i]);
       }
    }
    if (psUsePattern) {
-      for (i = 0; psPatternMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psPatternMacro[i]);
+      for (i=0; psPatternMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psPatternMacro[i]);
       }
    }
    if (psUseCenterText) {
-      for (i = 0; psCenterTextMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psCenterTextMacro[i]);
+      for (i=0; psCenterTextMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psCenterTextMacro[i]);
       }
    }
    if (psUseRightText) {
-      for (i = 0; psRightTextMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psRightTextMacro[i]);
+      for (i=0; psRightTextMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psRightTextMacro[i]);
+      }
+   }
+   if (psUseMiniLines) {
+      for (i=0; psMiniLinesMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psMiniLinesMacro[i]);
       }
    }
    if (psUseReencode) {
-      for (i = 0; psReencodeMacro[i] != NULL; i++) {
-         fprintf (FP, "%s\n", psReencodeMacro[i]);
+      for (i=0; psReencodeMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psReencodeMacro[i]);
       }
    }
-}
-
-int ParsePsAdobeString (s, use_ps_adobe_string, adobe_str, epsf_str)
-   char	* s, * adobe_str, * epsf_str;
-   int	* use_ps_adobe_string;
-{
-   register char	* c_ptr=s, * c_ptr1;
-
-   if (!(strcmp ("false", c_ptr) == 0 || strcmp ("False", c_ptr) == 0))
-   {
-      if (*c_ptr >= '0' && *c_ptr <= '9')
-      {
-         *use_ps_adobe_string = TRUE;
-         for (c_ptr1 = c_ptr; *c_ptr1 != '\0'; c_ptr1++)
-            if (*c_ptr1 == '/')
-            {
-               *c_ptr1 = ' ';
-               break;
-            }
-         sscanf (c_ptr, "%s", adobe_str);
-         c_ptr = &c_ptr[strlen(adobe_str)];
-         switch (*c_ptr)
-         {
-            case '\0': break;
-            case ' ':
-               c_ptr++;
-               if (*c_ptr >= '0' && *c_ptr <= '9')
-                  sscanf (c_ptr, "%s", epsf_str);
-               else
-                  return (FALSE);
-               break;
-            default: return (FALSE);
+   if (psUseShortHand) {
+      for (i=0; psShortHandMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psShortHandMacro[i]);
+      }
+      for (i=0; gaszPsLongHand[i] != NULL; i++) {
+         if (*gaszPsLongHand[i] == '\t') {
+            fprintf(FP, "/%s { %s } def\n", &(gaszPsShortHand[i])[1],
+                  &(gaszPsLongHand[i])[1]);
+         } else {
+            fprintf(FP, "/%s { %s } bd\n", gaszPsShortHand[i],
+                  gaszPsLongHand[i]);
          }
       }
-      else
-         return (FALSE);
+      fprintf(FP, "\n");
+      gPsCmd = gaszPsShortHand;
+   } else {
+      gPsCmd = gaszPsLongHand;
    }
-   return (TRUE);
+#ifdef _PS_USE_EUC_FONT
+   if (psUseEucFont) {
+      for (i=0; psEucFontMacro[i] != NULL; i++) {
+         fprintf(FP, "%s\n", psEucFontMacro[i]);
+      }
+   }
+#endif /* _PS_USE_EUC_FONT */
 }
+
+int ParsePsAdobeString(buf, use_ps_adobe_string, adobe_str, epsf_str)
+   char *buf, *adobe_str, *epsf_str;
+   int *use_ps_adobe_string;
+{
+   if (UtilStrICmp("false", buf) == 0 || UtilStrICmp("no", buf) == 0) {
+      *use_ps_adobe_string = FALSE;
+   } else if (UtilStrICmp("true", buf) == 0 || UtilStrICmp("yes", buf) == 0) {
+      *use_ps_adobe_string = TRUE;
+   } else if (*buf >= '0' && *buf <= '9') {
+      char *c_ptr=buf, *c_ptr1=NULL;
+
+      *use_ps_adobe_string = TRUE;
+      for (c_ptr1=c_ptr; *c_ptr1 != '\0'; c_ptr1++) {
+         if (*c_ptr1 == '/') {
+            *c_ptr1 = ' ';
+            break;
+         }
+      }
+      sscanf(c_ptr, "%s", adobe_str);
+      c_ptr = &c_ptr[strlen(adobe_str)];
+      switch (*c_ptr) {
+      case '\0': break;
+      case ' ':
+         c_ptr++;
+         if (*c_ptr >= '0' && *c_ptr <= '9') {
+            sscanf(c_ptr, "%s", epsf_str);
+         } else {
+            return FALSE;
+         }
+         break;
+      default: return FALSE;
+      }
+   } else {
+      return FALSE;
+   }
+   return TRUE;
+}
+
+void CleanUpPS()
+{
+}
+
+void InitPS()
+{
+}
+
