@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.12 2000/01/15 22:24:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.13 2000/05/10 17:34:20 william Exp $
  */
 
 #define _INCLUDE_FROM_RASTER_C_
@@ -225,6 +225,7 @@
 #include "xbm/pdf.xbm"
 #include "xbm/tiffepsi.xbm"
 #include "xbm/png.xbm"
+#include "xbm/jpeg.xbm"
 
 #include "xbm/file.xbm"
 #include "xbm/landscap.xbm"
@@ -941,6 +942,8 @@ void InitPattern()
          mainWindow, (char *)tiffepsi_bits, tiffepsi_width, tiffepsi_height);
    whereToPrintPixmap[PNG_FILE] = XCreateBitmapFromData(mainDisplay,
          mainWindow, (char *)png_bits, png_width, png_height);
+   whereToPrintPixmap[JPEG_FILE] = XCreateBitmapFromData(mainDisplay,
+         mainWindow, (char *)jpeg_bits, jpeg_width, jpeg_height);
 
    shortLineTypePixmap[0] = XCreateBitmapFromData(mainDisplay, mainWindow,
          (char *)lt0s_bits, lt0s_width, lt0s_height);
