@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/rcbox.c,v 4.9 2000/01/15 22:24:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/rcbox.c,v 4.12 2000/05/01 20:19:08 william Exp $
  */
 
 #define _INCLUDE_FROM_RCBOX_C_
@@ -582,27 +582,12 @@ void ContinueRCBox(OrigX, OrigY)
 {
    int end_x, end_y, grid_x, grid_y, saved_x, saved_y;
    int done=FALSE, abort=FALSE;
-   int xor_pixel, radius=ZOOMED_SIZE(rcbRadius);
+   int radius=ZOOMED_SIZE(rcbRadius);
    char buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80];
-   XGCValues values;
    XEvent input, ev;
    XMotionEvent *motion_ev;
 
-   xor_pixel = xorColorPixels[colorIndex];
-
-   values.foreground = xor_pixel;
-   values.function = GXxor;
-   values.fill_style = FillSolid;
-#ifdef NO_THIN_LINE
-   values.line_width = 1;
-#else
-   values.line_width = 0;
-#endif
-   values.line_style = LineSolid;
-
-   XChangeGC(mainDisplay, drawGC,
-         GCForeground | GCFunction | GCFillStyle | GCLineWidth | GCLineStyle,
-         &values);
+   SetXorDrawGC(xorColorPixels[colorIndex]);
 
    saved_x = grid_x = OrigX;
    saved_y = grid_y = OrigY; 
@@ -613,16 +598,18 @@ void ContinueRCBox(OrigX, OrigY)
    sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
    StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
    BeginIntervalRulers(grid_x, grid_y, grid_x, grid_y);
-   XGrabPointer(mainDisplay, drawWindow, FALSE,
-         PointerMotionMask | ButtonReleaseMask,
-         GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
-   
+   if (!debugNoPointerGrab) {
+      XGrabPointer(mainDisplay, drawWindow, FALSE,
+            PointerMotionMask | ButtonReleaseMask,
+            GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
+   }
    SetRCBoxVertex(OrigX, OrigY, saved_x, saved_y, radius);
    while (!done) {
       XNextEvent(mainDisplay, &input);
 
       if (input.type == Expose || input.type == VisibilityNotify) {
          ExposeEventHandler(&input, TRUE);
+         SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if (input.type == ButtonRelease) {
          XUngrabPointer(mainDisplay, CurrentTime);
          MyRCBox(drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
