@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.26 2000/01/15 22:24:10 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.27 2000/04/15 19:19:08 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
@@ -704,6 +704,27 @@ void AddNewSelObj(ObjPtr)
    numObjSelected++;
 }
 
+void ExpandCurSelBBoxes(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   if (obj_ptr->bbox.ltx < selLtX) selLtX = obj_ptr->bbox.ltx;
+   if (obj_ptr->bbox.lty < selLtY) selLtY = obj_ptr->bbox.lty;
+   if (obj_ptr->bbox.rbx < selRbX) selRbX = obj_ptr->bbox.rbx;
+   if (obj_ptr->bbox.rby < selRbY) selRbY = obj_ptr->bbox.rby;
+   if (obj_ptr->obbox.ltx < selObjLtX) {
+      selObjLtX = obj_ptr->obbox.ltx;
+   }
+   if (obj_ptr->obbox.lty < selObjLtY) {
+      selObjLtY = obj_ptr->obbox.lty;
+   }
+   if (obj_ptr->obbox.rbx < selObjRbX) {
+      selObjRbX = obj_ptr->obbox.rbx;
+   }
+   if (obj_ptr->obbox.rby < selObjRbY) {
+      selObjRbY = obj_ptr->obbox.rby;
+   }
+}
+
 static
 void SetNoLockBBox(ObjPtr)
    struct ObjRec *ObjPtr;
