@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.46 2000/02/27 15:08:56 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.50 2000/05/10 19:31:26 william Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -233,6 +233,7 @@ int	myBlackPixel=INVALID;
 int	myLtGryPixel=INVALID;
 int	myDkGryPixel=INVALID;
 int	myVryLtPixel=INVALID;
+int	myYellowPixel=INVALID;
 int	reverseVideo = FALSE;
 
 char	drawPath[MAXPATHLENGTH]; /* last char is NOT '/' */
@@ -261,6 +262,8 @@ Atom	executeCmdByIDAtom=(Atom)0;
 
 int	curChoiceBeforeMakeQuiescent=NOTHING;
 
+int	noMinWinSize=TRUE;
+
 static Atom	protocolAtom=(Atom)0;
 
 static int	canvasWindowOnly=FALSE;
@@ -915,6 +918,8 @@ void Setup()
          whereToPrint = TIFFEPSI_FILE;
       } else if (UtilStrICmp(c_ptr, "PNG") == 0) {
          whereToPrint = PNG_FILE;
+      } else if (UtilStrICmp(c_ptr, "JPEG") == 0) {
+         whereToPrint = JPEG_FILE;
       } else {
          fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
                TOOL_NAME, "WhereToPrint", c_ptr, "EPS");
@@ -1437,7 +1442,34 @@ void Setup()
    if (*bootDir == '\0') {
       GetWorkingDirectory(bootDir, sizeof(bootDir));
    }
-   sizehints.flags = PPosition | PSize | PMinSize;
+   InitFonts();
+
+   initialMenubarWindowH = defaultFontHeight;
+   if (msgFontSet==NULL && msgFontPtr == NULL) {
+      statusSubWindowH[0] = defaultFontHeight+2+windowPadding;
+   } else {
+      statusSubWindowH[0] = msgFontHeight+2+windowPadding;
+   }
+   statusWindowH = max(statusSubWindowH[0]+(brdrW<<1)+2, btn1_height+2);
+   if (threeDLook) {
+      statusWindowH += windowPadding;
+   }
+   userRedrawWindowH = statusWindowH;
+/* chatWindowH = GetChatWindowHeight();
+   if (threeDLook) {
+      chatWindowH += windowPadding;
+   }
+ */
+   noMinWinSize = TRUE;
+   if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "NoMinWinSize")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
+      noMinWinSize = FALSE;
+   }
+   if (noMinWinSize) {
+      sizehints.flags = PSize;
+   } else {
+      sizehints.flags = PSize | PMinSize;
+   }
    sizehints.x = 0;
    sizehints.y = 0;
    sizehints.width = mainWinW;
@@ -1452,6 +1484,10 @@ void Setup()
          sizehints.flags |= USSize;
          drawWinW = sizehints.width;
          drawWinH = sizehints.height;
+         InitWinSizes();
+         mainWinW = sizehints.width;
+         mainWinH = sizehints.height;
+         InverseInitWinSizes();
       }
       if (bitmask & XNegative) {
          x_neg_in_def = TRUE;
@@ -1472,28 +1508,14 @@ void Setup()
          sizehints.flags |= USSize;
          drawWinW = sizehints.width;
          drawWinH = sizehints.height;
+         InitWinSizes();
+         mainWinW = sizehints.width;
+         mainWinH = sizehints.height;
+         InverseInitWinSizes();
       }
       if (bitmask & XValue) x_in_cmdline = TRUE;
       if (bitmask & YValue) y_in_cmdline = TRUE;
    }
-   InitFonts();
-
-   initialMenubarWindowH = defaultFontHeight;
-   if (msgFontSet==NULL && msgFontPtr == NULL) {
-      statusSubWindowH[0] = defaultFontHeight+2+windowPadding;
-   } else {
-      statusSubWindowH[0] = msgFontHeight+2+windowPadding;
-   }
-   statusWindowH = max(statusSubWindowH[0]+(brdrW<<1)+2, btn1_height+2);
-   if (threeDLook) {
-      statusWindowH += windowPadding;
-   }
-   userRedrawWindowH = statusWindowH;
-/* chatWindowH = GetChatWindowHeight();
-   if (threeDLook) {
-      chatWindowH += windowPadding;
-   }
- */
    InitWinSizes();
 
    if (canvasWindowOnly) {
@@ -1762,6 +1784,7 @@ void Setup()
    InitShortCut();
    InitHtml();
    InitRecentFiles();
+   InitMeasureTooltip();
 
 #ifdef MAPBEFORESELECT
    XMapWindow(mainDisplay, mainWindow);
@@ -1785,7 +1808,7 @@ void Setup()
       XMapWindow(mainDisplay, drawWindow); 
       XSelectInput(mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask | EnterWindowMask |
+            KeyReleaseMask | ExposureMask | EnterWindowMask | LeaveWindowMask |
             PropertyChangeMask);
    } else {
       XMapWindow(mainDisplay, titleWindow);
@@ -1816,7 +1839,7 @@ void Setup()
       XMapWindow(mainDisplay, drawWindow); 
       XSelectInput(mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask | EnterWindowMask |
+            KeyReleaseMask | ExposureMask | EnterWindowMask | LeaveWindowMask |
             PropertyChangeMask);
 
       if (colorLayers) {
@@ -1879,7 +1902,7 @@ void Setup()
    if (canvasWindowOnly) {
       XSelectInput(mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask | EnterWindowMask |
+            KeyReleaseMask | ExposureMask | EnterWindowMask | LeaveWindowMask |
             PropertyChangeMask);
       XMapWindow(mainDisplay, drawWindow); 
    } else {
@@ -1912,7 +1935,7 @@ void Setup()
 
       XSelectInput(mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask | EnterWindowMask |
+            KeyReleaseMask | ExposureMask | EnterWindowMask | LeaveWindowMask |
             PropertyChangeMask);
       XMapWindow(mainDisplay, drawWindow); 
 
