@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,17 +28,13 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/shape.c,v 3.20 1998/11/05 22:56:26 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shape.c,v 4.14 2000/01/15 22:24:11 william Exp $
  */
 
-#include <math.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_SHAPE_C_
+
+#include "tgifdefs.h"
+#include "cmdids.h"
 
 #include "arc.e"
 #include "attr.e"
@@ -54,8 +50,10 @@
 #include "group.e"
 #include "mainloop.e"
 #include "menu.e"
+#include "menuinfo.e"
 #include "move.e"
 #include "msg.e"
+#include "navigate.e"
 #include "obj.e"
 #include "oval.e"
 #include "page.e"
@@ -66,66 +64,71 @@
 #include "ruler.e"
 #include "select.e"
 #include "setup.e"
-#ifndef _NO_EXTERN
 #include "shape.e"
-#endif
 #include "spline.e"
+#include "strtbl.e"
 #include "text.e"
 #include "util.e"
 
-#define SHAPE_BOX 0
-#define SHAPE_PARALLEL 1
-#define SHAPE_TRAPEZOID 2
-#define SHAPE_RHOMBUS 3
-#define SHAPE_RCBOX 4
-#define SHAPE_OCTAGON 5
-#define SHAPE_CROSS 6
-#define SHAPE_OVAL 7
-#define SHAPE_WORDS 8
-#define SHAPE_HEXAGON 9
-#define SHAPE_TRIANGLE 10
-#define SHAPE_BLAST 11
-#define SHAPE_STAR 12
-#define SHAPE_DISK 13
-#define SHAPE_RIGHTARROW 14
-#define SHAPE_UPARROW 15
-#define SHAPE_FATRIGHTARROW 16
-#define SHAPE_FATUPARROW 17
-#define SHAPE_RIGHTTAB 18
-#define SHAPE_UPTAB 19
-
-#define MAXSHAPES 20
-
 int numShapes=MAXSHAPES;
 int curShape=(-1);
 int shapeShadowInResource=FALSE;
 int shapeShadowDx=0;
 int shapeShadowDy=0;
 
-static char *shapeMenuDesc[] = {
-      "Create a box",
-      "Create a parallelegram",
-      "Create a trapezoid",
-      "Create a rhombus",
-      "Create a rounded-corner box",
-      "Create an octagon",
-      "Create a cross",
-      "Create an oval",
-      "Create a word box",
-      "Create a hexagon",
-      "Create a triangle",
-      "Create a blast box",
-      "Create a star",
-      "Create a disk",
-      "Create a right arrow",
-      "Create a up arrow",
-      "Create a fat right arrow",
-      "Create a fat up arrow",
-      "Create a right tab",
-      "Create a up tab",
-      NULL
+/* ----------------------- String Functions ----------------------- */
+
+static struct MouseStatusStrRec shapeDescMouseStatus[] = {
+   /*
+    * note: the m field is used for the localized strings
+    */
+   { N_("Create a box"),                NULL, NULL },
+   { N_("Create a parallelegram"),      NULL, NULL },
+   { N_("Create a trapezoid"),          NULL, NULL },
+   { N_("Create a rhombus"),            NULL, NULL },
+   { N_("Create a rounded-corner box"), NULL, NULL },
+   { N_("Create an octagon"),           NULL, NULL },
+   { N_("Create a cross"),              NULL, NULL },
+   { N_("Create an oval"),              NULL, NULL },
+   { N_("Create a word box"),           NULL, NULL },
+   { N_("Create a hexagon"),            NULL, NULL },
+   { N_("Create a triangle"),           NULL, NULL },
+   { N_("Create a blast box"),          NULL, NULL },
+   { N_("Create a star"),               NULL, NULL },
+   { N_("Create a disk"),               NULL, NULL },
+   { N_("Create a right arrow"),        NULL, NULL },
+   { N_("Create a up arrow"),           NULL, NULL },
+   { N_("Create a fat right arrow"),    NULL, NULL },
+   { N_("Create a fat up arrow"),       NULL, NULL },
+   { N_("Create a right tab"),          NULL, NULL },
+   { N_("Create a up tab"),             NULL, NULL },
+   { NULL, NULL, NULL }
 };
 
+static
+void CleanUpShapeDescMouseStatusStrings()
+{
+   int index=0;
+
+   for (index=0; index < MAXCHOICES; index++) {
+      UtilFree(shapeDescMouseStatus[index].m);
+      shapeDescMouseStatus[index].m = NULL;
+   }
+}
+
+static
+char *GetShapeDesc(index)
+   int index;
+   /* returned string has been translated */
+{
+   if (shapeDescMouseStatus[index].m == NULL) {
+      shapeDescMouseStatus[index].m =
+            UtilStrDup(_(shapeDescMouseStatus[index].l));
+      if (shapeDescMouseStatus[index].m == NULL) FailAllocMessage();
+   }
+   return shapeDescMouseStatus[index].m;
+}
+
 /* ----------------------- Create Shape Functions ----------------------- */
 
 static IntPoint *gpVertices=NULL;
@@ -727,7 +730,7 @@ static
 struct AttrRec *GenerateInvisibleBox()
 {
    int w, h, saved_text_just=textJust, attr_w, attr_h, attr_ltx, attr_lty;
-   int saved_fill=objFill;
+   int saved_fill=objFill, saved_trans_pat=transPat;
    struct BBRec bbox;
    struct BoxRec *box_ptr;
    struct AttrRec *attr_ptr, *label_attr=NULL;
@@ -787,6 +790,7 @@ struct AttrRec *GenerateInvisibleBox()
    box_ptr->fill = NONEPAT;
    box_ptr->pen = NONEPAT;
    objFill = NONEPAT;
+   transPat = FALSE;
    AddAttrByNameAndValue(topObj, "", "auto_center_attr");
    attr_ptr = topObj->fattr;
    if (attr_ptr != NULL) {
@@ -801,6 +805,7 @@ struct AttrRec *GenerateInvisibleBox()
             attr_lty-attr_ptr->obj->obbox.lty);
    }
    AddAttrByNameAndValue(topObj, "label=", "");
+   transPat = saved_trans_pat;
    objFill = saved_fill;
    label_attr = FindAttrWithName(topObj, "label=", NULL);
    if (label_attr != NULL) {
@@ -861,7 +866,9 @@ void GenerateShadow()
    }
    botObj = obj_ptr;
 
+   ChangeObjTransPat(obj_ptr, NO_TRANSPAT_MODE);
    ChangeObjFill(obj_ptr, SOLIDPAT);
+   ChangeObjPen(obj_ptr, SOLIDPAT);
    MoveObj(obj_ptr, shapeShadowDx, shapeShadowDy);
 }
 
@@ -908,7 +915,8 @@ struct AttrRec *GenerateShape()
       CreateArcObj(gShapeBBox.ltx+(w>>1), gShapeBBox.rby-radius_y,
             gShapeBBox.ltx, gShapeBBox.rby-radius_y,
             gShapeBBox.ltx+w, gShapeBBox.rby-radius_y, ARC_CCW,
-            gShapeBBox.ltx, gShapeBBox.rby-diam_y, w, diam_y, 180<<6, 180<<6);
+            gShapeBBox.ltx, gShapeBBox.rby-diam_y, w, diam_y, 180<<6, 180<<6,
+            FALSE);
       ResetCreatePoly();
       AddPtToCreatePoly(gShapeBBox.ltx, gShapeBBox.lty+radius_y);
       AddPtToCreatePoly(gShapeBBox.ltx, gShapeBBox.rby-radius_y);
@@ -1015,8 +1023,7 @@ void ContinueCreateShape(orig_x, orig_y)
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
 
    if (!StartCreateShape(orig_x, orig_y)) {
-      sprintf(gszMsgBox, "%s failed.", shapeMenuDesc[curShape]);
-      Msg(gszMsgBox);
+      Msg(TgLoadString(STID_CREATE_A_SHAPE_FAILED));
       return;
    }
    HighLightCreateShape(grid_x, grid_y, GENERATE, &need_to_draw);
@@ -1099,9 +1106,9 @@ void DoCreateShape()
    unsigned int button;
    int mouse_x=0, mouse_y=0;
 
-   sprintf(gszMsgBox, "%s by dragging your mouse...", shapeMenuDesc[curShape]);
-   Msg(gszMsgBox);
-   SetMouseStatus(shapeMenuDesc[curShape], "Cancel", "Cancel");
+   Msg(TgLoadCachedString(CSTID_DRAG_MOUSE_CREATE_SHAPE_DOTS));
+   SetMouseStatus(GetShapeDesc(curShape), TgLoadCachedString(CSTID_CANCEL),
+         TgLoadCachedString(CSTID_CANCEL));
    button = DrawWindowLoop(&mouse_x, &mouse_y, cornerCursor, TRUE);
    if (button == Button1) {
       int grid_x=0, grid_y=0;
@@ -1109,8 +1116,7 @@ void DoCreateShape()
       GridXY(mouse_x, mouse_y, &grid_x, &grid_y);
       ContinueCreateShape(grid_x, grid_y);
    } else {
-      sprintf(gszMsgBox, "%s aborted.", shapeMenuDesc[curShape]);
-      Msg(gszMsgBox);
+      Msg(TgLoadCachedString(CSTID_CREATE_A_SHAPE_ABORTED));
    }
 }
 
@@ -1119,11 +1125,13 @@ void CreateShape()
 {
    int force_no_shadow=FALSE, saved_dx=shapeShadowDx, saved_dy=shapeShadowDy;
 
-   if (objFill == NONEPAT && (shapeShadowDx != 0 || shapeShadowDy != 0)) {
-      sprintf(gszMsgBox, "%s %s.\n\n%s?",
-            "You are attempting to create a shadowed shape",
-            "but the shape has no fill (the shadow will show through)",
-            "Would you like to create such a shape without a shadow");
+   MakeQuiescent();
+
+   if ((objFill == NONEPAT || (objFill == BACKPAT && transPat)) &&
+         (shapeShadowDx != 0 || shapeShadowDy != 0)) {
+      sprintf(gszMsgBox, TgLoadString(objFill==NONEPAT ?
+            STID_CREATE_SHADOW_SHAPE_NO_FILL :
+            STID_CREATE_SHADOW_SHAPE_TRAN_FILL));
       switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
       case MB_ID_YES: force_no_shadow = TRUE; break;
       case MB_ID_NO: break;
@@ -1133,7 +1141,6 @@ void CreateShape()
    if (force_no_shadow) {
       shapeShadowDx = shapeShadowDy = 0;
    }
-   MakeQuiescent();
    SaveStatusStrings();
    DoCreateShape();
    RestoreStatusStrings();
@@ -1147,42 +1154,7 @@ void CreateShape()
 
 void CleanUpShape()
 {
-}
-
-static
-int ParseShapeShadowSpec(spec)
-   char *spec;
-{
-   char *psz_dx=NULL, *psz_dy=NULL, *psz_dup_spec=NULL;
-   int dx, dy;
-
-   UtilTrimBlanks(spec);
-   if (*spec == '\0') return FALSE;
-   if ((psz_dup_spec=UtilStrDup(spec)) == NULL) return FailAllocMessage();
-
-   if ((psz_dx=strtok(psz_dup_spec, " ,\t\n\r")) == NULL ||
-         sscanf(psz_dx, "%d", &dx) != 1) {
-      sprintf(gszMsgBox, "Error in parsing '%s' for 2 values.", spec);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-      free(psz_dup_spec);
-      return FALSE;
-   }
-   psz_dy = strtok(NULL, " ,\t\n\r");
-   if (psz_dy == NULL) {
-      dy = dx;
-   } else {
-      if (sscanf(psz_dy, "%d", &dy) != 1) {
-         sprintf(gszMsgBox, "Error in parsing '%s' for 2 values.", spec);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         free(psz_dup_spec);
-         return FALSE;
-      }
-   }
-   shapeShadowDx = dx;
-   shapeShadowDy = dy;
-
-   free(psz_dup_spec);
-   return TRUE;
+   CleanUpShapeDescMouseStatusStrings();
 }
 
 void InitShape()
@@ -1192,7 +1164,7 @@ void InitShape()
    shapeShadowInResource = FALSE;
    shapeShadowDx = shapeShadowDy = 0;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"ShapeShadowSpec")) != NULL) {
-      if (ParseShapeShadowSpec(c_ptr)) {
+      if (ParseXYSpec(c_ptr, &shapeShadowDx, &shapeShadowDy)) {
          shapeShadowInResource = TRUE;
       }
    }
@@ -1209,20 +1181,68 @@ void ShapeSubMenu(nIndex)
    curShape = (-1);
 }
 
+void RefreshShapeMenu(menu)
+   TgMenu *menu;
+{
+}
+
+TgMenu *CreateShapeMenu(parent_menu, x, y, menu_info, status_str_xlated)
+   TgMenu *parent_menu;
+   int x, y;
+   TgMenuInfo *menu_info;
+   int status_str_xlated; /* ignored, always 0 */
+{
+   int i=0;
+   TgMenu *menu=NULL;
+   TgMenuInfo stMenuInfo;
+   TgMenuItemInfo *item_info=NULL;
+
+   memcpy(&stMenuInfo, menu_info, sizeof(TgMenuInfo));
+   stMenuInfo.items = (TgMenuItemInfo*)malloc(
+         (MAXSHAPES+1)*sizeof(TgMenuItemInfo));
+   if (stMenuInfo.items == NULL) FailAllocMessage();
+   memset(stMenuInfo.items, 0, (MAXSHAPES+1)*sizeof(TgMenuItemInfo));
+   for (item_info=stMenuInfo.items, i=0; i < MAXSHAPES; item_info++, i++) {
+      item_info->menu_str = (char*)(Pixmap*)(&shapePixmap[i]);
+      item_info->shortcut_str = NULL;
+      sprintf(gszMsgBox, "%s", GetShapeDesc(i));
+      item_info->status_str = UtilStrDup(gszMsgBox);
+      if (item_info->status_str == NULL) FailAllocMessage();
+      item_info->submenu_info = NULL;
+      item_info->cmdid = CMDID_CREATESHAPE;
+   }
+   stMenuInfo.items[MAXSHAPES].cmdid = INVALID;
+
+   /* the status_str has been translated in GetShapeDesc() */
+   menu = TgCreateMenuFromMenuInfo(parent_menu, x, y, &stMenuInfo, TRUE);
+   for (item_info=stMenuInfo.items, i=0; i < MAXSHAPES; item_info++, i++) {
+      UtilFree(item_info->status_str);
+   }
+   memset(stMenuInfo.items, 0, (MAXSHAPES+1)*sizeof(TgMenuItemInfo));
+   free(stMenuInfo.items);
+   stMenuInfo.items = NULL;
+   if (menu != NULL) {
+      menu->track_menubar = TRUE;
+      TgAdjustMenuGeometry(menu, choiceImageW, choiceImageH, 8);
+   }
+   return menu;
+}
+
 int ShapeMenu(X, Y, TrackMenubar)
    int X, Y, TrackMenubar;
 {
-   int index, *fore_colors, *valid, *init_rv;
+   int rc=INVALID;
+   TgMenu *menu=(shapeMenuInfo. create_proc)(NULL, X, Y, &shapeMenuInfo,
+         INVALID);
 
-   DefaultColorArrays(MAXSHAPES, &fore_colors, &valid, &init_rv, NULL);
-   free(valid);
    activeMenu = MENU_SHAPE;
-   index = PxMpMenuLoop(X, Y, choiceImageW, choiceImageH, 8, 3, MAXSHAPES,
-         fore_colors, shapePixmap, init_rv, shapeMenuDesc, SINGLECOLOR,
-         TrackMenubar);
+   if (menu != NULL) {
+      menu->track_menubar = TrackMenubar;
 
-   if (index >= 0) ShapeSubMenu(index);
-   return index;
+      rc = TgMenuLoop(menu);
+      TgDestroyMenu(menu, TRUE);
+   }
+   return rc;
 }
 
 /* ----------------------- Public Functions ----------------------- */
@@ -1232,11 +1252,11 @@ void SetShapeShadow()
    char spec[MAXSTRING+1];
 
    *spec = '\0';
-   sprintf(gszMsgBox, "%s: [current: %1d,%1d]",
-         "Please specify X and Y offsets", shapeShadowDx, shapeShadowDy);
+   sprintf(gszMsgBox, TgLoadString(STID_ENTER_XY_OFFSET_FOR_SHP_SHDW),
+         shapeShadowDx, shapeShadowDy);
    Dialog(gszMsgBox, NULL, spec);
-   if (ParseShapeShadowSpec(spec, FALSE)) {
-      sprintf(gszMsgBox, "Shape shadow offsets set to %1d and %1d.",
+   if (ParseXYSpec(spec, &shapeShadowDx, &shapeShadowDy)) {
+      sprintf(gszMsgBox, TgLoadString(STID_SHAPE_SHDW_XY_OFFSETS_SET_TO),
             shapeShadowDx, shapeShadowDy);
       Msg(gszMsgBox);
    }
