@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.57 2000/01/18 22:55:45 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.62 2000/05/20 22:31:36 william Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -374,12 +374,19 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "ReloadDomainInfoFromX()", 0, CMDID_RELOADDOMAININFOFROMX },
    { '\0', 0, "ImportPNGFile()", 0, CMDID_IMPORTPNGFILE },
    { '\0', 0, "EditIndexedAttrInEditor()", 0, CMDID_EDITATTRINEDITOR },
+   { '\0', 0, "GetProperty()", 0, CMDID_GETPROPERTY },
    { '\0', 0, "SetHtmlExportTemplate()", 0, CMDID_SETHTMLEXPORTTEMPLATE },
    { '\0', 0, "PrintPages()", 0, CMDID_PRINTPAGES },
    { '\0', 0, "GoHyperSpaceInSlideShow()",0, CMDID_GOHYPERSPACEINSLIDESHOW },
    { '\0', 0, "FreehandModeInSlideShow()",0, CMDID_FREEHANDMODEINSLIDESHOW },
    { '\0', 0, "OpenARecentlyUsedFile()", 0, CMDID_RECENTLYUSEDFILES },
    { '\0', 0, "MoveEditTextBox()", 0, CMDID_MOVEEDITTEXTBOX },
+   { '\0', 0, "ReplaceGraphic()", 0, CMDID_REPLACEGRAPHIC },
+   { '\0', 0, "ToggleShowMeasurementInTooltip()", 0,
+               CMDID_TOGGLESHOWMEASUREMENTINTT },
+   { '\0', 0, "ImportJPEGFile()", 0, CMDID_IMPORTJPEGFILE },
+   { '\0', 0, "ToggleAutoEPSPreviewBitmap()", 0,
+               CMDID_TOGGLEAUTOEPSPREVIEWBITMAP },
    { '\0', 0, "", 0, 0 }
 };
 
