@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.26 2000/01/15 22:24:13 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.27 2000/04/15 20:27:04 william Exp $
  */
 
 #define _INCLUDE_FROM_SPECIAL_C_
@@ -661,129 +661,137 @@ void UnMakeSymbolic()
    }
 }
 
-int MakeIconic(sym_path)
+int MakeIconic(sym_path, record_cmd)
    char *sym_path;
+   int record_cmd;
 {
    char icon_name[MAXPATHLENGTH], file_name[MAXPATHLENGTH];
    char icon_full_name[MAXPATHLENGTH], *rest=NULL, *psz=NULL;
    char sym_ext_str[MAXSTRING];
    FILE *fp=NULL;
-   int len, short_name, sym_ext_len, ltx, lty, rbx, rby;
+   int len, short_name, sym_ext_len, ltx, lty, rbx, rby, no_name=FALSE;
 
-   if (topSel!=NULL && topSel==botSel) {
-      int no_name=FALSE;
+   if (topSel == NULL || topSel != botSel) {
+      MsgBox(TgLoadString(STID_SEL_ONLY_ONE_FOR_MAKEICONIC), TOOL_NAME,
+            INFO_MB);
+      return FALSE;
+   }
+   if (sym_path == NULL) {
+      *icon_name = '\0';
+      Dialog(TgLoadString(STID_ENTER_NAME_FOR_THE_ICON), NULL, icon_name);
+      len = strlen(icon_name);
+      if (*icon_name == '\0') {
+         Msg(TgLoadString(STID_NAME_NOT_SPEC_ICON_NOT_CREATE));
+         return FALSE;
+      }
+   } else {
+      strncpy(icon_name, sym_path, sizeof(icon_name)-1);
+      icon_name[sizeof(icon_name)-1] = '\0';
+      len = strlen(icon_name);
+   }
+   sprintf(sym_ext_str, ".%s", SYM_FILE_EXT);
+   sym_ext_len = strlen(sym_ext_str);
 
-      if (sym_path == NULL) {
-         *icon_name = '\0';
-         Dialog(TgLoadString(STID_ENTER_NAME_FOR_THE_ICON), NULL, icon_name);
-         len = strlen(icon_name);
-         if (*icon_name == '\0') {
-            Msg(TgLoadString(STID_NAME_NOT_SPEC_ICON_NOT_CREATE));
-            return FALSE;
-         }
+   if (FileNameHasExtension(icon_name, OBJ_FILE_TYPE, NULL, NULL)) {
+      MsgBox(TgLoadString(STID_CANT_SAVE_OBJ_ICON_NOT_CREATE), TOOL_NAME,
+            INFO_MB);
+      return FALSE;
+   } else if (FileNameHasExtension(icon_name, SYM_FILE_TYPE, NULL, NULL)) {
+      strcpy(icon_full_name, icon_name);
+      if ((psz=UtilStrRChr(icon_name, '.')) != NULL) {
+         *psz = '\0';
       } else {
-         strncpy(icon_name, sym_path, sizeof(icon_name)-1);
-         icon_name[sizeof(icon_name)-1] = '\0';
-         len = strlen(icon_name);
+         TgAssert(FALSE, TgLoadString(STID_CANT_FIND_DOT_IN_ICON_NAME_MI),
+               NULL);
+         return FALSE;
       }
-      sprintf(sym_ext_str, ".%s", SYM_FILE_EXT);
-      sym_ext_len = strlen(sym_ext_str);
+   } else if (FileNameHasExtension(icon_name, PIN_FILE_TYPE, NULL, NULL)) {
+      MsgBox(TgLoadString(STID_CANT_SAVE_PIN_ICON_NOT_CREATE), TOOL_NAME,
+            INFO_MB);
+      return FALSE;
+   } else {
+      strcpy(icon_full_name, icon_name);
+      strcat(icon_full_name, sym_ext_str);
+   }
+   if (no_name) {
+      MsgBox(TgLoadString(STID_NO_FILE_NAME_FILE_NOT_SAVED), TOOL_NAME,
+            INFO_MB);
+      return FALSE;
+   }
+   if (sym_path == NULL) {
+      sprintf(file_name, "%s%c%s", curDir, DIR_SEP, icon_full_name);
+   } else {
+      strcpy(file_name, icon_full_name);
+   }
+   if (!OkayToCreateFile(file_name)) return FALSE;
+   if ((short_name=IsPrefix(bootDir, file_name, &rest))) ++rest;
 
-      if (FileNameHasExtension(icon_name, OBJ_FILE_TYPE, NULL, NULL)) {
-         MsgBox(TgLoadString(STID_CANT_SAVE_OBJ_ICON_NOT_CREATE), TOOL_NAME,
-               INFO_MB);
-         return FALSE;
-      } else if (FileNameHasExtension(icon_name, SYM_FILE_TYPE, NULL, NULL)) {
-         strcpy(icon_full_name, icon_name);
-         if ((psz=UtilStrRChr(icon_name, '.')) != NULL) {
-            *psz = '\0';
-         } else {
-            TgAssert(FALSE, TgLoadString(STID_CANT_FIND_DOT_IN_ICON_NAME_MI),
-                  NULL);
-            return FALSE;
-         }
-      } else if (FileNameHasExtension(icon_name, PIN_FILE_TYPE, NULL, NULL)) {
-         MsgBox(TgLoadString(STID_CANT_SAVE_PIN_ICON_NOT_CREATE), TOOL_NAME,
-               INFO_MB);
-         return FALSE;
-      } else {
-         strcpy(icon_full_name, icon_name);
-         strcat(icon_full_name, sym_ext_str);
-      }
-      if (no_name) {
-         MsgBox(TgLoadString(STID_NO_FILE_NAME_FILE_NOT_SAVED), TOOL_NAME,
-               INFO_MB);
-         return FALSE;
-      }
-      if (sym_path == NULL) {
-         sprintf(file_name, "%s%c%s", curDir, DIR_SEP, icon_full_name);
-      } else {
-         strcpy(file_name, icon_full_name);
-      }
-      if (!OkayToCreateFile(file_name)) return FALSE;
-      if ((short_name=IsPrefix(bootDir, file_name, &rest))) ++rest;
-
-      if ((fp=fopen(file_name, "w")) == NULL) {
-         sprintf(gszMsgBox, TgLoadString(STID_CANT_OPEN_FOR_WRITE_ICON_CRET),
-               (short_name ? rest : file_name));
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         return FALSE;
-      }
-      if (sym_path == NULL) {
-         if (!DirInSymPath(curDir)) UpdateSymInfo();
-      }
-      sprintf(gszMsgBox, TgLoadCachedString(CSTID_CREATING_FILE_DOTS),
+   if ((fp=fopen(file_name, "w")) == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANT_OPEN_FOR_WRITE_ICON_CRET),
             (short_name ? rest : file_name));
-      Msg(gszMsgBox);
-      SetStringStatus(gszMsgBox);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return FALSE;
+   }
+   if (sym_path == NULL) {
+      if (!DirInSymPath(curDir)) UpdateSymInfo();
+   }
+   sprintf(gszMsgBox, TgLoadCachedString(CSTID_CREATING_FILE_DOTS),
+         (short_name ? rest : file_name));
+   Msg(gszMsgBox);
+   SetStringStatus(gszMsgBox);
 
-      ltx = selLtX; lty = selLtY; rbx = selRbX; rby = selRbY;
+   ltx = selLtX; lty = selLtY; rbx = selRbX; rby = selRbY;
 
+   if (record_cmd) {
       PrepareToRecord(CMD_REPLACE, topSel, botSel, numObjSelected);
-      if (topSel->obj->type == OBJ_GROUP || topSel->obj->type == OBJ_SYM ||
-            topSel->obj->type == OBJ_ICON || topSel->obj->type == OBJ_PIN) {
-         JustMoveSelToTop();
-      } else {
-         GroupSingleObj(FALSE);
-         if (topSel->obj->fattr != NULL) {
-            Msg(TgLoadCachedString(CSTID_ATTR_PROMOTED_TO_NEW_ICON));
-         }
+   }
+   if (topSel->obj->type == OBJ_GROUP || topSel->obj->type == OBJ_SYM ||
+         topSel->obj->type == OBJ_ICON || topSel->obj->type == OBJ_PIN) {
+      JustMoveSelToTop();
+   } else {
+      GroupSingleObj(FALSE);
+      if (topSel->obj->fattr != NULL) {
+         Msg(TgLoadCachedString(CSTID_ATTR_PROMOTED_TO_NEW_ICON));
       }
+   }
 
-      topSel->obj->type = OBJ_SYM;
-      strcpy(topSel->obj->detail.r->s, icon_name);
-      topSel->obj->detail.r->rotate = ROTATE0;
-      topSel->obj->detail.r->flip = NO_FLIP;
-      topSel->obj->detail.r->deck_index = (-1);
+   topSel->obj->type = OBJ_SYM;
+   strcpy(topSel->obj->detail.r->s, icon_name);
+   topSel->obj->detail.r->rotate = ROTATE0;
+   topSel->obj->detail.r->flip = NO_FLIP;
+   topSel->obj->detail.r->deck_index = (-1);
 /*
  * Need to check for pins.
  * Need to handle the case where connection objects need to be dupped!
  */
-      topSel->obj->detail.r->pin_connected = 0;
-      topSel->obj->detail.r->first_conn = topSel->obj->detail.r->last_conn =
-            NULL;
+   topSel->obj->detail.r->pin_connected = 0;
+   topSel->obj->detail.r->first_conn = topSel->obj->detail.r->last_conn =
+         NULL;
 
+   writeFileFailed = FALSE;
+   Save(fp, topSel->obj, 0, 1);
+
+   if (writeFileFailed) {
       writeFileFailed = FALSE;
-      Save(fp, topSel->obj, 0, 1);
-
-      if (writeFileFailed) {
-         writeFileFailed = FALSE;
-         FailToWriteFileMessage(file_name);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         unlink(file_name);
-      } else {
-         sprintf(gszMsgBox, TgLoadCachedString(CSTID_NAMED_FILE_CREATED),
-               (short_name ? rest : file_name));
-         Msg(gszMsgBox);
-      }
-      fclose(fp);
+      FailToWriteFileMessage(file_name);
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      unlink(file_name);
+   } else {
+      sprintf(gszMsgBox, TgLoadCachedString(CSTID_NAMED_FILE_CREATED),
+            (short_name ? rest : file_name));
+      Msg(gszMsgBox);
+   }
+   fclose(fp);
 
+   if (record_cmd) {
       HighLightReverse();
-      topSel->obj->type = OBJ_ICON;
-      topSel->obj->id = objId++;
-      Msg(TgLoadString(STID_SEL_OBJ_IS_NOW_ICONIC));
-      AdjObjBBox(topSel->obj);
-      UpdSelBBox();
+   }
+   topSel->obj->type = OBJ_ICON;
+   topSel->obj->id = objId++;
+   Msg(TgLoadString(STID_SEL_OBJ_IS_NOW_ICONIC));
+   AdjObjBBox(topSel->obj);
+   UpdSelBBox();
+   if (record_cmd) {
       RecordCmd(CMD_REPLACE, NULL, topSel, botSel, numObjSelected);
       ltx = min(ltx,selLtX); lty = min(lty,selLtY);
       rbx = max(rbx,selRbX); rby = max(rby,selRbY);
@@ -792,10 +800,8 @@ int MakeIconic(sym_path)
       HighLightForward();
       SetFileModified(TRUE);
       justDupped = FALSE;
-      return TRUE;
    }
-   MsgBox(TgLoadString(STID_SEL_ONLY_ONE_FOR_MAKEICONIC), TOOL_NAME, INFO_MB);
-   return FALSE;
+   return TRUE;
 }
 
 void UnMakeIconic()
