@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.7 2000/01/15 22:24:13 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.8 2000/05/20 19:21:03 william Exp $
  */
 
 #define _INCLUDE_FROM_SPLINE_C_
@@ -209,6 +209,7 @@ XPoint *MakeSplinePolyVertex(N, XOff, YOff, NumVs, Vs)
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, (NumVs+1)*sizeof(XPoint));
       splineVs[0].x = ZOOMED_SIZE(Vs[0].x-x_off);
       splineVs[0].y = ZOOMED_SIZE(Vs[0].y-y_off);
       splineVs[1].x = ZOOMED_SIZE(Vs[1].x-x_off);
@@ -228,6 +229,7 @@ XPoint *MakeSplinePolyVertex(N, XOff, YOff, NumVs, Vs)
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, (max_n+1)*sizeof(XPoint));
       splineVs[0].x = mx1;
       splineVs[0].y = my1;
       *N = 1;
@@ -247,6 +249,7 @@ XPoint *MakeSplinePolyVertex(N, XOff, YOff, NumVs, Vs)
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, (max_n+1)*sizeof(XPoint));
       splineVs[0].x = mx1;
       splineVs[0].y = my1;
       *N = 1;
@@ -313,7 +316,8 @@ XPoint *MakeMultiSplinePolyVertex(N, Smooth, XOff, YOff, NumVs, Vs)
       FailAllocMessage();
       return NULL;
    }
-   for (i=0; i < segments; i++) msptr[i].vlist = NULL;
+   memset(msptr, 0, segments*sizeof(struct MultiSplineRec));
+
    start_index = 0;
    seg_index = 0;
    for (i=1; i <= NumVs-1; i++) {
@@ -328,6 +332,7 @@ XPoint *MakeMultiSplinePolyVertex(N, Smooth, XOff, YOff, NumVs, Vs)
    if (total > 0) total++;
    splineVs = (XPoint *)malloc((total+2)*sizeof(XPoint));
    if (splineVs == NULL) FailAllocMessage();
+   memset(splineVs, 0, (total+2)*sizeof(XPoint));
    xpptr = splineVs;
    for (i=0; i < segments; i++) {
       if (msptr[i].vlist != NULL) {
@@ -364,6 +369,7 @@ XPoint *MakeSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, 5*sizeof(XPoint));
       splineVs[0].x = ZOOMED_SIZE(Vs[0].x-x_off);
       splineVs[0].y = ZOOMED_SIZE(Vs[0].y-y_off);
       splineVs[1].x = ZOOMED_SIZE(Vs[1].x-x_off);
@@ -384,6 +390,7 @@ XPoint *MakeSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
       *N = 0;
       return splineVs;
    }
+   memset(splineVs, 0, (max_n+1)*sizeof(XPoint));
    splineVs[0].x = mx4;
    splineVs[0].y = my4;
    *N = 1;
@@ -423,6 +430,7 @@ XPoint *MakeDoubleSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, 5*sizeof(XPoint));
       x = ZOOMED_DOUBLE_SIZE(Vs[0].x-dx_off);
       y = ZOOMED_DOUBLE_SIZE(Vs[0].y-dy_off);
       splineVs[0].x = round(x);
@@ -450,6 +458,7 @@ XPoint *MakeDoubleSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
       *N = 0;
       return NULL;
    }
+   memset(splineDoubleVs, 0, (max_n+1)*sizeof(DoublePoint));
    splineDoubleVs[0].x = mx4;
    splineDoubleVs[0].y = my4;
    *N = 1;
@@ -471,6 +480,7 @@ XPoint *MakeDoubleSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
       *N = 0;
       return NULL;
    }
+   memset(splineVs, 0, ((*N)+2)*sizeof(XPoint));
    for (i=0; i < *N; i++) {
       splineVs[i].x = round(splineDoubleVs[i].x);
       splineVs[i].y = round(splineDoubleVs[i].y);
@@ -512,7 +522,7 @@ XPoint *MakeMultiSplinePolygonVertex(N, Smooth, XOff, YOff, NumVs, Vs)
    msptr = (struct MultiSplineRec *)malloc(num_hinge_points *
          sizeof(struct MultiSplineRec));
    if (msptr == NULL) FailAllocMessage();
-   for (i=0; i < num_hinge_points; i++) msptr[i].vlist = NULL;
+   memset(msptr, 0, num_hinge_points*sizeof(struct MultiSplineRec));
 
    for (i=0; i < NumVs; i++) {
       if (!Smooth[i]) {
@@ -521,6 +531,7 @@ XPoint *MakeMultiSplinePolygonVertex(N, Smooth, XOff, YOff, NumVs, Vs)
    }
    tmp_vs = (IntPoint*)malloc((NumVs+1)*sizeof(IntPoint));
    if (tmp_vs == NULL) FailAllocMessage();
+   memset(tmp_vs, 0, (NumVs+1)*sizeof(IntPoint));
    start_index = i;
    seg_index = 0;
 
@@ -549,6 +560,7 @@ XPoint *MakeMultiSplinePolygonVertex(N, Smooth, XOff, YOff, NumVs, Vs)
    if (total > 0) total++;
    splineVs = (XPoint*)malloc((total+2)*sizeof(XPoint));
    if (splineVs == NULL) FailAllocMessage();
+   memset(splineVs, 0, (total+2)*sizeof(XPoint));
    xpptr = splineVs;
    for (i=0; i < num_hinge_points; i++) {
       if (msptr[i].vlist != NULL) {
@@ -592,9 +604,11 @@ void OpenSetupMatrix(NumPts, Vs)
    }
    mtxInfo.mtx = (double**)malloc(NumPts*sizeof(double*));
    if (mtxInfo.mtx == NULL) FailAllocMessage();
+   memset(mtxInfo.mtx, 0, NumPts*sizeof(double*));
    for (i=0; i < NumPts; i++) {
       mtxInfo.mtx[i] = (double*)malloc(3*sizeof(double));
       if (mtxInfo.mtx[i] == NULL) FailAllocMessage();
+      memset(mtxInfo.mtx[i], 0, 3*sizeof(double));
    }
    mtxInfo.mtx[0][0] = mtxInfo.mtx[NumPts-1][2] = (double)0.0;
    mtxInfo.mtx[0][1] = mtxInfo.mtx[NumPts-1][1] = (double)theSum;
@@ -637,12 +651,17 @@ void FreeMtxInfo(NumPts)
 {
    register int i;
 
-   free(mtxInfo.x); mtxInfo.x = NULL;
-   free(mtxInfo.y); mtxInfo.y = NULL;
-   free(mtxInfo.dx); mtxInfo.dx = NULL;
-   free(mtxInfo.dy); mtxInfo.dy = NULL;
-   for (i=0; i < NumPts; i++) free(mtxInfo.mtx[i]);
-   free(mtxInfo.mtx); mtxInfo.mtx = NULL;
+   if (mtxInfo.x != NULL) free(mtxInfo.x);
+   if (mtxInfo.y != NULL) free(mtxInfo.y);
+   if (mtxInfo.dx != NULL) free(mtxInfo.dx);
+   if (mtxInfo.dy != NULL) free(mtxInfo.dy);
+   if (mtxInfo.mtx != NULL) {
+      for (i=0; i < NumPts; i++) {
+         if (mtxInfo.mtx[i] != NULL) free(mtxInfo.mtx[i]);
+      }
+      free(mtxInfo.mtx);
+   }
+   memset(&mtxInfo, 0, sizeof(struct MtxRec));
 }
 
 static
@@ -657,6 +676,8 @@ IntPoint *OpenControlPts(NumPts, N)
 
    v = (IntPoint*)malloc((((NumPts-2)<<1)+2)*sizeof(IntPoint));
    if (v == NULL) FailAllocMessage();
+   memset(v, 0, (((NumPts-2)<<1)+2)*sizeof(IntPoint));
+
    v[index].x = (int)(mtxInfo.x[0]);
    v[index].y = (int)(mtxInfo.y[0]);
    index++;
@@ -702,12 +723,14 @@ XPoint *MakeIntSplinePolyVertex(N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
    case 2:
       *CntrlVs = (IntPoint*)malloc((NumVs+1)*sizeof(IntPoint));
       if (*CntrlVs == NULL) FailAllocMessage();
+      memset(*CntrlVs, 0, (NumVs+1)*sizeof(IntPoint));
       splineVs = (XPoint*)malloc((NumVs+1)*sizeof(XPoint));
       if (splineVs == NULL) {
          FailAllocMessage();
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, (NumVs+1)*sizeof(XPoint));
       splineVs[0].x = ZOOMED_SIZE(Vs[0].x-x_off);
       splineVs[0].y = ZOOMED_SIZE(Vs[0].y-y_off);
       splineVs[1].x = ZOOMED_SIZE(Vs[1].x-x_off);
@@ -750,9 +773,11 @@ void ClosedSetupMatrix(NumPts, Vs)
    /* the first NumPts-2 rows have an extra column */
    mtxInfo.mtx = (double**)malloc(NumPts*sizeof(double*));
    if (mtxInfo.mtx == NULL) FailAllocMessage();
+   memset(mtxInfo.mtx, 0, NumPts*sizeof(double*));
    for (i=0; i < NumPts; i++) {
       mtxInfo.mtx[i] = (double*)malloc(4*sizeof(double));
       if (mtxInfo.mtx[i] == NULL) FailAllocMessage();
+      memset(mtxInfo.mtx[i], 0, 4*sizeof(double));
    }
    mtxInfo.mtx[0][0] = mtxInfo.mtx[NumPts-1][2] = (double)1.0;
    mtxInfo.mtx[0][1] = mtxInfo.mtx[NumPts-1][1] = (double)SUM_MINUS_2;
@@ -804,9 +829,11 @@ void DoubleClosedSetupMatrix(NumPts, Vs)
    /* the first NumPts-2 rows have an extra column */
    mtxInfo.mtx = (double**)malloc(NumPts*sizeof(double*));
    if (mtxInfo.mtx == NULL) FailAllocMessage();
+   memset(mtxInfo.mtx, 0, NumPts*sizeof(double*));
    for (i=0; i < NumPts; i++) {
       mtxInfo.mtx[i] = (double*)malloc(4*sizeof(double));
       if (mtxInfo.mtx[i] == NULL) FailAllocMessage();
+      memset(mtxInfo.mtx[i], 0, 4*sizeof(double));
    }
    mtxInfo.mtx[0][0] = mtxInfo.mtx[NumPts-1][2] = (double)1.0;
    mtxInfo.mtx[0][1] = mtxInfo.mtx[NumPts-1][1] = (double)SUM_MINUS_2;
@@ -965,6 +992,7 @@ IntPoint *ClosedControlPts(NumPts, N)
 
    v = (IntPoint*)malloc(((NumPts<<1)+2)*sizeof(IntPoint));
    if (v == NULL) FailAllocMessage();
+   memset(v, 0, ((NumPts<<1)+2)*sizeof(IntPoint));
    for (i=0; i<NumPts; i++) {
       v[index].x = (int)((weight*mtxInfo.x[i]+mtxInfo.x[(i+1) % NumPts])/half);
       v[index].y = (int)((weight*mtxInfo.y[i]+mtxInfo.y[(i+1) % NumPts])/half);
@@ -995,6 +1023,7 @@ DoublePoint *DoubleClosedControlPts(NumPts, N)
 
    v = (DoublePoint*)malloc(((NumPts<<1)+2)*sizeof(DoublePoint));
    if (v == NULL) FailAllocMessage();
+   memset(v, 0, ((NumPts<<1)+2)*sizeof(DoublePoint));
    for (i=0; i<NumPts; i++) {
       v[index].x = ((weight*mtxInfo.x[i]+mtxInfo.x[(i+1) % NumPts])/half);
       v[index].y = ((weight*mtxInfo.y[i]+mtxInfo.y[(i+1) % NumPts])/half);
@@ -1031,6 +1060,7 @@ XPoint *MakeIntSplinePolygonVertex(N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
          *N = 0;
          return (splineVs);
       }
+      memset(splineVs, 0, 5*sizeof(XPoint));
       splineVs[0].x = ZOOMED_SIZE(Vs[0].x-x_off);
       splineVs[0].y = ZOOMED_SIZE(Vs[0].y-y_off);
       splineVs[1].x = ZOOMED_SIZE(Vs[1].x-x_off);
@@ -1067,6 +1097,7 @@ XPoint *MakeDoubleIntSplinePolygonVertex(N, CntrlN, CntrlVs, XOff, YOff,
          *N = 0;
          return splineVs;
       }
+      memset(splineVs, 0, 5*sizeof(XPoint));
       x = ZOOMED_DOUBLE_SIZE(Vs[0].x-((double)x_off));
       y = ZOOMED_DOUBLE_SIZE(Vs[0].y-((double)y_off));
       splineVs[0].x = round(x);
@@ -1290,6 +1321,7 @@ void DumpMultiCurvedPolygonPoints(FP, Smooth, Curved, NumPts, V, Indent)
    }
    tmp_vs = (IntPoint*)malloc((NumPts+1)*sizeof(IntPoint));
    if (tmp_vs == NULL) FailAllocMessage();
+   memset(tmp_vs, 0, (NumPts+1)*sizeof(IntPoint));
 
    for (i=0; i < NumPts; i++) {
       if (!Smooth[i]) {
