@@ -29,7 +29,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.15 2000/02/21 17:02:51 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.17 2000/04/18 00:27:53 william Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -41,12 +41,15 @@
 #include "box.e"
 #include "cmd.e"
 #include "dialog.e"
+#include "drawing.e"
 #include "exec.e"
 #include "menu.e"
 #include "msg.e"
 #include "miniline.e"
+#include "move.e"
 #include "obj.e"
 #include "pattern.e"
+#include "rcbox.e"
 #include "select.e"
 #include "setup.e"
 #include "tangram2.e"
@@ -61,6 +64,12 @@ static TgMenuItemInfo tangram2MenuItemInfo[] = {
      CMDID_ABOUT_TANGRAM2 },
    { "Generate Markov Chain", NULL, "Generate Markov Chain", NULL,
      CMDID_TANGRAM2_GENERATE_CHAIN },
+   { "Start Simulator", NULL, "Start Simulation", NULL,
+     CMDID_TANGRAM2_START_SIMULATOR },
+   { "Simulation Step", NULL, "Simulation Step", NULL,
+     CMDID_TANGRAM2_SIMULATION_STEP },
+   { "End Simulator", NULL, "End Simulation", NULL,
+     CMDID_TANGRAM2_END_SIMULATOR },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo tangram2MenuInfo={ TGMUTYPE_TEXT, tangram2MenuItemInfo,
@@ -84,7 +93,10 @@ int InitTangram2()
 
 static struct ShortCutRec tangram2ShortCutXlateTbl[] = {
    { '\0', 0, "AboutTangram2()", 0, CMDID_ABOUT_TANGRAM2 },
-   { '\0', 0, "Tangram2GenerateChain()", 0, CMDID_TANGRAM2_GENERATE_CHAIN },
+   { '\0', 0, "Tangram2GenerateChain()", 1, CMDID_TANGRAM2_GENERATE_CHAIN },
+   { '\0', 0, "Tangram2StartSimulator()", 1, CMDID_TANGRAM2_START_SIMULATOR },
+   { '\0', 0, "Tangram2SimulationStep()", 0, CMDID_TANGRAM2_SIMULATION_STEP },
+   { '\0', 0, "Tangram2EndSimulator()", 0, CMDID_TANGRAM2_END_SIMULATOR },
    { '\0', 0, "", 0, 0 }
 };
 
@@ -161,7 +173,10 @@ int DoTangram2Cmd(nCmdId, args)
 {
    switch (nCmdId) {
    case CMDID_ABOUT_TANGRAM2: AboutTangram2(); break;
-   case CMDID_TANGRAM2_GENERATE_CHAIN: Tangram2GenerateChain(); break;
+   case CMDID_TANGRAM2_GENERATE_CHAIN:  Tangram2GenerateChain(args);  break;
+   case CMDID_TANGRAM2_START_SIMULATOR: Tangram2StartSimulator(args); break;
+   case CMDID_TANGRAM2_SIMULATION_STEP: Tangram2SimulationStep();     break;
+   case CMDID_TANGRAM2_END_SIMULATOR:   Tangram2EndSimulator();       break;
    default: break;
    }
    return BAD;
@@ -222,7 +237,7 @@ ExecInfo *Tangram2GetExecInfo(func_name)
 
 #define MAXVARS              20
 #define MAXVARNAME          250
-#define MAXCONNECTTRIES      10
+#define MAXCONNECTTRIES       3
 #define PORT               6743
 
 #define END_OF_SIMULATION     1
@@ -244,6 +259,9 @@ typedef struct tagT_Packet {
 } T_Packet;
 
 static int simulator_socket=INVALID;
+static struct ObjRec *objGC=NULL; /* Generate Chain   button */
+static struct ObjRec *objSS=NULL; /* Start Simulation button */
+static struct ObjRec *objSp=NULL; /* Simulation Step  button */
 
 static
 int BadAttr_Simulator(attr_name, cmd_name)
@@ -262,55 +280,122 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
    struct ObjRec *obj_ptr;
    /* start_simulator(); */
 {
-   char servidor[80];
-   struct sockaddr_in endereco;
-   struct hostent *hptr;
-   int i;
-   
-   memset((char*)(&endereco), 0, sizeof(endereco)); 
+    char host[ 10 ];
+    char *protocol = "udp";
+    struct hostent     *phe;
+    struct servent     *pse;
+    struct protoent    *ppe;
+    struct sockaddr_in  sin;
+    int sin_size;
+    int s, type, max_desc;
+    char buffer[ sizeof( T_Packet ) ];
+    fd_set soc_pool;
+    struct timeval timeout;
+    int one = 1;
+    char *service = "6743";
 
-   /* get the local host information - use localhost (127.0.0.1)  */
-   /* gethostname(servidor, 80); */
-   strcpy(servidor, "localhost");
-   if ((hptr=gethostbyname(servidor)) != NULL) {
-      memcpy((char*)&endereco.sin_addr, hptr->h_addr, hptr->h_length); 
-   } else {
-      perror("gethostbyname");
-      return;
-   }
-
-   endereco.sin_family = AF_INET;
-   endereco.sin_port = htons((u_short)PORT);
-
-   if ((simulator_socket=socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-      perror("socket creation");
-      return; 
-   }
-
-   /* connect to server. Try for MAXCONNECTTRIES times */
-   i = 0;
-   while (i < MAXCONNECTTRIES)
-      if ((connect(simulator_socket, (struct sockaddr *)(&endereco),
-            sizeof(endereco))) < 0) {
-         perror("connect");
-         printf("Still trying... (%d)\n",MAXCONNECTTRIES-i);
-         close(simulator_socket);
-         if ((simulator_socket=socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-           perror("socket creation");
-           return; 
-         }
-         sleep(3);
-         i++;
-      } else {
-         break;
-      }
-   if (i == MAXCONNECTTRIES) {
-      perror("connect");
-      printf("Bailing out... after %d tries\n", MAXCONNECTTRIES);
-      return;
-   }
-
-   printf("Cliente Conectado.\n");
+    strcpy( host, "localhost" );
+    bzero( (char *)&sin, sizeof( sin ) );
+    sin.sin_family = AF_INET;
+    if( (pse = getservbyname( service, protocol )) != NULL )
+        sin.sin_port = pse->s_port;
+    else
+        if( (sin.sin_port = htons( (u_short)atoi( service ) )) == 0 )
+        {
+            fprintf( stderr, "TGIF: Can't get \"%s\" service entry\n", service );
+            return;
+        }
+    
+    if( (phe = gethostbyname( host )) != NULL )
+        bcopy( phe->h_addr, (char *)&sin.sin_addr, phe->h_length );
+    else
+    {
+        sin.sin_addr.s_addr = inet_addr( host );
+#ifdef linux
+        if( sin.sin_addr.s_addr == INADDR_NONE )
+        {
+            fprintf( stderr, "TGIF: Can't get \"%s\" host entry\n", host );
+            return;
+        }
+#endif /* linux */
+    }
+        
+    if( (ppe = getprotobyname( protocol )) == NULL )
+    {
+        fprintf( stderr, "TGIF: Can't get \"%s\" protocol entry\n", protocol );
+        return;
+    }
+    if( strcmp( protocol, "udp" ) == 0 )
+        type = SOCK_DGRAM;
+    else
+        type = SOCK_STREAM;
+        
+    if( (s = socket( PF_INET, type, ppe->p_proto )) < 0 )
+    {
+        fprintf( stderr, "TGIF: Can't create socket\n" );
+        return;
+    }
+    
+    if( setsockopt( s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof( one ) ) < 0 )
+    {
+        perror( "TGIF: REUSEADDR" );
+        return;
+    }
+    
+    if( bind( s, (struct sockaddr *)&sin, sizeof( sin ) ) < 0 )
+    {
+        perror( "TGIF: bin" );
+        return;
+    }
+    
+    if( type == SOCK_STREAM && listen( s, 0 ) < 0 )
+    {
+        fprintf( stderr, "TGIF: Can't listen on %s port\n", service );
+        return;
+    }
+    
+    FD_ZERO( &soc_pool );
+    FD_SET( s, &soc_pool );
+    max_desc = s + 1;
+    timeout.tv_sec  = 20;
+    timeout.tv_usec = 0;
+    if( select( max_desc, &soc_pool, NULL, NULL, &timeout ) < 0 )
+    {
+        perror( "TGIF: select" );
+        return;
+    }
+    if( FD_ISSET( s, &soc_pool ) )
+    {
+        sin_size = sizeof( sin );
+        if( recvfrom( s, buffer, sizeof( T_Packet ), 0, (struct sockaddr *)&sin, &sin_size ) < 0 )
+        {
+            perror( "TGIF: recv" );
+            return;
+        }
+        
+        if( connect( s, (struct sockaddr *)&sin, sin_size ) < 0 )
+        {
+            fprintf( stderr, "TGIF: Can't connect to %s.%s\n", host, service );
+            return;
+        }
+    
+        sleep( 1 );
+        if( send( s, buffer, sizeof( T_Packet ), 0 ) < 0 )
+        {
+            perror( "TGIF: send" );
+            return;
+        }
+    
+    }
+    else
+    {
+        fprintf( stderr, "TGIF: Timeout expired\n" );
+        return;
+    }
+    
+    simulator_socket = s;
+    
+    printf("TGIF: Cliente Conectado.\n");
 }
 
 void ExecEndSimulator(obj_ptr, orig_cmd)
@@ -574,35 +659,24 @@ int Tangram2Menu(X, Y, TrackMenubar)
    return rc;
 }
 
-/* ======================= Tangram2 Menu Functions ======================= */
+/* ========================= Tangram2 Menu Data ========================= */
 
-void AboutTangram2()
-{
-   sprintf(gszMsgBox, "%s\n\n%s %s.",
-         "Welcome to Tangram-II!",
-         "Brought to you by the Tangram-II team at the",
-         "Universidade Federal do Rio de Janeiro");
-   MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-}
 
-static char *GenerateChainExec[] = {
-  "disable_undo();",
+static char *GenerateChainInit[] = {
   "get_current_file(\"str\");",
   "read_file_into_attr(\"| basename $(str) .obj\", \"basename\");",
-  "read_file_into_attr(\"| head -1 $(basename).setup\", \"dbglevel\");",
-  "read_file_into_attr(\"| head -2 $(basename).setup | tail -1\", \"maxstates\");",
-  "read_file_into_attr(\"| tail -2 $(basename).setup | head -1\", \"output\");",
-  "read_file_into_attr(\"| tail -1 $(basename).setup\", \"processID\");",
   "strcpy(file_name, \"$(basename).parser\");",
   "strcpy(str, \" \");",
   "write_attr_into_file(str, $(file_name));",
+  NULL
+};
+
+static char *GenerateChainExec[] = {
+  "disable_undo();",
+  "exec(init);",
   "exec(write_model);",
   "exec(write_global_rewards);",
   "exec(write_indep_chains);",
-  "strcpy(command, \"gramatica $(file_name) $(basename) $(dbglevel) > $(output)\");",
-  "launch(command);",
-  "strcpy(command, \"generate_chain $(basename) $(maxstates) $(dbglevel) >> $(output)\");",
-  "launch(command);",
   "enable_undo();",
    NULL
 };
@@ -717,6 +791,159 @@ static char *GenerateChainWriteObj[] = {
   NULL
 };
 
+
+static char *StartSimulatorSetModeB[] = {
+  "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) B $(__GENERATE_CHAIN__.dbglevel) $(Event) $(Transitions) $(Time) $(Runs) >> $(__GENERATE_CHAIN__.output) & \");",
+  "launch(command);",
+  NULL
+};
+
+static char *StartSimulatorSetModeI[] = {
+ "strcpy(Transitions, \"0\");",
+ "strcpy(Time, \"0.0\");",
+ "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) I $(__GENERATE_CHAIN__.dbglevel) >> $(__GENERATE_CHAIN__.output) & \");",
+ "launch(command);",
+ "start_simulator();",
+  NULL
+};
+
+static char *StartSimulatorErrorMode[] = {
+ "message_box(NULL,\"Please, Mode has to be I(Interactive) or B(Batch)!\",\"Mode Error\", \"stop\");",
+  NULL
+};
+
+static char *StartSimulatorExec[] = {
+ "disable_undo();",
+ "show_attr(Transitions);",
+ "show_attr(Time);",
+ "exec(__GENERATE_CHAIN__.init);",
+ "if(\"$(Mode)\"!=\"I\"&&\"$(Mode)\"!=\"B\",error_mode,NULL);"
+ "strcpy(file_name, \"$(__GENERATE_CHAIN__.file_name)\");",
+ "strcpy(str, \" \");",
+ "write_attr_into_file(str, $(file_name));",
+ "exec(__GENERATE_CHAIN__.write_model);"
+ "exec(__GENERATE_CHAIN__.write_global_rewards);"
+ "exec(__GENERATE_CHAIN__.write_indep_chains);",
+ "strcpy(command, \"gramatica $(file_name) $(__GENERATE_CHAIN__.basename) $(__GENERATE_CHAIN__.dbglevel) -nomaxcheck > $(__GENERATE_CHAIN__.output)\");",
+ "launch(command);",
+ "if(\"$(Mode)\"==\"I\",set_mode_I,NULL);",
+ "if(\"$(Mode)\"==\"B\",set_mode_B,NULL);",
+ "enable_undo();",
+ NULL
+};
+
+static char *SimulationStepWriteVarLine[] = {
+ "tokenize(state_var_tok,$(state_var_line),'=');",
+ "get_line_in_attr(state_var_name,state_var_tok,1);",
+ "get_line_in_attr(state_var_value,state_var_tok,2);",
+ "strcpy(new_line, \"$(cur_obj).$(state_var_name)=$(state_var_value)\");",
+ "strcpy(buffer,\"$(buffer)$(new_line)@\");",
+  NULL
+};
+
+static char *SimulationStepLoopAnimation[] = {
+ "for_i(Ani_C_Step,1,$(Ani_T_Steps),1,Do_Animation);",
+ "round(sleep_time,$(Time_Scale)*$(step_time)*1000);",
+ "sleep(NULL,$(sleep_time));",
+  NULL
+};
+
+static char *SimulationStepAnimateObj[] = {
+ "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
+ "exec(\"$(cur_obj).Animation\");",
+  NULL
+};
+
+static char *SimulationStepDoAnimation[] = {
+ "find_obj_names(obj_list,\"\",\"Animation=*\");",
+ "get_line_in_attr(no_objs,obj_list,0);",
+ "for_i(obj_index,1,$(no_objs),1,Animate_Obj);",
+  NULL
+};
+
+static char *SimulationStepCallSimulator[] = {
+  "simulate_step();",
+  NULL
+};
+
+static char *SimulationStepPlayOneStep[] = {
+ "set_allow_interrupt(FALSE);",
+ "exec(step_simulation);",
+ "set_allow_interrupt(TRUE);",
+ "sleep(NULL,$(Delay));",
+  NULL
+};
+
+static char *SimulationStepUpdateObjects[] = {
+ "tokenize(state_var_list,$(buffer),'@');",
+ "get_line_in_attr(no_state_vars,state_var_list,0);",
+ "for_i(var_index,1,\"$(no_state_vars)-1\",1,update_vars);",
+  NULL
+};
+
+static char *SimulationStepStop[] = {
+  "stop();",
+  NULL
+};
+
+static char *SimulationStepStepSimulation[] = {
+ "strcpy(buffer,\"\");",
+ "find_obj_names(obj_list,\"\",\"State_vars=*\");",
+ "get_line_in_attr(no_objs,obj_list,0);",
+ "for_i(obj_index,1,$(no_objs),1,parse_obj);",
+ "exec(call_simulator);",
+ "exec(update_objects);",
+ "redraw_drawing_area();",
+ "if(\"$(Animation)\"==\"ON\",Loop_Animation,NULL);",
+  NULL
+};
+
+static char *SimulationStepPlaySimulation[] = {
+  "while(1,play_one_step);",
+  NULL
+};
+
+static char *SimulationStepUpdateVars[] = {
+ "get_line_in_attr(state_var_str,state_var_list,$(var_index));",
+ "tokenize(state_var_tok,$(state_var_str),'.');",
+ "get_line_in_attr(obj_name,state_var_tok,1);",
+ "get_line_in_attr(state_var_str,state_var_tok,2);",
+ "tokenize(state_var_tok,$(state_var_str),'=');",
+ "get_line_in_attr(state_var_name,state_var_tok,1);",
+ "get_line_in_attr(state_var_value,state_var_tok,2);",
+ "append_line_into_attr($(obj_name).State_vars,\" $(state_var_name)=$(state_var_value)\");",
+  NULL
+};
+
+static char *SimulationStepParseObj[] = {
+ "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
+ "get_number_of_lines_in_attr(no_state_vars,$(cur_obj).State_vars);",
+ "for_i(var_index,1,$(no_state_vars)-1,1,write_vars);",
+ "clear_attr($(cur_obj).State_vars);",
+  NULL
+};
+
+static char *SimulationStepExec[] = {
+ "disable_undo();",
+ "show_attr(Step);",
+ "show_attr(Delay);",
+ "show_attr(Animation);",
+ "show_attr(Time_Scale);",
+ "if(\"$(Step)\"==\"0\",play_simulation,step_simulation);",
+ "enable_undo();",
+ "redraw_drawing_area();",
+  NULL
+};
+
+static char *SimulationStepWriteVars[] = {
+ "get_line_in_attr(state_var_line,$(cur_obj).State_vars,$(var_index));",
+ "trim(state_var_line);",
+ "strlen(line_size, $(state_var_line));",
+ "if(\"$(line_size)\">=\"3\",write_var_line,NULL);",
+  NULL
+};
+
+
 static
 void AppendLinesToAttr(attr_ptr, ppsz_lines)
    struct AttrRec *attr_ptr;
@@ -738,25 +965,23 @@ void AppendLinesToAttr(attr_ptr, ppsz_lines)
    UpdTextBBox(attr_ptr->obj);
 }
 
-void Tangram2GenerateChain()
+/* ======================= Tangram2 Auxiliar Functions ======================= */
+
+struct ObjRec *createGenerateChainButton( dbglevel, maxstates, output )
+char *dbglevel;
+char *maxstates;
+char *output;
 {
-   int saved_fill=objFill, saved_pen=penPat;
    struct ObjRec *tmp_box_obj=NULL;
-   struct AttrRec *exec_attr=NULL;
 
-   MakeQuiescent(); /* select nothing, edit nothing */
-
-   objFill = penPat = NONEPAT;
    CreateBoxObj(0,0,10,10,TRUE);
-   objFill = saved_fill;
-   penPat = saved_pen;
    tmp_box_obj = topObj;
 
    /* Note: AddAttrByNameAndValue() adds the new object at tmp_box_obj->fattr */
    AddAttrByNameAndValue(tmp_box_obj, "i=", "");
-   AddAttrByNameAndValue(tmp_box_obj, "dbglevel=", "");
-   AddAttrByNameAndValue(tmp_box_obj, "maxstates=", "");
-   AddAttrByNameAndValue(tmp_box_obj, "output=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "dbglevel=", dbglevel);
+   AddAttrByNameAndValue(tmp_box_obj, "maxstates=", maxstates);
+   AddAttrByNameAndValue(tmp_box_obj, "output=", output);
    AddAttrByNameAndValue(tmp_box_obj, "processID=", "");
    AddAttrByNameAndValue(tmp_box_obj, "name=", "__GENERATE_CHAIN__");
    AddAttrByNameAndValue(tmp_box_obj, "basename=", "");
@@ -769,6 +994,9 @@ void Tangram2GenerateChain()
    AddAttrByNameAndValue(tmp_box_obj, "max_file_name=", "");
    AddAttrByNameAndValue(tmp_box_obj, "obj_list=", "");
 
+   AddAttrByNameAndValue(tmp_box_obj, "init=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, GenerateChainInit);
+
    AddAttrByNameAndValue(tmp_box_obj, "exec=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, GenerateChainExec);
 
@@ -799,12 +1027,234 @@ void Tangram2GenerateChain()
    AddAttrByNameAndValue(tmp_box_obj, "append_indep_chains=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, GenerateChainAppendIndepChains);
 
-   StartCompositeCmd();
+   AddNewSelObj(tmp_box_obj);
+
+   return( tmp_box_obj );
+}
+
+
+struct ObjRec *createStartSimulatorButton( runs, event, trans, time, mode )
+char *runs;
+char *event;
+char *trans;
+char *time;
+char *mode;
+{
+   struct ObjRec *tmp_box_obj = NULL;
+   int y = 5;
+
+   CreateRCBoxObj(4,y,200,y+40,TRUE);
+   tmp_box_obj = topObj;
+
+   tmp_box_obj->color = 9;
+   tmp_box_obj->detail.rcb->width = 2;
+   y += 5;
+
+   AddAttrByNameAndValue(tmp_box_obj, "Runs=", runs);
+   AddAttrByNameAndValue(tmp_box_obj, "Event=", event);
+   AddAttrByNameAndValue(tmp_box_obj, "Transitions=", trans);
+   tmp_box_obj->fattr->shown = TRUE;
+   MoveObj(tmp_box_obj->fattr->obj, 20, y - tmp_box_obj->fattr->obj->obbox.lty);
+   y += 14;
+
+   AddAttrByNameAndValue(tmp_box_obj, "Time=", time);
+   tmp_box_obj->fattr->shown = TRUE;
+   MoveObj(tmp_box_obj->fattr->obj, 20, y - tmp_box_obj->fattr->obj->obbox.lty);
+   y += 14;
+
+   AddAttrByNameAndValue(tmp_box_obj, "Mode=", mode);
+   AddAttrByNameAndValue(tmp_box_obj, "file_name=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "command=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "simulating=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "str=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "name=", "__START_SIMULATOR__");
+
+   AddAttrByNameAndValue(tmp_box_obj, "set_mode_B=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, StartSimulatorSetModeB);
+
+   AddAttrByNameAndValue(tmp_box_obj, "set_mode_I=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, StartSimulatorSetModeI);
+
+   AddAttrByNameAndValue(tmp_box_obj, "error_mode=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, StartSimulatorErrorMode);
+
+   AddAttrByNameAndValue(tmp_box_obj, "exec=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, StartSimulatorExec);
 
    AdjObjBBox(tmp_box_obj);
-   RecordNewObjCmd();
 
    AddNewSelObj(tmp_box_obj);
+   RedrawDrawWindow(botObj);
+
+   return( tmp_box_obj );
+}
+
+struct ObjRec *createSimulationStepButton( time, anim, delay, step )
+char *time;
+char *anim;
+char *delay;
+char *step;
+{
+   struct ObjRec *tmp_box_obj = NULL;
+
+   CreateBoxObj(0,0,10,10,TRUE);
+   tmp_box_obj = topObj;
+
+   /* Note: AddAttrByNameAndValue() adds the new object at tmp_box_obj->fattr */
+   AddAttrByNameAndValue(tmp_box_obj, "Time_Scale=", time);
+   AddAttrByNameAndValue(tmp_box_obj, "Animation=", anim);
+   AddAttrByNameAndValue(tmp_box_obj, "Delay=", delay);
+   AddAttrByNameAndValue(tmp_box_obj, "Step=", step);
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_str=", "1");
+   AddAttrByNameAndValue(tmp_box_obj, "line_size=", "7");
+   AddAttrByNameAndValue(tmp_box_obj, "obj_list=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "sleep_time=", "2049");
+   AddAttrByNameAndValue(tmp_box_obj, "step_time=", "2049");
+   AddAttrByNameAndValue(tmp_box_obj, "name=", "__SIMULATION_STEP__");
+   AddAttrByNameAndValue(tmp_box_obj, "cur_obj=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "obj_index=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "no_objs=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "Ani_T_Steps=", "10");
+   AddAttrByNameAndValue(tmp_box_obj, "Ani_C_Step=", "11");
+   AddAttrByNameAndValue(tmp_box_obj, "step_index=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "no_state_vars=", "3");
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_value=", "8");
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_name=", "8");
+   AddAttrByNameAndValue(tmp_box_obj, "obj_name=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "new_line=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "buffer=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_line=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_list=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "state_var_tok=", "");
+   AddAttrByNameAndValue(tmp_box_obj, "var_index=", "");
+
+   AddAttrByNameAndValue(tmp_box_obj, "write_var_line=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepWriteVarLine);
+
+   AddAttrByNameAndValue(tmp_box_obj, "Loop_Animation=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepLoopAnimation);
+
+   AddAttrByNameAndValue(tmp_box_obj, "Animate_Obj=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepAnimateObj);
+
+   AddAttrByNameAndValue(tmp_box_obj, "Do_Animation=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepDoAnimation);
+
+   AddAttrByNameAndValue(tmp_box_obj, "call_simulator=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepCallSimulator);
+
+   AddAttrByNameAndValue(tmp_box_obj, "play_one_step=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepPlayOneStep);
+
+   AddAttrByNameAndValue(tmp_box_obj, "update_objects=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepUpdateObjects);
+
+   AddAttrByNameAndValue(tmp_box_obj, "stop=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepStop);
+
+   AddAttrByNameAndValue(tmp_box_obj, "step_simulation=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepStepSimulation);
+
+   AddAttrByNameAndValue(tmp_box_obj, "play_simulation=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepPlaySimulation);
+
+   AddAttrByNameAndValue(tmp_box_obj, "update_vars=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepUpdateVars);
+
+   AddAttrByNameAndValue(tmp_box_obj, "parse_obj=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepParseObj);
+
+   AddAttrByNameAndValue(tmp_box_obj, "exec=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepExec);
+
+   AddAttrByNameAndValue(tmp_box_obj, "write_vars=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepWriteVars);
+
+   AddNewSelObj(tmp_box_obj);
+
+   return( tmp_box_obj );
+}
+
+/******************************************************************************/
+/* This function extracts the args that are comma separated in 'args' and     */
+/* stores them in 'dst'.                                                      */
+/* return 0 if failed, no_args if success.                                    */
+/* arguments:                                                                 */
+/*    args: comma separated list of tokens                                    */
+/*    dst : char vector pointer where the tokens will be stored               */
+/*    no_args: number of tokens in 'args'                                     */
+/*    max_arg_length: max token's size                                        */
+/******************************************************************************/
+int parseArgs( args, dst, no_args, max_arg_length )
+char *args;
+char *dst;
+int no_args;
+int max_arg_length;
+{
+    int i, j;
+    char *ptr;
+
+    bzero( dst, no_args * max_arg_length );    
+    ptr = args;
+    /* Remove leading " and spaces */
+    while( *ptr == '\"' || *ptr == ' ' )
+        ptr++;
+        
+    for( i = 0; i < no_args; i++ )
+    {
+        j = 0;
+        while( *ptr != '\0' && *ptr != ';' && *ptr != '\"' &&
+               j < max_arg_length )
+            *(dst + max_arg_length * i + j++) = *ptr++;
+            
+        if( *ptr == ';' || *ptr == '\"' )
+            ptr++;
+        else
+            if( *ptr != '\0' )
+                break;
+    }
+    
+    return( i == no_args );
+}
+/* ======================= Tangram2 Menu Functions ======================= */
+
+void AboutTangram2()
+{
+   sprintf(gszMsgBox, "%s\n\n%s %s.",
+         "Welcome to Tangram-II!",
+         "Brought to you by the Tangram-II team at the",
+         "Universidade Federal do Rio de Janeiro");
+   MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+}
+
+/******************************************************************************/
+/* This function creates a invisible button, executes this button and removes */
+/* it. This function also needs to extract three arguments that are colapsed  */
+/* in arg. This is done because tgif supports only one argument functions.    */
+/******************************************************************************/
+void Tangram2GenerateChain( args )
+char *args; /* semi-colon separated list (arg1;arg2;arg3) */
+{
+   int saved_fill=objFill, saved_pen=penPat;
+   struct ObjRec *tmp_box_obj=NULL;
+   struct AttrRec *exec_attr=NULL;
+   char arg[3][1024];
+
+   /* Extracts parameters from args */
+   if( !parseArgs( args, arg, 3, 1024 ) )
+   {
+        fprintf( stderr, "TGIF: Usage Tangram2GenerateChain( \"arg1;arg2;arg3\" )\n" );
+        return;
+   }
+
+   MakeQuiescent(); /* select nothing, edit nothing */
+
+   objFill = penPat = NONEPAT;
+
+   tmp_box_obj = createGenerateChainButton( arg[0], arg[1], arg[2] );
+
+   objFill = saved_fill;
+   penPat = saved_pen;
 
    exec_attr = FindAttrWithName(topSel->obj, "exec=", NULL);
    if (exec_attr != NULL) {
@@ -818,8 +1268,59 @@ void Tangram2GenerateChain()
    RemoveAllSel();
    RecordCmd(CMD_DELETE, NULL, NULL, NULL, 0);
 
-   EndCompositeCmd();
+}
+
+void Tangram2StartSimulator( args )
+char *args;
+{
+   int saved_fill=objFill, saved_pen=penPat;
+   struct AttrRec *exec_attr=NULL;
+   char arg[12][1024]; /* first 3 for generate_chain */
+
+   /* Extracts parameters from args */
+   if( !parseArgs( args, arg, 12, 1024 ) )
+   {
+        return;
+   }
+
+   MakeQuiescent(); /* select nothing, edit nothing */
+   objFill = penPat = NONEPAT;
+
+   objGC = createGenerateChainButton( arg[0], arg[1], arg[2] );
+   objSp = createSimulationStepButton( arg[8], arg[9], arg[10], arg[11] );
+
+   objFill = saved_fill;
+   penPat = saved_pen;
+   objSS = createStartSimulatorButton( arg[3], arg[4], arg[5], arg[6], arg[7] );
+
+   exec_attr = FindAttrWithName(objSS, "exec=", NULL);
+   if (exec_attr != NULL) {
+      DoExecLoop(objSS, exec_attr);
+   }
+}
+
+void Tangram2SimulationStep()
+{
+   struct AttrRec *exec_attr=NULL;
+   
+   exec_attr = FindAttrWithName(objSp, "exec=", NULL);
+   if (exec_attr != NULL) {
+      DoExecLoop(objSp, exec_attr);
+   }
+}
+
+void Tangram2EndSimulator()
+{
+   ExecEndSimulator( NULL, NULL );
+
+   /* Delete all created buttons */
+   MakeQuiescent();
+   AddNewSelObj(objGC);
+   AddNewSelObj(objSS);
+   AddNewSelObj(objSp);
+   DelAllSelObj();
+
+   RedrawDrawWindow(botObj);
 }
 
 #endif /* _TANGRAM_II */
-
