@@ -1,11 +1,11 @@
 /*
- * Code in this file is derived from the public domain code in
+ * Some code in this file is derived from the public domain code in
  *              WWW/Library/Implementation/HTTCP.c distributed with lynx-2.2,
  *              whose original author is Tim Berners-lee <timbl@info.cern.ch>.
  *
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -22,7 +22,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -32,40 +32,17 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/tcp.c,v 3.9 1998/11/05 22:56:29 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tcp.c,v 4.7 2000/01/15 22:24:18 william Exp $
  */
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <signal.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#include <fcntl.h>
-#ifdef isc /* SunSoft/Interactive UNIX */
-#include <net/errno.h>
-#include <sys/bsdtypes.h>
-#endif
+#define _INCLUDE_FROM_TCP_C_
 
-#ifndef O_NONBLOCK
-#include <sys/ioctl.h>
-#endif /* ~O_NONBLOCK */
-
-#if !defined(FD_SET)
-#include <sys/select.h>
-#endif /* !defined(FD_SET) */
-
-#include "const.h"
+#include "tgifdefs.h"
 
+#include "msg.e"
 #include "remote.e"
-#ifndef _NO_EXTERN
+#include "strtbl.e"
 #include "tcp.e"
-#endif /* !_NO_EXTERN */
 #include "util.e"
 
 void SetSocketBlockingState(pn_socket, n_blocking)
@@ -83,8 +60,9 @@ void SetSocketBlockingState(pn_socket, n_blocking)
    rc = ioctl(*pn_socket, FIONBIO, &val);
 #endif /* O_NONBLOCK */
    if (rc == (-1)) {
-      fprintf(stderr, "Could not make connection %s.\n",
-            (n_blocking ? "blocking" : "non-blocking"));
+      fprintf(stderr, "%s\n",
+            TgLoadString(n_blocking ? STID_FAIL_TO_MAKE_SOCKET_BLOCK :
+            STID_FAIL_TO_MAKE_SOCKET_NON_BLOCK));
    }
 }
 
@@ -94,7 +72,7 @@ void TcpFreeBuf(buf)
    free(buf);
 }
 
-static gnPipeBroken=FALSE;
+static int gnPipeBroken=FALSE;
 
 static
 void BrokenPipe(nSig)
@@ -125,7 +103,6 @@ int TcpDoConnect(psz_host, us_port, pn_socket)
    } else {
       p_hostent = gethostbyname(psz_host);
       if (p_hostent == NULL) {
-         fprintf(stderr, "Cannot find Internet node name.\n");
          return TG_REMOTE_STATUS_HOST;
       }
       memcpy(&sin->sin_addr, p_hostent->h_addr, p_hostent->h_length);
@@ -138,11 +115,11 @@ int TcpDoConnect(psz_host, us_port, pn_socket)
 
    status = connect(*pn_socket, (struct sockaddr*)&soc_address,
          sizeof(soc_address));
-#ifdef SVR4
+#ifdef EAGAIN
    if ((status < 0) && (errno==EINPROGRESS || errno==EAGAIN)) {
-#else /* ~SVR4 */
+#else /* ~EAGAIN */
    if ((status < 0) && (errno == EINPROGRESS)) {
-#endif /* SVR4 */
+#endif /* EAGAIN */
       struct timeval timeout;
       int rc=0;
 
@@ -166,23 +143,25 @@ int TcpDoConnect(psz_host, us_port, pn_socket)
             status = connect(*pn_socket, (struct sockaddr*)&soc_address,
                   sizeof(soc_address));
             if (gnPipeBroken) {
-               fprintf(stderr, "Broken pipe while contacting '%s'.\n",
+               fprintf(stderr, TgLoadString(STID_BROKEN_PIPE_CONTACT_HOST),
                      psz_host);
+               fprintf(stderr, "\n");
             }
             if ((status < 0)&&(errno == EISCONN)) status = TG_REMOTE_STATUS_OK;
-            if (errno == EALREADY)
+            if (errno == EALREADY) {
                rc = 0;
-            else
+            } else {
                break;
+            }
          } else {
             status = connect(*pn_socket, (struct sockaddr*)&soc_address,
                   sizeof(soc_address));
-#ifdef SVR4
+#ifdef EAGAIN
             if ((status < 0) && (errno != EALREADY) && (errno != EISCONN) &&
                   (errno != EAGAIN))
-#else /* ~SVR4 */
+#else /* ~EAGAIN */
             if ((status < 0) && (errno != EALREADY) && (errno != EISCONN))
-#endif /* SVR4 */
+#endif /* EAGAIN */
                break;
          }
          if (UserAbortComm()) {
@@ -211,9 +190,10 @@ int TcpDoWrite(n_socket, buf, buf_sz)
    status = write(n_socket, buf, (int)buf_sz);
    if (status <= 0) {
       if (status == 0) {
-         fprintf(stderr, "Get status 0 on initial network write.\n");
+         fprintf(stderr, "%s\n", TgLoadString(STID_WRITE_TO_SOCKET_FAILED));
       } else if ((errno == ENOTCONN || errno == ECONNRESET || errno == EPIPE)) {
-         fprintf(stderr, "Unexpected network write error.\n");
+         fprintf(stderr, "%s\n",
+               TgLoadString(STID_UNEXP_NETWORK_ERR_WRITE_SOCK));
          return TG_REMOTE_STATUS_WRITE;
       }
    }
@@ -232,7 +212,7 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
    if (buf == NULL) {
-      fprintf(stderr, "Memory allocation failed.\n");
+      FailAllocMessage();
       return TG_REMOTE_STATUS_MEM;
    }
    do {
@@ -241,7 +221,7 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
       if (buf_sz - len < MIN_READ_SIZE) {
          buf_sz += 0x400;
          if ((buf=(char*)realloc(buf, buf_sz)) == NULL) {
-            fprintf(stderr, "Memory allocation failed.\n");
+            FailAllocMessage();
             return TG_REMOTE_STATUS_MEM;
          }
       }
@@ -250,11 +230,13 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
          if (bytes_read < 0 && (errno == ENOTCONN || errno == ECONNRESET ||
                errno == EPIPE)) {
             free(buf);
-            fprintf(stderr, "Network read error.\n");
+            fprintf(stderr, "%s\n",
+                  TgLoadString(STID_READ_FROM_SOCKET_FAILED));
             return TG_REMOTE_STATUS_READ;
          } else if (bytes_read < 0) {
             free(buf);
-            fprintf(stderr, "Network error.\n");
+            fprintf(stderr, "%s\n",
+                  TgLoadString(STID_UNEXP_NETWORK_ERR_READ_SOCK));
             return TG_REMOTE_STATUS_NET;
          }
          end_of_file = TRUE;
@@ -264,7 +246,7 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    } while (!end_of_file);
    buf[len] = '\0';
    *ppsz_buf = buf;
-   if (pn_buf_sz != NULL) *pn_buf_sz = (len+1);
+   if (pn_buf_sz != NULL) *pn_buf_sz = len;
    return TG_REMOTE_STATUS_OK;
 }
 
