@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,13 +28,12 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/testdrive.c,v 3.8 1998/11/05 22:56:30 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/testdrive.c,v 4.13 2000/01/15 22:24:18 william Exp $
  */
 
-#include <stdio.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_TESTDRIVE_C_
+
+#include "tgifdefs.h"
 
 #include "mainloop.e"
 #include "msg.e"
@@ -45,9 +44,9 @@
  * extern int	malloc_debug ARGS_DECL((int));
  */
 
-int	lastFile;
-short	*pDrawFontAsc;
-short	*pDrawFontDes;
+int	lastFile=TRUE;
+short	*pDrawFontAsc=NULL;
+short	*pDrawFontDes=NULL;
 
 /*
  * static
@@ -98,9 +97,10 @@ void PrintObjId(ObjPtr, Level)
    case OBJ_GROUP: printf("group: %1d\n", id); break;
    case OBJ_SYM: printf("sym: %1d\n", id); break;
    case OBJ_ICON: printf("icon: %1d\n", id); break;
+   case OBJ_PIN: printf("pin: %1d\n", id); break;
    }
    if (ObjPtr->type == OBJ_GROUP || ObjPtr->type == OBJ_SYM ||
-         ObjPtr->type == OBJ_ICON) {
+         ObjPtr->type == OBJ_ICON || ObjPtr->type == OBJ_PIN) {
       /* for these composite objects, each on maintains a LIST of objects */
       for (obj_ptr=ObjPtr->detail.r->last; obj_ptr != NULL;
             obj_ptr=obj_ptr->prev) {
@@ -121,6 +121,77 @@ void PrintObjId(ObjPtr, Level)
    }
 }
 
+#define DO_CMD(cmd) ExecACommandFromBuffer(cmd,NULL)
+
+static
+void ScriptDemo()
+{
+   MakeQuiescent(); /* select nothing, edit nothing */
+
+   if (BeginExecCommandsFromBuffer()) {
+      int i=0;
+
+      /* set current color to 'blue' */
+      DO_CMD("set_selected_obj_color(\"blue\")");
+      /* create a 200 pixels by 100 pixels blue box */
+      DO_CMD("create_box_obj(100,100,300,200)");
+      /* select the top object, which is the box object */
+      DO_CMD("select_top_obj()");
+      /* name the box object "the_box" */
+      DO_CMD("add_attr_to_selected_obj(\"name\",\"the_box\",100,100)");
+      /* hide all the attributes of the box object */
+      DO_CMD("call_simple_shortcut(\"HideAttr\")");
+      /* unselect everything */
+      DO_CMD("unselect_all_obj()");
+
+      /* set current text justification to center justified */
+      DO_CMD("set_selected_text_just(\"center\")");
+      /* set current text font to Helvetica Bold */
+      DO_CMD("set_selected_text_font(\"Helvetica-Bold\")");
+      /* set current text size to 14 */
+      DO_CMD("set_selected_text_size(14)");
+      /* create a text object that sits on top the blue box */
+      DO_CMD("create_text_obj(200,100,\"My Box\")");
+
+      /* select the box object */
+      DO_CMD("select_obj_by_name(\"the_box\")");
+      /* sleep for 500ms */
+      DO_CMD("sleep(NULL,500)");
+      /* change the box to 'green' */
+      DO_CMD("set_selected_obj_color(\"green\")");
+
+      /* cycle through all the fill patterns */
+      for (i=31; i >= 0; i--) {
+         char buf[80];
+
+         sprintf(buf, "set_selected_obj_fill(%1d)", i);
+         DO_CMD(buf);
+      }
+      /* sleep for another 500ms */
+      DO_CMD("sleep(NULL,500)");
+      /* change the box to 'red' */
+      DO_CMD("set_selected_obj_color(\"red\")");
+      /* sleep for another 500ms */
+      DO_CMD("sleep(NULL,500)");
+
+      /* lock the box */
+      DO_CMD("call_simple_shortcut(\"Lock\")");
+
+      /* select the top object, which is the text object */
+      DO_CMD("select_top_obj()");
+      /* also select the box */
+      DO_CMD("select_additional_obj(\"the_box\")");
+      /* align the text object so that it sits in the middle of the box */
+      DO_CMD("call_simple_shortcut(\"AlignObjsMiddle\")");
+      /* unlock the box */
+      DO_CMD("call_simple_shortcut(\"UnLock\")");
+      /* unselect everything */
+      DO_CMD("unselect_all_obj()");
+
+      EndExecCommandsFromBuffer();
+   }
+}
+
 int main(argc, argv)
    int argc;
    char *argv[];
@@ -128,7 +199,7 @@ int main(argc, argv)
    /*    Prolog's foreign function interface. */
 {
    register int i;
-   char op_name[80], file_name[80];
+   char op_name[80], file_name[(MAXPATHLENGTH<<1)+1];
    char *sp[6], *func_strp;
 /*
  * char	color_name[80], val_name[80];
@@ -139,7 +210,7 @@ int main(argc, argv)
  * malloc_debug (1);
  */
 
-   if (!ProcessTgifOptions(argc, argv, file_name)) return 1;
+   if (!ProcessTgifOptions(argc, argv, file_name, sizeof(file_name))) return 1;
 
    if (file_name[0] == '\0') {
       MainLoop("init", "", &func_strp,
@@ -183,6 +254,8 @@ int main(argc, argv)
             PrintObjId(obj_ptr, 0);
          }
          printf("\n");
+      } else if (strcmp(s, "Anim") == 0) {
+         ScriptDemo();
       }
 
       Msg("Returned from basic driver.");
