@@ -1,29 +1,34 @@
 %
 % Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
 %
-% Copyright (C) 1992-1993, William Cheng.
+% Copyright (C) 1992-2000, William Chia-Wei Cheng.
 %
-% Permission limited to the use, copy, modify, and distribute this software
-% and its documentation for any purpose is hereby granted by the Author without
-% fee, provided that the above copyright notice appear in all copies and
-% that both the copyright notice and this permission notice appear in
-% supporting documentation, and that the name of the Author not be used
-% in advertising or publicity pertaining to distribution of the software
-% without specific, written prior permission.  The Author makes no
-% representations about the suitability of this software for any purpose.
-% It is provided "as is" without express or implied warranty.  All other
-% rights (including the right to sell "tgif" and the right to sell derivative
-% works of tgif) are reserved by the Author.
+% Permission limited to the use, copy, display, distribute without
+% charging for a fee, and produce derivative works of "tgif" and
+% its documentation for not-for-profit purpose is hereby granted by
+% the Author, provided that the above copyright notice appears in
+% all copies made of "tgif" and that both the copyright notice
+% and this permission notice appear in supporting documentation,
+% and that the name of the Author not be used in advertising or
+% publicity pertaining to distribution of the software without
+% specific, written prior permission.  The Author makes no
+% representations about the suitability of this software for any
+% purpose.  It is provided "as is" without express or implied
+% warranty.  All other rights (including, but not limited to, the
+% right to sell "tgif", the right to sell or distribute derivative
+% works of "tgif", the right to distribute "tgif" for a fee, and
+% the right to include "tgif" or derivative works of "tgif" in a
+% for-sale product or service) are reserved by the Author.
 %
 % THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
-% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
-% EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
-% CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
-% USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
-% OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-% PERFORMANCE OF THIS SOFTWARE.
+% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
+% IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT
+% OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+% LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+% NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
+% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 %
-% @(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/testdrive.pl,v 3.0 1996/05/06 16:12:12 william Exp $
+% @(#)$Header: /mm/src/tgif/v4/RCS/testdrive.pl,v 4.3 2000/01/16 15:22:29 william Exp $
 %
 
 % ?- compile(testdrive).
