@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1998, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2000, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -18,7 +18,7 @@
  * right to sell "tgif", the right to sell or distribute derivative
  * works of "tgif", the right to distribute "tgif" for a fee, and
  * the right to include "tgif" or derivative works of "tgif" in a
- * for-sale product) are reserved by the Author.
+ * for-sale product or service) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
@@ -28,16 +28,16 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/tgif.c,v 3.17 1998/11/05 22:56:31 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif.c,v 4.23 2000/01/15 22:24:21 william Exp $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include "const.h"
-#include "types.h"
+#define _INCLUDE_FROM_TGIF_C_
+
+#include "tgifdefs.h"
 
 #include "color.e"
+#include "dialog.e"
+#include "exec.e"
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
@@ -46,7 +46,9 @@
 #include "obj.e"
 #include "page.e"
 #include "setup.e"
+#include "strtbl.e"
 #include "util.e"
+#include "version.e"
 
 /*
  * extern int	malloc_debug ARGS_DECL((int));
@@ -131,7 +133,7 @@ static
 void SetProgramName(FileName)
    char *FileName;
 {
-   char *c_ptr=UtilStrRChr(FileName, '/');
+   char *c_ptr=UtilStrRChr(FileName, DIR_SEP);
 
    if (c_ptr != NULL && *(++c_ptr) != '\0') {
       strcpy(progName, c_ptr);
@@ -144,67 +146,97 @@ static
 int PrTgifLoad(FileName)
    char *FileName;
 {
-   struct ObjRec *obj_ptr;
-   char full_name[MAXPATHLENGTH+1], tmp_filename[MAXPATHLENGTH+1];
-   int len, read_status, obj_file=TRUE;
-   FILE *fp;
-   int tmp_linenum, obj_ext_len, sym_ext_len;
-   char obj_ext_str[MAXSTRING+1], sym_ext_str[MAXSTRING+1];
+   struct ObjRec *obj_ptr=NULL;
+   char full_name[MAXPATHLENGTH+1], gzipped_fname[MAXPATHLENGTH+1];
+   char tmp_filename[MAXPATHLENGTH+1], tmp_filefullpath[MAXPATHLENGTH+1];
+   int read_status, obj_file=TRUE;
+   FILE *fp=NULL;
+   int tmp_linenum=0, gzipped=FALSE;
 
-   sprintf(obj_ext_str, ".%s", OBJ_FILE_EXT);
-   obj_ext_len = strlen(obj_ext_str);
-   sprintf(sym_ext_str, ".%s", SYM_FILE_EXT);
-   sym_ext_len = strlen(sym_ext_str);
+   *gzipped_fname = '\0';
+   if (FileNameHasExtension(FileName, OBJ_FILE_TYPE, &gzipped, NULL)) {
+      if (gzipped) {
+         if (cmdLineOpenDisplay) {
+            char *tmp_fname=NULL;
 
-   len = strlen(FileName);
-
-   if (len>=obj_ext_len &&
-         strcmp(&FileName[len-obj_ext_len], obj_ext_str) == 0) {
+            if ((tmp_fname=GunzipFileIntoTemp(FileName)) == NULL) {
+               return FALSE;
+            } else {
+               strcpy(gzipped_fname, FileName);
+               strcpy(full_name, tmp_fname);
+               free(tmp_fname);
+            }
+         } else {
+            fprintf(stderr, TgLoadString(STID_TOOL_DONT_KNOW_HOW_RUN_GUNZIP),
+                  TOOL_NAME);
+            fprintf(stderr, "\n");
+            fprintf(stderr, "%s\n",
+                  TgLoadString(STID_SPECIFY_DISPLAY_ON_CMD_LINE));
+            return FALSE;
+         }
+      } else {
+         strcpy(full_name, FileName);
+      }
+   } else if (FileNameHasExtension(FileName, SYM_FILE_TYPE, NULL, NULL)) {
       strcpy(full_name, FileName);
-   } else if (len>=sym_ext_len &&
-         strcmp(&FileName[len-sym_ext_len],sym_ext_str)==0) {
+      obj_file = FALSE;
+   } else if (FileNameHasExtension(FileName, PIN_FILE_TYPE, NULL, NULL)) {
       strcpy(full_name, FileName);
       obj_file = FALSE;
    } else {
       sprintf(full_name, "%s.%s", FileName, OBJ_FILE_EXT);
    }
    if ((fp=fopen(full_name, "r")) == NULL) {
-      fprintf(stderr, "Can not open '%s'.\n", full_name);
+      fprintf(stderr, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_READING),
+            full_name);
+      fprintf(stderr, "\n");
+      if (*gzipped_fname != '\0') unlink(full_name);
       return FALSE;
    }
 
+   strcpy(tmp_filefullpath, scanFileFullPath);
    strcpy(tmp_filename, scanFileName);
    tmp_linenum = scanLineNum;
+   UtilStrCpyN(scanFileFullPath, sizeof(scanFileFullPath), full_name);
    strcpy(scanFileName, full_name);
    scanLineNum = 0;
 
    if (!(PRTGIF && cmdLineStdOut)) {
-      fprintf(stderr, "Reading '%s' ...\n", full_name);
+      fprintf(stderr, TgLoadCachedString(CSTID_READING_FILE_DOTS), full_name);
+      fprintf(stderr, "\n");
    }
-   while ((read_status = ReadObj(fp, &obj_ptr)) == TRUE) {
+   readingPageNum = loadedCurPageNum = 0;
+   foundGoodStateObject = FALSE;
+   while ((read_status=ReadObj(fp, &obj_ptr)) == TRUE) {
       if (obj_ptr != NULL) {
          AdjForOldVersion(obj_ptr);
          AddObj(NULL, topObj, obj_ptr);
       }
    }
 
+   strcpy(scanFileFullPath, tmp_filefullpath);
    strcpy(scanFileName, tmp_filename);
    scanLineNum = tmp_linenum;
 
    fclose(fp);
 
    if (read_status == INVALID) {
-      fprintf(stderr, "File version too large (=%1d).  Processing aborted!\n",
-            fileVersion);
+      fprintf(stderr, TgLoadString(STID_FILE_VER_ABORT_TOOL),
+            fileVersion, TOOL_NAME, TOOL_NAME, homePageURL);
+      if (*gzipped_fname != '\0') unlink(full_name);
       return FALSE;
    }
+   if (*gzipped_fname != '\0') {
+      unlink(full_name);
+      strcpy(full_name, gzipped_fname);
+   }
    if (cmdLineOpenDisplay) {
       char file_path[MAXPATHLENGTH+1];
 
-      if (*full_name == '/') {
+      if (*full_name == DIR_SEP) {
          strcpy(file_path, full_name);
       } else {
-         sprintf(file_path, "%s/%s", curDir, full_name);
+         sprintf(file_path, "%s%c%s", curDir, DIR_SEP, full_name);
       }
       curFileDefined = TRUE;
       if (obj_file) {
@@ -216,21 +248,74 @@ int PrTgifLoad(FileName)
    }
    if (cmdLineHasPageNum) {
       if (cmdLinePageNum > lastPageNum) {
-         fprintf(stderr, "%s is skipped because it only contains %1d pages!\n",
-               FileName, lastPageNum);
+         if (lastPageNum == 1) {
+            fprintf(stderr, TgLoadString(STID_FILE_SKIP_CONTAINS_ONE_PAGE),
+                  FileName);
+         } else {
+            fprintf(stderr, TgLoadString(STID_FILE_SKIP_CONTAINS_NUM_PAGE),
+                  FileName, lastPageNum);
+         }
+         fprintf(stderr, "\n");
          return FALSE;
       }
       GotoPageNum(cmdLinePageNum);
    }
    if (cmdLineColor && colorMenuItems == NULL) {
-      fprintf(stderr, "%s\n\t(%s).  Processing aborted!\n",
-            "File does not contain enough color information",
-            "not saved by tgif-3.0-p10 or beyond");
+      fprintf(stderr, TgLoadString(STID_FILE_NOT_CONTAIN_COLOR_ABORT),
+            TOOL_NAME);
+      fprintf(stderr, "\n");
       return FALSE;
    }
    return TRUE;
 }
 
+static
+void PrTgifDump(inbuf, tile_page_err_msg_id)
+   char *inbuf;
+   int tile_page_err_msg_id;
+{
+   UtilStrCpyN(cmdLineOpenFile, sizeof(cmdLineOpenFile), inbuf);
+   if (*cmdLineFileToExec != '\0') {
+      fprintf(stderr, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
+            cmdLineFileToExec);
+      fprintf(stderr, "\n");
+      StartExecCmdsFromFile();
+   } else {
+      colorDump = cmdLineColor;
+      if (cmdLineOpenDisplay) {
+         whereToPrint = cmdLineWhereToPrint;
+      }
+      if (cmdLineOneFilePerPage) {
+         if (pageLayoutMode == PAGE_TILE) {
+            fprintf(stderr, "%s\n", TgLoadString(tile_page_err_msg_id));
+         } else {
+            if (cmdLineHasPageNum) {
+               sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
+               GotoPageNum(cmdLinePageNum);
+               Dump(inbuf);
+            } else {
+               cmdLineHasPageNum = TRUE;
+               for (cmdLinePageNum=1; cmdLinePageNum <= lastPageNum;
+                     cmdLinePageNum++) {
+                  sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
+                  GotoPageNum(cmdLinePageNum);
+                  Dump(inbuf);
+               }
+               cmdLineHasPageNum = FALSE;
+            }
+         }
+      } else {
+         Dump(inbuf);
+      }
+   }
+   *cmdLineOpenFile = '\0';
+   DelAllObj();
+   if (cmdLineColor) CleanUpColors();
+   if (cmdLineOpenDisplay) {
+      CleanUp();
+   }
+}
+
 static
 void DoPrTgif(argc, argv, from_prtgif)
    int argc, from_prtgif;
@@ -239,6 +324,9 @@ void DoPrTgif(argc, argv, from_prtgif)
    char inbuf[MAXSTRING+1];
    int len, argc_to_be;
 
+   *scanFileFullPath = *scanFileName = '\0';
+   scanLineNum = 0;
+
    if ((argc_to_be=ProcessPrTgifOptions(argc, argv, from_prtgif)) == INVALID) {
       return;
    }
@@ -248,11 +336,12 @@ void DoPrTgif(argc, argv, from_prtgif)
    InitPaperSize();
    if (argc <= 0) {
       if (cmdLineStdOut) {
-         fprintf(stderr,
-               "\nFile name must be specified when -stdout is used.\n");
+         fprintf(stderr, "\n%s\n",
+               TgLoadString(STID_SPEC_FILE_WHEN_STDOUT_USED));
          return;
       }
-      fprintf(stderr, "\nTgif File Name to Print> ");
+      sprintf(gszMsgBox, TgLoadString(STID_TOOL_FILE_NAME_TO_PRINT), TOOL_NAME);
+      fprintf(stderr, "\n%s> ", gszMsgBox);
       fflush(stderr);
       while (fgets(inbuf, MAXSTRING, stdin) != NULL) {
          len = strlen(inbuf);
@@ -267,51 +356,21 @@ void DoPrTgif(argc, argv, from_prtgif)
                   JustInit(NULL, NULL);
                }
                if (PrTgifLoad(inbuf)) {
-                  colorDump = cmdLineColor;
-                  if (cmdLineOpenDisplay) {
-                     whereToPrint = cmdLineWhereToPrint;
-                  }
-                  if (cmdLineOneFilePerPage) {
-                     if (pageLayoutMode == PAGE_TILE) {
-                        fprintf(stderr, "%s %s.\n",
-                              "Cannot use -one_file_per_page in",
-                              "TILED page mode");
-                     } else {
-                        if (cmdLineHasPageNum) {
-                           sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
-                           GotoPageNum(cmdLinePageNum);
-                           Dump(*argv);
-                        } else {
-                           cmdLineHasPageNum = TRUE;
-                           for (cmdLinePageNum=1; cmdLinePageNum <= lastPageNum;
-                                 cmdLinePageNum++) {
-                              sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
-                              GotoPageNum(cmdLinePageNum);
-                              Dump(inbuf);
-                           }
-                           cmdLineHasPageNum = FALSE;
-                        }
-                     }
-                  } else {
-                     Dump(inbuf);
-                  }
-                  DelAllObj();
-                  if (cmdLineColor) CleanUpColors();
-                  if (cmdLineOpenDisplay) {
-                     CleanUp();
-                  }
+                  PrTgifDump(inbuf, STID_CANT_ONEFPPAGE_IN_TILED_PAGE);
                } else {
                   fprintf(stderr, "\n");
                }
             }
          }
-         fprintf(stderr, "\nTgif File Name to Print> ");
+         sprintf(gszMsgBox, TgLoadString(STID_TOOL_FILE_NAME_TO_PRINT),
+               TOOL_NAME);
+         fprintf(stderr, "\n%s> ", gszMsgBox);
          fflush(stderr);
       }
    } else {
       if (cmdLineStdOut && argc > 2) {
-         fprintf(stderr,
-               "\nOnly 1 file name caaan be specified with -stdout.\n");
+         fprintf(stderr, "\n%s\n",
+               TgLoadString(STID_ONE_FILE_ONLY_WITH_STDOUT));
          return;
       }
       for ( ; argc > 0; argc--, argv++) {
@@ -319,15 +378,15 @@ void DoPrTgif(argc, argv, from_prtgif)
          if (cmdLineDumpURL) {
             UrlDump(*argv);
             if (argc > 1) {
-               fprintf(stderr, "%s in %s mode.\n",
-                     "Cannot process more than one file", "-raw or -raw+h");
+               fprintf(stderr, "%s\n",
+                     TgLoadString(STID_ONE_FILE_ONLY_RAW_PLUS_H));
             }
             break;
          } else if (cmdLineDosEpsFilter) {
             DosEpsFilter(*argv);
             if (argc > 1) {
-               fprintf(stderr, "%s in %s mode.\n",
-                     "Cannot process more than one file", " -dosepsfilter");
+               fprintf(stderr, "%s\n",
+                     TgLoadString(STID_ONE_FILE_ONLY_DOSEPSFILTER));
             }
             break;
          } else {
@@ -335,39 +394,7 @@ void DoPrTgif(argc, argv, from_prtgif)
                JustInit(NULL, NULL);
             }
             if (PrTgifLoad(*argv)) {
-               colorDump = cmdLineColor;
-               if (cmdLineOpenDisplay) {
-                  whereToPrint = cmdLineWhereToPrint;
-               }
-               if (cmdLineOneFilePerPage) {
-                  if (pageLayoutMode == PAGE_TILE) {
-                     fprintf(stderr, "%s %s.\n",
-                           "Cannot use -one_file_per_page in",
-                           "TILED page mode");
-                  } else {
-                     if (cmdLineHasPageNum) {
-                        sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
-                        GotoPageNum(cmdLinePageNum);
-                        Dump(*argv);
-                     } else {
-                        cmdLineHasPageNum = TRUE;
-                        for (cmdLinePageNum=1; cmdLinePageNum <= lastPageNum;
-                              cmdLinePageNum++) {
-                           sprintf(cmdLinePageNumStr, "%1d", cmdLinePageNum);
-                           GotoPageNum(cmdLinePageNum);
-                           Dump(*argv);
-                        }
-                        cmdLineHasPageNum = FALSE;
-                     }
-                  }
-               } else {
-                  Dump(*argv);
-               }
-               DelAllObj();
-               if (cmdLineColor) CleanUpColors();
-               if (cmdLineOpenDisplay) {
-                  CleanUp();
-               }
+               PrTgifDump(*argv, STID_CANT_PRTGIF_OFPP_TILED_PAGE);
             } else {
                fprintf(stderr, "\n");
             }
@@ -411,19 +438,21 @@ void DoTgif(argc, argv)
 /*
  * register int i;
  */
-   char op_name[80], file_name[MAXPATHLENGTH+1];
+   char op_name[80], file_name[(MAXPATHLENGTH<<1)+1];
    char *sp[6], *func_strp;
 /*
  * char	color_name[80], val_name[80];
  * char	attr_name[80], speed_name[80], id_name[80];
  */
 
-   if (!ProcessTgifOptions(argc, argv, file_name)) return;
+   if (!ProcessTgifOptions(argc, argv, file_name, sizeof(file_name))) return;
 
    if (file_name[0] == '\0') {
+      /* do not translate -- program constants */
       MainLoop("init", "", &func_strp,
             &sp[0], &sp[1], &sp[2], &sp[3], &sp[4], &sp[5]);
    } else {
+      /* do not translate -- program constants */
       MainLoop("init", file_name, &func_strp,
             &sp[0], &sp[1], &sp[2], &sp[3], &sp[4], &sp[5]);
    }
@@ -444,6 +473,7 @@ void DoTgif(argc, argv)
 
       DeallocStrings(&func_strp,&sp[0],&sp[1],&sp[2],&sp[3],&sp[4],&sp[5]);
 
+      /* do not translate -- program constants */
       if (strcmp(s, "Quit") == 0) {
          *file_name = '\0';
          MainLoop("quit", file_name, &func_strp,
@@ -452,7 +482,7 @@ void DoTgif(argc, argv)
          break;
       }
 
-      Msg("Returned from basic driver.");
+      Msg(TgLoadCachedString(CSTID_RETURNED_FROM_BASIC_DRIVER));
 /*
  *    Prompt2("Input an operation and a sub command.\n",op_name,file_name);
  *
@@ -499,12 +529,13 @@ int main(argc, argv)
    from_prtgif = FALSE;
    SetProgramName(*argv);
    for (i = 1; i < argc; i++) {
-      if (strcmp(argv[i],"-prtgif")==0) {
+      /* do not translate -- program constants */
+      if (strcmp(argv[i], "-prtgif")==0) {
          PRTGIF = TRUE;
          from_prtgif = TRUE;
          break;
-      } else if ((strcmp(argv[i],"-print")==0) ||
-            (strcmp(argv[i],"-PRINT")==0)) {
+      } else if ((strcmp(argv[i], "-print")==0) ||
+            (strcmp(argv[i], "-PRINT")==0)) {
          PRTGIF = TRUE;
          break;
       }
