@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.10 2000/01/15 22:24:21 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.12 2000/05/20 22:17:37 william Exp $
  */
 #define _INCLUDE_FROM_TGIF_DBG_C_
 
@@ -389,6 +389,33 @@ Pixmap Tgif_XCreateBitmapFromData(display, d, data, width, height)
    return pixmap;
 }
 
+int Tgif_XReadBitmapFile(display, d, filename, width_return, height_return,
+      bitmap_return, x_hot_return, y_hot_return)
+   Display *display;
+   Drawable d;
+   _Xconst char *filename;
+   unsigned int *width_return;
+   unsigned int *height_return;
+   Pixmap *bitmap_return;
+   int *x_hot_return, *y_hot_return;
+{
+   int rc=XReadBitmapFile(display, d, filename, width_return, height_return,
+         bitmap_return, x_hot_return, y_hot_return);
+
+   if (gpTraceFile != NULL) {
+      if (++gnCreatePixmapCount == gnCreatePixmapTrigger) {
+         got_trigger(1);
+      }
+      fprintf(gpTraceFile, "p: %6d 0x%08x	%d\n",
+            gnCreatePixmapCount,
+            (bitmap_return==NULL || rc != BitmapSuccess) ? 0 :
+            (unsigned int)(*bitmap_return),
+            (width_return==NULL || height_return==NULL) ? 0 :
+            (int)((*width_return)*(*height_return)));
+   }
+   return rc;
+}
+
 void Tgif_XFreePixmap(display, pixmap)
    Display *display;
    Pixmap pixmap;
