@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.h,v 4.4 2000/01/15 22:25:20 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.h,v 4.5 2000/05/20 22:06:47 william Exp $
  */
 
 #ifndef _TGIF_DBG_H_
@@ -68,6 +68,8 @@ extern Pixmap Tgif_XCreatePixmap ARGS_DECL((Display*, Drawable, unsigned int,
        unsigned int, unsigned int));
 extern Pixmap Tgif_XCreateBitmapFromData ARGS_DECL((Display*, Drawable,
        _Xconst char*, unsigned int, unsigned int));
+extern int Tgif_XReadBitmapFile ARGS_DECL((Display*, Drawable,
+       _Xconst char*, unsigned int *, unsigned int *, Pixmap*, int*, int*));
 extern void Tgif_XFreePixmap ARGS_DECL((Display*, Pixmap));
 
 extern char *Tgif_XFetchBytes ARGS_DECL((Display*, int*));
@@ -116,6 +118,7 @@ extern void Tgif_XFree ARGS_DECL((void*));
 
 #define XCreatePixmap Tgif_XCreatePixmap
 #define XCreateBitmapFromData Tgif_XCreateBitmapFromData
+#define XReadBitmapFile Tgif_XReadBitmapFile
 #define XFreePixmap Tgif_XFreePixmap
 
 #define XFetchBytes Tgif_XFetchBytes
