@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.40 2000/01/15 22:25:21 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.43 2000/04/12 19:25:39 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -607,7 +607,7 @@ typedef int (CheckUpdateFunc)ARGS_DECL((DspList**, char***,
       char *cur_buf, int col, int row, void*));
 
 typedef struct PropertiesRec {
-   int			color, width, aw, ah;
+   int			color, width, aw, ah, width_index;
    char			color_str[40], width_spec[40], aw_spec[40], ah_spec[40];
    int			fill, pen, dash, arrow_style, curved, rcb_radius;
    int			trans_pat;
@@ -768,7 +768,9 @@ typedef struct tagEditAttrRec {
    char	**attr_names;
    char	**attr_values;
    char	**attr_strings;
+   char	**status_strings;
    int	*fore_colors;
+   int	*attr_indices;
 } EditAttrInfo;
 
 /* ------------------ scroll stuff ------------------ */
