@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/xprtfltr.c,v 4.13 2000/01/15 22:24:29 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/xprtfltr.c,v 4.14 2000/05/10 19:22:39 william Exp $
  */
 
 #define _INCLUDE_FROM_XPRTFLTR_C_
@@ -346,22 +346,22 @@ int InitExportFilters()
 /* do not translate -- program constants */
 static char *gpszColorExportName[MAXDEFWHERETOPRINT] = {
    "Printer", "EPS", "PS", "XPM", "ASCII", "EPSI", "GIF/ISMAP", "HTML/USEMAP",
-   "PDF", "TiffEPSI", "PNG"
+   "PDF", "TiffEPSI", "PNG", "JPEG"
 };
 
 static char *gpszBWExportName[MAXDEFWHERETOPRINT] = {
    "Printer", "EPS", "PS", "XBM", "ASCII", "EPSI", "GIF/ISMAP", "HTML/USEMAP",
-   "PDF", "TiffEPSI", "PNG"
+   "PDF", "TiffEPSI", "PNG", "JPEG"
 };
 
 static char *gpszColorExportExt[MAXDEFWHERETOPRINT] = {
    NULL, EPSF_FILE_EXT, PS_FILE_EXT, XPM_FILE_EXT, TEXT_FILE_EXT,
-   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png"
+   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg"
 };
 
 static char *gpszBWExportExt[MAXDEFWHERETOPRINT] = {
    NULL, EPSF_FILE_EXT, PS_FILE_EXT, XBM_FILE_EXT, TEXT_FILE_EXT,
-   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png"
+   EPSF_FILE_EXT, "gif", "html", "pdf", EPSF_FILE_EXT, "png", "jpg"
 };
 
 int GetExportIndex(nWhereToPrint, nMode)
